@@ -34,8 +34,7 @@
 #include <libsolutil/FixedHash.h>
 #include <libsolutil/LazyInit.h>
 #include <libsolutil/Visitor.h>
-
-#include <json/json.h>
+#include <libsolutil/JSON.h>
 
 #include <range/v3/view/subrange.hpp>
 #include <range/v3/view/map.hpp>
@@ -49,7 +48,7 @@
 namespace solidity::yul
 {
 // Forward-declaration to <yul/AST.h>
-struct Block;
+class AST;
 struct Dialect;
 }
 
@@ -87,15 +86,19 @@ public:
 	static void listAccept(std::vector<T> const& _list, ASTVisitor& _visitor)
 	{
 		for (T const& element: _list)
-			if (element)
-				element->accept(_visitor);
+		{
+			solAssert(element);
+			element->accept(_visitor);
+		}
 	}
 	template <class T>
 	static void listAccept(std::vector<T> const& _list, ASTConstVisitor& _visitor)
 	{
 		for (T const& element: _list)
-			if (element)
-				element->accept(_visitor);
+		{
+			solAssert(element);
+			element->accept(_visitor);
+		}
 	}
 
 	/// @returns a copy of the vector containing only the nodes which derive from T.
@@ -121,6 +124,8 @@ public:
 	bool operator!=(ASTNode const& _other) const { return !operator==(_other); }
 	///@}
 
+	virtual bool experimentalSolidityOnly() const { return false; }
+
 protected:
 	size_t const m_id = 0;
 
@@ -167,9 +172,14 @@ public:
 		int64_t _id,
 		SourceLocation const& _location,
 		std::optional<std::string> _licenseString,
-		std::vector<ASTPointer<ASTNode>> _nodes
+		std::vector<ASTPointer<ASTNode>> _nodes,
+		bool _experimentalSolidity
 	):
-		ASTNode(_id, _location), m_licenseString(std::move(_licenseString)), m_nodes(std::move(_nodes)) {}
+		ASTNode(_id, _location),
+		m_licenseString(std::move(_licenseString)),
+		m_nodes(std::move(_nodes)),
+		m_experimentalSolidity(_experimentalSolidity)
+	{}
 
 	void accept(ASTVisitor& _visitor) override;
 	void accept(ASTConstVisitor& _visitor) const override;
@@ -180,10 +190,12 @@ public:
 
 	/// @returns a set of referenced SourceUnits. Recursively if @a _recurse is true.
 	std::set<SourceUnit const*> referencedSourceUnits(bool _recurse = false, std::set<SourceUnit const*> _skipList = std::set<SourceUnit const*>()) const;
+	bool experimentalSolidity() const { return m_experimentalSolidity; }
 
 private:
 	std::optional<std::string> m_licenseString;
 	std::vector<ASTPointer<ASTNode>> m_nodes;
+	bool m_experimentalSolidity = false;
 };
 
 /**
@@ -522,6 +534,10 @@ public:
 	std::vector<EventDefinition const*> events() const { return filteredNodes<EventDefinition>(m_subNodes); }
 	std::vector<EventDefinition const*> const& definedInterfaceEvents() const;
 	std::vector<EventDefinition const*> const usedInterfaceEvents() const;
+	/// @return all events defined in this contract and its base contracts and all events
+	/// that are emitted during the execution of the contract.
+	/// @param _requireCallGraph if false, do not fail if the call graph has not been computed yet.
+	std::vector<EventDefinition const*> interfaceEvents(bool _requireCallGraph = true) const;
 	/// @returns all errors defined in this contract or any base contract
 	/// and all errors referenced during execution.
 	/// @param _requireCallGraph if false, do not fail if the call graph has not been computed yet.
@@ -709,7 +725,7 @@ private:
 	bool m_global = false;
 };
 
-class StructDefinition: public Declaration, public ScopeOpener
+class StructDefinition: public Declaration, public StructurallyDocumented, public ScopeOpener
 {
 public:
 	StructDefinition(
@@ -717,9 +733,13 @@ public:
 		SourceLocation const& _location,
 		ASTPointer<ASTString> const& _name,
 		SourceLocation _nameLocation,
-		std::vector<ASTPointer<VariableDeclaration>> _members
+		std::vector<ASTPointer<VariableDeclaration>> _members,
+		ASTPointer<StructuredDocumentation> _documentation
 	):
-		Declaration(_id, _location, _name, std::move(_nameLocation)), m_members(std::move(_members)) {}
+		Declaration(_id, _location, _name, std::move(_nameLocation)),
+		StructurallyDocumented(std::move(_documentation)),
+		m_members(std::move(_members))
+	{}
 
 	void accept(ASTVisitor& _visitor) override;
 	void accept(ASTConstVisitor& _visitor) const override;
@@ -737,7 +757,7 @@ private:
 	std::vector<ASTPointer<VariableDeclaration>> m_members;
 };
 
-class EnumDefinition: public Declaration, public ScopeOpener
+class EnumDefinition: public Declaration, public StructurallyDocumented, public ScopeOpener
 {
 public:
 	EnumDefinition(
@@ -745,9 +765,14 @@ public:
 		SourceLocation const& _location,
 		ASTPointer<ASTString> const& _name,
 		SourceLocation _nameLocation,
-		std::vector<ASTPointer<EnumValue>> _members
+		std::vector<ASTPointer<EnumValue>> _members,
+		ASTPointer<StructuredDocumentation> _documentation
 	):
-		Declaration(_id, _location, _name, std::move(_nameLocation)), m_members(std::move(_members)) {}
+		Declaration(_id, _location, _name, std::move(_nameLocation)),
+		StructurallyDocumented(std::move(_documentation)),
+		m_members(std::move(_members))
+	{}
+
 	void accept(ASTVisitor& _visitor) override;
 	void accept(ASTConstVisitor& _visitor) const override;
 
@@ -936,7 +961,8 @@ public:
 		ASTPointer<ParameterList> const& _parameters,
 		std::vector<ASTPointer<ModifierInvocation>> _modifiers,
 		ASTPointer<ParameterList> const& _returnParameters,
-		ASTPointer<Block> const& _body
+		ASTPointer<Block> const& _body,
+		ASTPointer<Expression> const& _experimentalReturnExpression = {}
 	):
 		CallableDeclaration(_id, _location, _name, _nameLocation, _visibility, _parameters, _isVirtual, _overrides, _returnParameters),
 		StructurallyDocumented(_documentation),
@@ -945,10 +971,12 @@ public:
 		m_free(_free),
 		m_kind(_kind),
 		m_functionModifiers(std::move(_modifiers)),
-		m_body(_body)
+		m_body(_body),
+		m_experimentalReturnExpression(_experimentalReturnExpression)
 	{
 		solAssert(_kind == Token::Constructor || _kind == Token::Function || _kind == Token::Fallback || _kind == Token::Receive, "");
 		solAssert(isOrdinary() == !name().empty(), "");
+		// TODO: assert _returnParameters implies non-experimental _experimentalReturnExpression implies experimental
 	}
 
 	void accept(ASTVisitor& _visitor) override;
@@ -1006,12 +1034,15 @@ public:
 		ContractDefinition const* _searchStart = nullptr
 	) const override;
 
+	Expression const* experimentalReturnExpression() const { return m_experimentalReturnExpression.get(); }
+
 private:
 	StateMutability m_stateMutability;
 	bool m_free;
 	Token const m_kind;
 	std::vector<ASTPointer<ModifierInvocation>> m_functionModifiers;
 	ASTPointer<Block> m_body;
+	ASTPointer<Expression> m_experimentalReturnExpression;
 };
 
 /**
@@ -1021,7 +1052,7 @@ private:
 class VariableDeclaration: public Declaration, public StructurallyDocumented
 {
 public:
-	enum Location { Unspecified, Storage, Memory, CallData };
+	enum Location { Unspecified, Storage, Transient, Memory, CallData };
 	enum class Mutability { Mutable, Immutable, Constant };
 	static std::string mutabilityToString(Mutability _mutability)
 	{
@@ -1046,7 +1077,8 @@ public:
 		bool _isIndexed = false,
 		Mutability _mutability = Mutability::Mutable,
 		ASTPointer<OverrideSpecifier> _overrides = nullptr,
-		Location _referenceLocation = Location::Unspecified
+		Location _referenceLocation = Location::Unspecified,
+		ASTPointer<Expression> _typeExpression = {}
 	):
 		Declaration(_id, _location, _name, std::move(_nameLocation), _visibility),
 		StructurallyDocumented(std::move(_documentation)),
@@ -1055,15 +1087,18 @@ public:
 		m_isIndexed(_isIndexed),
 		m_mutability(_mutability),
 		m_overrides(std::move(_overrides)),
-		m_location(_referenceLocation)
+		m_location(_referenceLocation),
+		m_typeExpression(std::move(_typeExpression))
 	{
-		solAssert(m_typeName, "");
+		// TODO: consider still asserting unless we are in experimental solidity.
+		// solAssert(m_typeName, ""); solAssert(!m_typeExpression, "");
 	}
 
 
 	void accept(ASTVisitor& _visitor) override;
 	void accept(ASTConstVisitor& _visitor) const override;
 
+	bool hasTypeName() const { return m_typeName != nullptr; }
 	TypeName const& typeName() const { return *m_typeName; }
 	ASTPointer<Expression> const& value() const { return m_value; }
 
@@ -1118,6 +1153,7 @@ public:
 	/// @returns null when it is not accessible as a function.
 	FunctionTypePointer functionType(bool /*_internal*/) const override;
 
+	ASTPointer<Expression> const& typeExpression() const { return m_typeExpression; }
 	VariableDeclarationAnnotation& annotation() const override;
 
 protected:
@@ -1133,6 +1169,7 @@ private:
 	Mutability m_mutability = Mutability::Mutable;
 	ASTPointer<OverrideSpecifier> m_overrides; ///< Contains the override specifier node
 	Location m_location = Location::Unspecified; ///< Location of the variable if it is of reference type.
+	ASTPointer<Expression> m_typeExpression;
 };
 
 /**
@@ -1241,7 +1278,7 @@ public:
 	FunctionTypePointer functionType(bool /*_internal*/) const override;
 
 	bool isVisibleInDerivedContracts() const override { return true; }
-	bool isVisibleViaContractTypeAccess() const override { return false; /* TODO */ }
+	bool isVisibleViaContractTypeAccess() const override { return true; }
 
 	EventDefinitionAnnotation& annotation() const override;
 
@@ -1532,7 +1569,7 @@ public:
 		ASTPointer<ASTString> const& _docString,
 		yul::Dialect const& _dialect,
 		ASTPointer<std::vector<ASTPointer<ASTString>>> _flags,
-		std::shared_ptr<yul::Block> _operations
+		std::shared_ptr<yul::AST> _operations
 	):
 		Statement(_id, _location, _docString),
 		m_dialect(_dialect),
@@ -1543,7 +1580,7 @@ public:
 	void accept(ASTConstVisitor& _visitor) const override;
 
 	yul::Dialect const& dialect() const { return m_dialect; }
-	yul::Block const& operations() const { return *m_operations; }
+	yul::AST const& operations() const { return *m_operations; }
 	ASTPointer<std::vector<ASTPointer<ASTString>>> const& flags() const { return m_flags; }
 
 	InlineAssemblyAnnotation& annotation() const override;
@@ -1551,7 +1588,7 @@ public:
 private:
 	yul::Dialect const& m_dialect;
 	ASTPointer<std::vector<ASTPointer<ASTString>>> m_flags;
-	std::shared_ptr<yul::Block> m_operations;
+	std::shared_ptr<yul::AST> m_operations;
 };
 
 /**
@@ -2114,7 +2151,8 @@ public:
 	):
 		Expression(_id, _location), m_left(std::move(_left)), m_operator(_operator), m_right(std::move(_right))
 	{
-		solAssert(TokenTraits::isBinaryOp(_operator) || TokenTraits::isCompareOp(_operator), "");
+		// TODO: assert against colon for non-experimental solidity
+		solAssert(TokenTraits::isBinaryOp(_operator) || TokenTraits::isCompareOp(_operator) || _operator == Token::Colon || _operator == Token::RightArrow, "");
 	}
 	void accept(ASTVisitor& _visitor) override;
 	void accept(ASTConstVisitor& _visitor) const override;
@@ -2425,4 +2463,199 @@ private:
 
 /// @}
 
+/// Experimental Solidity nodes
+/// @{
+class TypeClassDefinition: public Declaration, public StructurallyDocumented, public ScopeOpener
+{
+public:
+	TypeClassDefinition(
+		int64_t _id,
+		SourceLocation const& _location,
+		ASTPointer<VariableDeclaration> _typeVariable,
+		ASTPointer<ASTString> const& _name,
+		SourceLocation _nameLocation,
+		ASTPointer<StructuredDocumentation> const& _documentation,
+		std::vector<ASTPointer<ASTNode>> _subNodes
+	):
+		Declaration(_id, _location, _name, std::move(_nameLocation)),
+		StructurallyDocumented(_documentation),
+		m_typeVariable(std::move(_typeVariable)),
+		m_subNodes(std::move(_subNodes))
+	{}
+
+	void accept(ASTVisitor& _visitor) override;
+	void accept(ASTConstVisitor& _visitor) const override;
+
+	VariableDeclaration const& typeVariable() const { return *m_typeVariable; }
+	std::vector<ASTPointer<ASTNode>> const& subNodes() const { return m_subNodes; }
+
+	TypeClassDefinitionAnnotation& annotation() const override;
+
+	Type const* type() const override { solAssert(false, "Requested type of experimental solidity node."); }
+
+	bool experimentalSolidityOnly() const override { return true; }
+
+private:
+	ASTPointer<VariableDeclaration> m_typeVariable;
+	std::vector<ASTPointer<ASTNode>> m_subNodes;
+};
+
+class TypeClassInstantiation: public ASTNode, public ScopeOpener
+{
+public:
+	TypeClassInstantiation(
+		int64_t _id,
+		SourceLocation const& _location,
+		ASTPointer<TypeName> _typeConstructor,
+		ASTPointer<ParameterList> _argumentSorts,
+		ASTPointer<TypeClassName> _class,
+		std::vector<ASTPointer<ASTNode>> _subNodes
+	):
+		ASTNode(_id, _location),
+		m_typeConstructor(std::move(_typeConstructor)),
+		m_argumentSorts(std::move(_argumentSorts)),
+		m_class(std::move(_class)),
+		m_subNodes(std::move(_subNodes))
+	{}
+
+	void accept(ASTVisitor& _visitor) override;
+	void accept(ASTConstVisitor& _visitor) const override;
+
+	TypeName const& typeConstructor() const { return *m_typeConstructor; }
+	ParameterList const* argumentSorts() const { return m_argumentSorts.get(); }
+	TypeClassName const& typeClass() const { return *m_class; }
+	std::vector<ASTPointer<ASTNode>> const& subNodes() const { return m_subNodes; }
+
+	bool experimentalSolidityOnly() const override { return true; }
+
+private:
+	ASTPointer<TypeName> m_typeConstructor;
+	ASTPointer<ParameterList> m_argumentSorts;
+	ASTPointer<TypeClassName> m_class;
+	std::vector<ASTPointer<ASTNode>> m_subNodes;
+};
+
+class TypeDefinition: public Declaration, public ScopeOpener
+{
+public:
+	TypeDefinition(
+		int64_t _id,
+		SourceLocation const& _location,
+		ASTPointer<ASTString> _name,
+		SourceLocation _nameLocation,
+		ASTPointer<ParameterList> _arguments,
+		ASTPointer<Expression> _typeExpression
+	):
+		Declaration(_id, _location, _name, std::move(_nameLocation), Visibility::Default),
+		m_arguments(std::move(_arguments)),
+		m_typeExpression(std::move(_typeExpression))
+	{
+	}
+
+	void accept(ASTVisitor& _visitor) override;
+	void accept(ASTConstVisitor& _visitor) const override;
+
+	Type const* type() const override { return nullptr; }
+
+	TypeDeclarationAnnotation& annotation() const override;
+
+	ParameterList const* arguments() const { return m_arguments.get(); }
+	Expression const* typeExpression() const { return m_typeExpression.get(); }
+
+	bool experimentalSolidityOnly() const override { return true; }
+
+private:
+	ASTPointer<ParameterList> m_arguments;
+	ASTPointer<Expression> m_typeExpression;
+};
+
+class TypeClassName: public ASTNode
+{
+public:
+	TypeClassName(
+		int64_t _id,
+		SourceLocation const& _location,
+		std::variant<Token, ASTPointer<IdentifierPath>> _name
+	):
+		ASTNode(_id, _location),
+		m_name(std::move(_name))
+	{
+		if (Token const* token = std::get_if<Token>(&_name))
+			solAssert(TokenTraits::isBuiltinTypeClassName(*token));
+	}
+
+	void accept(ASTVisitor& _visitor) override;
+	void accept(ASTConstVisitor& _visitor) const override;
+
+	bool experimentalSolidityOnly() const override { return true; }
+
+	std::variant<Token, ASTPointer<IdentifierPath>> name() const { return m_name; }
+
+private:
+	std::variant<Token, ASTPointer<IdentifierPath>> m_name;
+};
+
+class Builtin: public Expression
+{
+public:
+	Builtin(
+		int64_t _id,
+		SourceLocation _location,
+		ASTPointer<ASTString> _nameParameter,
+		SourceLocation _nameParameterLocation
+	):
+		Expression(_id, std::move(_location)),
+		m_nameParameter(std::move(_nameParameter)),
+		m_nameParameterLocation(std::move(_nameParameterLocation))
+	{
+		solAssert(m_nameParameter);
+	}
+
+	void accept(ASTVisitor& _visitor) override;
+	void accept(ASTConstVisitor& _visitor) const override;
+
+	bool experimentalSolidityOnly() const override { return true; }
+
+	ASTString const& nameParameter() const { return *m_nameParameter; }
+	SourceLocation const& nameParameterLocation() const { return m_nameParameterLocation; }
+
+private:
+	ASTPointer<ASTString> m_nameParameter;
+	SourceLocation m_nameParameterLocation;
+};
+
+// TODO: NatSpec used on the quantifier should be recognized as applying to the function.
+class ForAllQuantifier: public ASTNode, public Scopable, public ScopeOpener
+{
+public:
+	ForAllQuantifier(
+		int64_t _id,
+		SourceLocation _location,
+		ASTPointer<ParameterList> _typeVariableDeclarations,
+		ASTPointer<FunctionDefinition> _quantifiedDeclaration
+	):
+		ASTNode(_id, std::move(_location)),
+		m_typeVariableDeclarations(std::move(_typeVariableDeclarations)),
+		m_quantifiedDeclaration(std::move(_quantifiedDeclaration))
+	{
+		solAssert(m_typeVariableDeclarations);
+		solAssert(m_quantifiedDeclaration);
+	}
+
+	void accept(ASTVisitor& _visitor) override;
+	void accept(ASTConstVisitor& _visitor) const override;
+	ForAllQuantifierAnnotation& annotation() const override { return initAnnotation<ForAllQuantifierAnnotation>(); }
+
+	bool experimentalSolidityOnly() const override { return true; }
+
+	ParameterList const& typeVariableDeclarations() const { return *m_typeVariableDeclarations; }
+	FunctionDefinition const& quantifiedDeclaration() const { return *m_quantifiedDeclaration; }
+
+private:
+	ASTPointer<ParameterList> m_typeVariableDeclarations;
+	ASTPointer<FunctionDefinition> m_quantifiedDeclaration;
+};
+
+/// @}
+
 }
