@@ -26,42 +26,41 @@
 
 #include <range/v3/view.hpp>
 
-using namespace std;
 using namespace solidity;
 using namespace solidity::util;
 using namespace solidity::smtutil;
 using namespace solidity::frontend::smt;
 
 BlockchainVariable::BlockchainVariable(
-	string _name,
-	map<string, smtutil::SortPointer> _members,
+	std::string _name,
+	std::map<std::string, smtutil::SortPointer> _members,
 	EncodingContext& _context
 ):
 	m_name(std::move(_name)),
 	m_members(std::move(_members)),
 	m_context(_context)
 {
-	vector<string> members;
-	vector<SortPointer> sorts;
+	std::vector<std::string> members;
+	std::vector<SortPointer> sorts;
 	for (auto const& [component, sort]: m_members)
 	{
 		members.emplace_back(component);
 		sorts.emplace_back(sort);
 		m_componentIndices[component] = static_cast<unsigned>(members.size() - 1);
 	}
-	m_tuple = make_unique<SymbolicTupleVariable>(
-		make_shared<smtutil::TupleSort>(m_name + "_type", members, sorts),
+	m_tuple = std::make_unique<SymbolicTupleVariable>(
+		std::make_shared<smtutil::TupleSort>(m_name + "_type", members, sorts),
 		m_name,
 		m_context
 	);
 }
 
-smtutil::Expression BlockchainVariable::member(string const& _member) const
+smtutil::Expression BlockchainVariable::member(std::string const& _member) const
 {
 	return m_tuple->component(m_componentIndices.at(_member));
 }
 
-smtutil::Expression BlockchainVariable::assignMember(string const& _member, smtutil::Expression const& _value)
+smtutil::Expression BlockchainVariable::assignMember(std::string const& _member, smtutil::Expression const& _value)
 {
 	smtutil::Expression newTuple = smt::assignMember(m_tuple->currentValue(), {{_member, _value}});
 	m_context.addAssertion(m_tuple->increaseIndex() == newTuple);
@@ -104,9 +103,9 @@ smtutil::Expression SymbolicState::blockhash(smtutil::Expression _blockNumber) c
 
 void SymbolicState::newBalances()
 {
-	auto tupleSort = dynamic_pointer_cast<TupleSort>(stateSort());
+	auto tupleSort = std::dynamic_pointer_cast<TupleSort>(stateSort());
 	auto balanceSort = tupleSort->components.at(tupleSort->memberToIndex.at("balances"));
-	SymbolicVariable newBalances(balanceSort, "fresh_balances_" + to_string(m_context.newUniqueId()), m_context);
+	SymbolicVariable newBalances(balanceSort, "fresh_balances_" + std::to_string(m_context.newUniqueId()), m_context);
 	m_state->assignMember("balances", newBalances.currentValue());
 }
 
@@ -158,7 +157,7 @@ void SymbolicState::newStorage()
 {
 	auto newStorageVar = SymbolicTupleVariable(
 		m_state->member("storage").sort,
-		"havoc_storage_" + to_string(m_context.newUniqueId()),
+		"havoc_storage_" + std::to_string(m_context.newUniqueId()),
 		m_context
 	);
 	m_state->assignMember("storage", newStorageVar.currentValue());
@@ -170,7 +169,7 @@ void SymbolicState::writeStateVars(ContractDefinition const& _contract, smtutil:
 	if (stateVars.empty())
 		return;
 
-	map<string, smtutil::Expression> values;
+	std::map<std::string, smtutil::Expression> values;
 	for (auto var: stateVars)
 		values.emplace(stateVarStorageKey(*var, _contract), m_context.variable(*var)->currentValue());
 
@@ -207,7 +206,7 @@ void SymbolicState::addBalance(smtutil::Expression _address, smtutil::Expression
 	m_state->assignMember("balances", newBalances);
 }
 
-smtutil::Expression SymbolicState::txMember(string const& _member) const
+smtutil::Expression SymbolicState::txMember(std::string const& _member) const
 {
 	return m_tx.member(_member);
 }
@@ -268,49 +267,52 @@ void SymbolicState::prepareForSourceUnit(SourceUnit const& _source, bool _storag
 {
 	auto allSources = _source.referencedSourceUnits(true);
 	allSources.insert(&_source);
-	set<FunctionCall const*, ASTCompareByID<FunctionCall>> abiCalls;
-	set<ContractDefinition const*, ASTCompareByID<ContractDefinition>> contracts;
+	std::set<FunctionCall const*, ASTCompareByID<FunctionCall>> abiCalls;
+	std::set<FunctionCall const*, ASTCompareByID<FunctionCall>> bytesConcatCalls;
+	std::set<ContractDefinition const*, ASTCompareByID<ContractDefinition>> contracts;
 	for (auto const& source: allSources)
 	{
 		abiCalls += SMTEncoder::collectABICalls(source);
+		bytesConcatCalls += SMTEncoder::collectBytesConcatCalls(source);
 		for (auto node: source->nodes())
 			if (auto contract = dynamic_cast<ContractDefinition const*>(node.get()))
 				contracts.insert(contract);
 	}
 	buildState(contracts, _storage);
 	buildABIFunctions(abiCalls);
+	buildBytesConcatFunctions(bytesConcatCalls);
 }
 
 /// Private helpers.
 
-string SymbolicState::contractSuffix(ContractDefinition const& _contract) const
+std::string SymbolicState::contractSuffix(ContractDefinition const& _contract) const
 {
-	return "_" + _contract.name() + "_" + to_string(_contract.id());
+	return "_" + _contract.name() + "_" + std::to_string(_contract.id());
 }
 
-string SymbolicState::contractStorageKey(ContractDefinition const& _contract) const
+std::string SymbolicState::contractStorageKey(ContractDefinition const& _contract) const
 {
 	return "storage" + contractSuffix(_contract);
 }
 
-string SymbolicState::stateVarStorageKey(VariableDeclaration const& _var, ContractDefinition const& _contract) const
+std::string SymbolicState::stateVarStorageKey(VariableDeclaration const& _var, ContractDefinition const& _contract) const
 {
-	return _var.name() + "_" + to_string(_var.id()) + contractSuffix(_contract);
+	return _var.name() + "_" + std::to_string(_var.id()) + contractSuffix(_contract);
 }
 
-void SymbolicState::buildState(set<ContractDefinition const*, ASTCompareByID<ContractDefinition>> const& _contracts, bool _allStorages)
+void SymbolicState::buildState(std::set<ContractDefinition const*, ASTCompareByID<ContractDefinition>> const& _contracts, bool _allStorages)
 {
-	map<string, SortPointer> stateMembers{
-		{"balances", make_shared<smtutil::ArraySort>(smtutil::SortProvider::uintSort, smtutil::SortProvider::uintSort)}
+	std::map<std::string, SortPointer> stateMembers{
+		{"balances", std::make_shared<smtutil::ArraySort>(smtutil::SortProvider::uintSort, smtutil::SortProvider::uintSort)}
 	};
 
 	if (_allStorages)
 	{
-		vector<string> memberNames;
-		vector<SortPointer> memberSorts;
+		std::vector<std::string> memberNames;
+		std::vector<SortPointer> memberSorts;
 		for (auto contract: _contracts)
 		{
-			string suffix = contractSuffix(*contract);
+			std::string suffix = contractSuffix(*contract);
 
 			// z3 doesn't like empty tuples, so if the contract has 0
 			// state vars we can't put it there.
@@ -319,16 +321,16 @@ void SymbolicState::buildState(set<ContractDefinition const*, ASTCompareByID<Con
 				continue;
 
 			auto names = applyMap(stateVars, [&](auto var) {
-				return var->name() + "_" + to_string(var->id()) + suffix;
+				return var->name() + "_" + std::to_string(var->id()) + suffix;
 			});
 			auto sorts = applyMap(stateVars, [](auto var) { return smtSortAbstractFunction(*var->type()); });
 
-			string name = "storage" + suffix;
-			auto storageTuple = make_shared<smtutil::TupleSort>(
+			std::string name = "storage" + suffix;
+			auto storageTuple = std::make_shared<smtutil::TupleSort>(
 				name + "_type", names, sorts
 			);
 
-			auto storageSort = make_shared<smtutil::ArraySort>(
+			auto storageSort = std::make_shared<smtutil::ArraySort>(
 				smtSort(*TypeProvider::address()),
 				storageTuple
 			);
@@ -339,26 +341,100 @@ void SymbolicState::buildState(set<ContractDefinition const*, ASTCompareByID<Con
 
 		stateMembers.emplace(
 			"isActive",
-			make_shared<smtutil::ArraySort>(smtSort(*TypeProvider::address()), smtutil::SortProvider::boolSort)
+			std::make_shared<smtutil::ArraySort>(smtSort(*TypeProvider::address()), smtutil::SortProvider::boolSort)
 		);
 		stateMembers.emplace(
 			"storage",
-			make_shared<smtutil::TupleSort>(
+			std::make_shared<smtutil::TupleSort>(
 				"storage_type", memberNames, memberSorts
 			)
 		);
 	}
 
-	m_state = make_unique<BlockchainVariable>(
+	m_state = std::make_unique<BlockchainVariable>(
 		"state",
 		std::move(stateMembers),
 		m_context
 	);
 }
 
-void SymbolicState::buildABIFunctions(set<FunctionCall const*, ASTCompareByID<FunctionCall>> const& _abiFunctions)
+void SymbolicState::buildBytesConcatFunctions(std::set<FunctionCall const*, ASTCompareByID<FunctionCall>> const& _bytesConcatCalls)
 {
-	map<string, SortPointer> functions;
+	std::map<std::string, SortPointer> functions;
+
+	for (auto const* funCall: _bytesConcatCalls)
+	{
+		auto t = dynamic_cast<FunctionType const*>(funCall->expression().annotation().type);
+		solAssert(t->kind() == FunctionType::Kind::BytesConcat, "Expected bytes.concat function");
+
+		auto const& args = funCall->sortedArguments();
+
+		auto argTypes = [](auto const& _args) {
+			return util::applyMap(_args, [](auto arg) { return arg->annotation().type; });
+		};
+
+		// bytes.concat : (bytes/literal/fixed bytes, ... ) -> bytes
+		std::vector<frontend::Type const*> inTypes = argTypes(args);
+
+		auto replaceUserDefinedValueTypes = [](auto& _types) {
+			for (auto& t: _types)
+				if (auto userType = dynamic_cast<UserDefinedValueType const*>(t))
+					t = &userType->underlyingType();
+		};
+		auto replaceStringLiteralTypes = [](auto& _types) {
+			for (auto& t: _types)
+				if (t->category() == frontend::Type::Category::StringLiteral)
+					t = TypeProvider::bytesMemory();
+		};
+		replaceUserDefinedValueTypes(inTypes);
+		replaceStringLiteralTypes(inTypes);
+
+		auto name = t->richIdentifier();
+		for (auto paramType: inTypes)
+			name += "_" + paramType->richIdentifier();
+
+		frontend::Type const* outType = TypeProvider::bytesMemory();
+		name += "_" + outType->richIdentifier();
+
+		m_bytesConcatMembers[funCall] = {name, inTypes, outType};
+
+		if (functions.count(name))
+			continue;
+
+		/// If there is only one parameter, we use that type directly.
+		/// Otherwise we create a tuple wrapping the necessary types.
+		auto typesToSort = [](auto const& _types, std::string const& _name) -> std::shared_ptr<Sort> {
+			if (_types.size() == 1)
+				return smtSortAbstractFunction(*_types.front());
+
+			std::vector<std::string> inNames;
+			std::vector<SortPointer> sorts;
+			for (unsigned i = 0; i < _types.size(); ++i)
+			{
+				inNames.emplace_back(_name + "_input_" + std::to_string(i));
+				sorts.emplace_back(smtSortAbstractFunction(*_types.at(i)));
+			}
+			return std::make_shared<smtutil::TupleSort>(
+				_name + "_input",
+				inNames,
+				sorts
+			);
+		};
+
+		auto functionSort = std::make_shared<smtutil::ArraySort>(
+			typesToSort(inTypes, name),
+			smtSortAbstractFunction(*outType)
+		);
+
+		functions[name] = functionSort;
+	}
+
+	m_bytesConcat = std::make_unique<BlockchainVariable>("bytesConcat", std::move(functions), m_context);
+}
+
+void SymbolicState::buildABIFunctions(std::set<FunctionCall const*, ASTCompareByID<FunctionCall>> const& _abiFunctions)
+{
+	std::map<std::string, SortPointer> functions;
 
 	for (auto const* funCall: _abiFunctions)
 	{
@@ -368,15 +444,14 @@ void SymbolicState::buildABIFunctions(set<FunctionCall const*, ASTCompareByID<Fu
 		auto const& paramTypes = t->parameterTypes();
 		auto const& returnTypes = t->returnParameterTypes();
 
-
 		auto argTypes = [](auto const& _args) {
 			return util::applyMap(_args, [](auto arg) { return arg->annotation().type; });
 		};
 
 		/// Since each abi.* function may have a different number of input/output parameters,
 		/// we generically compute those types.
-		vector<frontend::Type const*> inTypes;
-		vector<frontend::Type const*> outTypes;
+		std::vector<frontend::Type const*> inTypes;
+		std::vector<frontend::Type const*> outTypes;
 		if (t->kind() == FunctionType::Kind::ABIDecode)
 		{
 			/// abi.decode : (bytes, tuple_of_types(return_types)) -> (return_types)
@@ -415,7 +490,7 @@ void SymbolicState::buildABIFunctions(set<FunctionCall const*, ASTCompareByID<Fu
 				/// abi.encodeWithSelector : (bytes4, one_or_more_types) -> bytes
 				/// abi.encodeWithSignature : (string, one_or_more_types) -> bytes
 				inTypes.emplace_back(paramTypes.front());
-				inTypes += argTypes(vector<ASTPointer<Expression const>>(args.begin() + 1, args.end()));
+				inTypes += argTypes(std::vector<ASTPointer<Expression const>>(args.begin() + 1, args.end()));
 			}
 			else
 			{
@@ -455,25 +530,25 @@ void SymbolicState::buildABIFunctions(set<FunctionCall const*, ASTCompareByID<Fu
 
 		/// If there is only one input or output parameter, we use that type directly.
 		/// Otherwise we create a tuple wrapping the necessary input or output types.
-		auto typesToSort = [](auto const& _types, string const& _name) -> shared_ptr<Sort> {
+		auto typesToSort = [](auto const& _types, std::string const& _name) -> std::shared_ptr<Sort> {
 			if (_types.size() == 1)
 				return smtSortAbstractFunction(*_types.front());
 
-			vector<string> inNames;
-			vector<SortPointer> sorts;
+			std::vector<std::string> inNames;
+			std::vector<SortPointer> sorts;
 			for (unsigned i = 0; i < _types.size(); ++i)
 			{
-				inNames.emplace_back(_name + "_input_" + to_string(i));
+				inNames.emplace_back(_name + "_input_" + std::to_string(i));
 				sorts.emplace_back(smtSortAbstractFunction(*_types.at(i)));
 			}
-			return make_shared<smtutil::TupleSort>(
+			return std::make_shared<smtutil::TupleSort>(
 				_name + "_input",
 				inNames,
 				sorts
 			);
 		};
 
-		auto functionSort = make_shared<smtutil::ArraySort>(
+		auto functionSort = std::make_shared<smtutil::ArraySort>(
 			typesToSort(inTypes, name),
 			typesToSort(outTypes, name)
 		);
@@ -481,16 +556,27 @@ void SymbolicState::buildABIFunctions(set<FunctionCall const*, ASTCompareByID<Fu
 		functions[name] = functionSort;
 	}
 
-	m_abi = make_unique<BlockchainVariable>("abi", std::move(functions), m_context);
+	m_abi = std::make_unique<BlockchainVariable>("abi", std::move(functions), m_context);
 }
 
 smtutil::Expression SymbolicState::abiFunction(frontend::FunctionCall const* _funCall)
 {
 	solAssert(m_abi, "");
-	return m_abi->member(get<0>(m_abiMembers.at(_funCall)));
+	return m_abi->member(std::get<0>(m_abiMembers.at(_funCall)));
 }
 
 SymbolicState::SymbolicABIFunction const& SymbolicState::abiFunctionTypes(FunctionCall const* _funCall) const
 {
 	return m_abiMembers.at(_funCall);
 }
+
+smtutil::Expression SymbolicState::bytesConcatFunction(frontend::FunctionCall const* _funCall)
+{
+	solAssert(m_bytesConcat, "");
+	return m_bytesConcat->member(std::get<0>(m_bytesConcatMembers.at(_funCall)));
+}
+
+SymbolicState::SymbolicBytesConcatFunction const& SymbolicState::bytesConcatFunctionTypes(FunctionCall const* _funCall) const
+{
+	return m_bytesConcatMembers.at(_funCall);
+}
