@@ -25,7 +25,7 @@ class GotoDefinition: public HandlerBase
 public:
 	explicit GotoDefinition(LanguageServer& _server): HandlerBase(_server) {}
 
-	void operator()(MessageID, Json::Value const&);
+	void operator()(MessageID, Json const&);
 };
 
 }
