@@ -46,13 +46,13 @@
 
 #include <fmt/format.h>
 
-using namespace std;
 using namespace std::string_literals;
 using namespace std::placeholders;
 
 using namespace solidity::lsp;
 using namespace solidity::langutil;
 using namespace solidity::frontend;
+using namespace solidity;
 
 namespace fs = boost::filesystem;
 
@@ -86,44 +86,44 @@ int toDiagnosticSeverity(Error::Type _errorType)
 	return -1;
 }
 
-Json::Value semanticTokensLegend()
+Json semanticTokensLegend()
 {
-	Json::Value legend = Json::objectValue;
+	Json legend;
 
 	// NOTE! The (alphabetical) order and items must match exactly the items of
 	//       their respective enum class members.
 
-	Json::Value tokenTypes = Json::arrayValue;
-	tokenTypes.append("class");
-	tokenTypes.append("comment");
-	tokenTypes.append("enum");
-	tokenTypes.append("enumMember");
-	tokenTypes.append("event");
-	tokenTypes.append("function");
-	tokenTypes.append("interface");
-	tokenTypes.append("keyword");
-	tokenTypes.append("macro");
-	tokenTypes.append("method");
-	tokenTypes.append("modifier");
-	tokenTypes.append("number");
-	tokenTypes.append("operator");
-	tokenTypes.append("parameter");
-	tokenTypes.append("property");
-	tokenTypes.append("string");
-	tokenTypes.append("struct");
-	tokenTypes.append("type");
-	tokenTypes.append("typeParameter");
-	tokenTypes.append("variable");
+	Json tokenTypes = Json::array();
+	tokenTypes.emplace_back("class");
+	tokenTypes.emplace_back("comment");
+	tokenTypes.emplace_back("enum");
+	tokenTypes.emplace_back("enumMember");
+	tokenTypes.emplace_back("event");
+	tokenTypes.emplace_back("function");
+	tokenTypes.emplace_back("interface");
+	tokenTypes.emplace_back("keyword");
+	tokenTypes.emplace_back("macro");
+	tokenTypes.emplace_back("method");
+	tokenTypes.emplace_back("modifier");
+	tokenTypes.emplace_back("number");
+	tokenTypes.emplace_back("operator");
+	tokenTypes.emplace_back("parameter");
+	tokenTypes.emplace_back("property");
+	tokenTypes.emplace_back("std::string");
+	tokenTypes.emplace_back("struct");
+	tokenTypes.emplace_back("type");
+	tokenTypes.emplace_back("typeParameter");
+	tokenTypes.emplace_back("variable");
 	legend["tokenTypes"] = tokenTypes;
 
-	Json::Value tokenModifiers = Json::arrayValue;
-	tokenModifiers.append("abstract");
-	tokenModifiers.append("declaration");
-	tokenModifiers.append("definition");
-	tokenModifiers.append("deprecated");
-	tokenModifiers.append("documentation");
-	tokenModifiers.append("modification");
-	tokenModifiers.append("readonly");
+	Json tokenModifiers = Json::array();
+	tokenModifiers.emplace_back("abstract");
+	tokenModifiers.emplace_back("declaration");
+	tokenModifiers.emplace_back("definition");
+	tokenModifiers.emplace_back("deprecated");
+	tokenModifiers.emplace_back("documentation");
+	tokenModifiers.emplace_back("modification");
+	tokenModifiers.emplace_back("readonly");
 	legend["tokenModifiers"] = tokenModifiers;
 
 	return legend;
@@ -137,36 +137,36 @@ LanguageServer::LanguageServer(Transport& _transport):
 		{"$/cancelRequest", [](auto, auto) {/*nothing for now as we are synchronous */}},
 		{"cancelRequest", [](auto, auto) {/*nothing for now as we are synchronous */}},
 		{"exit", [this](auto, auto) { m_state = (m_state == State::ShutdownRequested ? State::ExitRequested : State::ExitWithoutShutdown); }},
-		{"initialize", bind(&LanguageServer::handleInitialize, this, _1, _2)},
-		{"initialized", bind(&LanguageServer::handleInitialized, this, _1, _2)},
-		{"$/setTrace", [this](auto, Json::Value const& args) { setTrace(args["value"]); }},
+		{"initialize", std::bind(&LanguageServer::handleInitialize, this, _1, _2)},
+		{"initialized", std::bind(&LanguageServer::handleInitialized, this, _1, _2)},
+		{"$/setTrace", [this](auto, Json const& args) { setTrace(args["value"]); }},
 		{"shutdown", [this](auto, auto) { m_state = State::ShutdownRequested; }},
 		{"textDocument/definition", GotoDefinition(*this) },
-		{"textDocument/didOpen", bind(&LanguageServer::handleTextDocumentDidOpen, this, _2)},
-		{"textDocument/didChange", bind(&LanguageServer::handleTextDocumentDidChange, this, _2)},
-		{"textDocument/didClose", bind(&LanguageServer::handleTextDocumentDidClose, this, _2)},
+		{"textDocument/didOpen", std::bind(&LanguageServer::handleTextDocumentDidOpen, this, _2)},
+		{"textDocument/didChange", std::bind(&LanguageServer::handleTextDocumentDidChange, this, _2)},
+		{"textDocument/didClose", std::bind(&LanguageServer::handleTextDocumentDidClose, this, _2)},
 		{"textDocument/hover", DocumentHoverHandler(*this) },
 		{"textDocument/rename", RenameSymbol(*this) },
 		{"textDocument/implementation", GotoDefinition(*this) },
-		{"textDocument/semanticTokens/full", bind(&LanguageServer::semanticTokensFull, this, _1, _2)},
-		{"workspace/didChangeConfiguration", bind(&LanguageServer::handleWorkspaceDidChangeConfiguration, this, _2)},
+		{"textDocument/semanticTokens/full", std::bind(&LanguageServer::semanticTokensFull, this, _1, _2)},
+		{"workspace/didChangeConfiguration", std::bind(&LanguageServer::handleWorkspaceDidChangeConfiguration, this, _2)},
 	},
 	m_fileRepository("/" /* basePath */, {} /* no search paths */),
 	m_compilerStack{m_fileRepository.reader()}
 {
 }
 
-Json::Value LanguageServer::toRange(SourceLocation const& _location)
+Json LanguageServer::toRange(SourceLocation const& _location)
 {
 	return HandlerBase(*this).toRange(_location);
 }
 
-Json::Value LanguageServer::toJson(SourceLocation const& _location)
+Json LanguageServer::toJson(SourceLocation const& _location)
 {
 	return HandlerBase(*this).toJson(_location);
 }
 
-void LanguageServer::changeConfiguration(Json::Value const& _settings)
+void LanguageServer::changeConfiguration(Json const& _settings)
 {
 	// The settings item: "file-load-strategy" (enum) defaults to "project-directory" if not (or not correctly) set.
 	// It can be overridden during client's handshake or at runtime, as usual.
@@ -177,9 +177,9 @@ void LanguageServer::changeConfiguration(Json::Value const& _settings)
 	//
 	// If this value is set to "directly-opened-and-on-import", then only currently directly opened files and
 	// those files being imported directly or indirectly will be included in operations.
-	if (_settings["file-load-strategy"])
+	if (_settings.contains("file-load-strategy"))
 	{
-		auto const text = _settings["file-load-strategy"].asString();
+		auto const text = _settings["file-load-strategy"].get<std::string>();
 		if (text == "project-directory")
 			m_fileLoadStrategy = FileLoadStrategy::ProjectDirectory;
 		else if (text == "directly-opened-and-on-import")
@@ -189,18 +189,18 @@ void LanguageServer::changeConfiguration(Json::Value const& _settings)
 	}
 
 	m_settingsObject = _settings;
-	Json::Value jsonIncludePaths = _settings["include-paths"];
+	Json jsonIncludePaths = _settings.contains("include-paths") ? _settings["include-paths"] : Json::object();
 
-	if (jsonIncludePaths)
+	if (!jsonIncludePaths.empty())
 	{
 		int typeFailureCount = 0;
-		if (jsonIncludePaths.isArray())
+		if (jsonIncludePaths.is_array())
 		{
-			vector<boost::filesystem::path> includePaths;
-			for (Json::Value const& jsonPath: jsonIncludePaths)
+			std::vector<boost::filesystem::path> includePaths;
+			for (Json const& jsonPath: jsonIncludePaths)
 			{
-				if (jsonPath.isString())
-					includePaths.emplace_back(boost::filesystem::path(jsonPath.asString()));
+				if (jsonPath.is_string())
+					includePaths.emplace_back(jsonPath.get<std::string>());
 				else
 					typeFailureCount++;
 			}
@@ -214,9 +214,9 @@ void LanguageServer::changeConfiguration(Json::Value const& _settings)
 	}
 }
 
-vector<boost::filesystem::path> LanguageServer::allSolidityFilesFromProject() const
+std::vector<boost::filesystem::path> LanguageServer::allSolidityFilesFromProject() const
 {
-	vector<fs::path> collectedPaths{};
+	std::vector<fs::path> collectedPaths{};
 
 	// We explicitly decided against including all files from include paths but leave the possibility
 	// open for a future PR to enable such a feature to be optionally enabled (default disabled).
@@ -242,7 +242,7 @@ void LanguageServer::compile()
 	// so we just remove all non-open files.
 
 	FileRepository oldRepository(m_fileRepository.basePath(), m_fileRepository.includePaths());
-	swap(oldRepository, m_fileRepository);
+	std::swap(oldRepository, m_fileRepository);
 
 	// Load all solidity files from project.
 	if (m_fileLoadStrategy == FileLoadStrategy::ProjectDirectory)
@@ -256,7 +256,7 @@ void LanguageServer::compile()
 		}
 
 	// Overwrite all files as opened by the client, including the ones which might potentially have changes.
-	for (string const& fileName: m_openFiles)
+	for (std::string const& fileName: m_openFiles)
 		m_fileRepository.setSourceByUri(
 			fileName,
 			oldRepository.sourceUnits().at(oldRepository.uriToSourceUnitName(fileName))
@@ -266,7 +266,7 @@ void LanguageServer::compile()
 
 	m_compilerStack.reset(false);
 	m_compilerStack.setSources(m_fileRepository.sourceUnits());
-	m_compilerStack.compile(CompilerStack::State::AnalysisPerformed);
+	m_compilerStack.compile(CompilerStack::State::AnalysisSuccessful);
 }
 
 void LanguageServer::compileAndUpdateDiagnostics()
@@ -275,25 +275,25 @@ void LanguageServer::compileAndUpdateDiagnostics()
 
 	// These are the source units we will sent diagnostics to the client for sure,
 	// even if it is just to clear previous diagnostics.
-	map<string, Json::Value> diagnosticsBySourceUnit;
-	for (string const& sourceUnitName: m_fileRepository.sourceUnits() | ranges::views::keys)
-		diagnosticsBySourceUnit[sourceUnitName] = Json::arrayValue;
-	for (string const& sourceUnitName: m_nonemptyDiagnostics)
-		diagnosticsBySourceUnit[sourceUnitName] = Json::arrayValue;
+	std::map<std::string, Json> diagnosticsBySourceUnit;
+	for (std::string const& sourceUnitName: m_fileRepository.sourceUnits() | ranges::views::keys)
+		diagnosticsBySourceUnit[sourceUnitName] = Json::array();
+	for (std::string const& sourceUnitName: m_nonemptyDiagnostics)
+		diagnosticsBySourceUnit[sourceUnitName] = Json::array();
 
-	for (shared_ptr<Error const> const& error: m_compilerStack.errors())
+	for (std::shared_ptr<Error const> const& error: m_compilerStack.errors())
 	{
 		SourceLocation const* location = error->sourceLocation();
 		if (!location || !location->sourceName)
 			// LSP only has diagnostics applied to individual files.
 			continue;
 
-		Json::Value jsonDiag;
+		Json jsonDiag;
 		jsonDiag["source"] = "solc";
 		jsonDiag["severity"] = toDiagnosticSeverity(error->type());
-		jsonDiag["code"] = Json::UInt64{error->errorId().error};
-		string message = Error::formatErrorType(error->type()) + ":";
-		if (string const* comment = error->comment())
+		jsonDiag["code"] = Json(error->errorId().error);
+		std::string message = Error::formatErrorType(error->type()) + ":";
+		if (std::string const* comment = error->comment())
 			message += " " + *comment;
 		jsonDiag["message"] = std::move(message);
 		jsonDiag["range"] = toRange(*location);
@@ -301,26 +301,26 @@ void LanguageServer::compileAndUpdateDiagnostics()
 		if (auto const* secondary = error->secondarySourceLocation())
 			for (auto&& [secondaryMessage, secondaryLocation]: secondary->infos)
 			{
-				Json::Value jsonRelated;
+				Json jsonRelated;
 				jsonRelated["message"] = secondaryMessage;
 				jsonRelated["location"] = toJson(secondaryLocation);
-				jsonDiag["relatedInformation"].append(jsonRelated);
+				jsonDiag["relatedInformation"].emplace_back(jsonRelated);
 			}
 
-		diagnosticsBySourceUnit[*location->sourceName].append(jsonDiag);
+		diagnosticsBySourceUnit[*location->sourceName].emplace_back(jsonDiag);
 	}
 
 	if (m_client.traceValue() != TraceValue::Off)
 	{
-		Json::Value extra;
-		extra["openFileCount"] = Json::UInt64(diagnosticsBySourceUnit.size());
-		m_client.trace("Number of currently open files: " + to_string(diagnosticsBySourceUnit.size()), extra);
+		Json extra;
+		extra["openFileCount"] = Json(diagnosticsBySourceUnit.size());
+		m_client.trace("Number of currently open files: " + std::to_string(diagnosticsBySourceUnit.size()), extra);
 	}
 
 	m_nonemptyDiagnostics.clear();
 	for (auto&& [sourceUnitName, diagnostics]: diagnosticsBySourceUnit)
 	{
-		Json::Value params;
+		Json params;
 		params["uri"] = m_fileRepository.sourceUnitNameToUri(sourceUnitName);
 		if (!diagnostics.empty())
 			m_nonemptyDiagnostics.insert(sourceUnitName);
@@ -336,14 +336,15 @@ bool LanguageServer::run()
 		MessageID id;
 		try
 		{
-			optional<Json::Value> const jsonMessage = m_client.receive();
+			std::optional<Json> const jsonMessage = m_client.receive();
 			if (!jsonMessage)
 				continue;
 
-			if ((*jsonMessage)["method"].isString())
+			if ((*jsonMessage).contains("method") && (*jsonMessage)["method"].is_string())
 			{
-				string const methodName = (*jsonMessage)["method"].asString();
-				id = (*jsonMessage)["id"];
+				std::string const methodName = (*jsonMessage)["method"].get<std::string>();
+				if ((*jsonMessage).contains("id"))
+					id = (*jsonMessage)["id"];
 				lspDebug(fmt::format("received method call: {}", methodName));
 
 				if (auto handler = util::valueOrDefault(m_handlers, methodName))
@@ -354,7 +355,7 @@ bool LanguageServer::run()
 			else
 				m_client.error({}, ErrorCode::ParseError, "\"method\" has to be a string.");
 		}
-		catch (Json::Exception const&)
+		catch (Json::exception const&)
 		{
 			m_client.error(id, ErrorCode::InvalidParams, "JSON object access error. Most likely due to a badly formatted JSON request message."s);
 		}
@@ -379,7 +380,7 @@ void LanguageServer::requireServerInitialized()
 	);
 }
 
-void LanguageServer::handleInitialize(MessageID _id, Json::Value const& _args)
+void LanguageServer::handleInitialize(MessageID _id, Json const& _args)
 {
 	lspRequire(
 		m_state == State::Started,
@@ -391,10 +392,10 @@ void LanguageServer::handleInitialize(MessageID _id, Json::Value const& _args)
 
 	// The default of FileReader is to use `.`, but the path from where the LSP was started
 	// should not matter.
-	string rootPath("/");
-	if (Json::Value uri = _args["rootUri"])
+	std::string rootPath("/");
+	if (_args.contains("rootUri") && _args["rootUri"].is_string())
 	{
-		rootPath = uri.asString();
+		rootPath = _args["rootUri"].get<std::string>();
 		lspRequire(
 			boost::starts_with(rootPath, "file://"),
 			ErrorCode::InvalidParams,
@@ -402,19 +403,19 @@ void LanguageServer::handleInitialize(MessageID _id, Json::Value const& _args)
 		);
 		rootPath = stripFileUriSchemePrefix(rootPath);
 	}
-	else if (Json::Value rootPath = _args["rootPath"])
-		rootPath = rootPath.asString();
+	else if (_args.contains("rootPath"))
+		rootPath = _args["rootPath"].get<std::string>();
 
-	if (_args["trace"])
+	if (_args.contains("trace"))
 		setTrace(_args["trace"]);
 
 	m_fileRepository = FileRepository(rootPath, {});
-	if (_args["initializationOptions"].isObject())
+	if (_args.contains("initializationOptions") && _args["initializationOptions"].is_object())
 		changeConfiguration(_args["initializationOptions"]);
 
-	Json::Value replyArgs;
+	Json replyArgs;
 	replyArgs["serverInfo"]["name"] = "solc";
-	replyArgs["serverInfo"]["version"] = string(VersionNumber);
+	replyArgs["serverInfo"]["version"] = std::string(VersionNumber);
 	replyArgs["capabilities"]["definitionProvider"] = true;
 	replyArgs["capabilities"]["implementationProvider"] = true;
 	replyArgs["capabilities"]["textDocumentSync"]["change"] = 2; // 0=none, 1=full, 2=incremental
@@ -428,44 +429,49 @@ void LanguageServer::handleInitialize(MessageID _id, Json::Value const& _args)
 	m_client.reply(_id, std::move(replyArgs));
 }
 
-void LanguageServer::handleInitialized(MessageID, Json::Value const&)
+void LanguageServer::handleInitialized(MessageID, Json const&)
 {
 	if (m_fileLoadStrategy == FileLoadStrategy::ProjectDirectory)
 		compileAndUpdateDiagnostics();
 }
 
-void LanguageServer::semanticTokensFull(MessageID _id, Json::Value const& _args)
+void LanguageServer::semanticTokensFull(MessageID _id, Json const& _args)
 {
-	auto uri = _args["textDocument"]["uri"];
+	if (_args.contains("textDocument") && _args["textDocument"].contains("uri"))
+	{
+		auto uri = _args["textDocument"]["uri"];
 
-	compile();
+		compile();
 
-	auto const sourceName = m_fileRepository.uriToSourceUnitName(uri.as<string>());
-	SourceUnit const& ast = m_compilerStack.ast(sourceName);
-	m_compilerStack.charStream(sourceName);
-	Json::Value data = SemanticTokensBuilder().build(ast, m_compilerStack.charStream(sourceName));
+		auto const sourceName = m_fileRepository.uriToSourceUnitName(uri.get<std::string>());
+		SourceUnit const& ast = m_compilerStack.ast(sourceName);
+		m_compilerStack.charStream(sourceName);
+		Json data = SemanticTokensBuilder().build(ast, m_compilerStack.charStream(sourceName));
 
-	Json::Value reply = Json::objectValue;
-	reply["data"] = data;
+		Json reply;
+		reply["data"] = data;
 
-	m_client.reply(_id, std::move(reply));
+		m_client.reply(_id, std::move(reply));
+	}
+	else
+		m_client.error(_id, ErrorCode::InvalidParams, "Invalid parameter: textDocument.uri expected.");
 }
 
-void LanguageServer::handleWorkspaceDidChangeConfiguration(Json::Value const& _args)
+void LanguageServer::handleWorkspaceDidChangeConfiguration(Json const& _args)
 {
 	requireServerInitialized();
 
-	if (_args["settings"].isObject())
+	if (_args.contains("settings") && _args["settings"].is_object())
 		changeConfiguration(_args["settings"]);
 }
 
-void LanguageServer::setTrace(Json::Value const& _args)
+void LanguageServer::setTrace(Json const& _args)
 {
-	if (!_args.isString())
+	if (!_args.is_string())
 		// Simply ignore invalid parameter.
 		return;
 
-	string const stringValue = _args.asString();
+	std::string const stringValue = _args.get<std::string>();
 	if (stringValue == "off")
 		m_client.setTrace(TraceValue::Off);
 	else if (stringValue == "messages")
@@ -474,93 +480,105 @@ void LanguageServer::setTrace(Json::Value const& _args)
 		m_client.setTrace(TraceValue::Verbose);
 }
 
-void LanguageServer::handleTextDocumentDidOpen(Json::Value const& _args)
+void LanguageServer::handleTextDocumentDidOpen(Json const& _args)
 {
 	requireServerInitialized();
 
 	lspRequire(
-		_args["textDocument"],
+		_args.contains("textDocument"),
 		ErrorCode::RequestFailed,
 		"Text document parameter missing."
 	);
 
-	string text = _args["textDocument"]["text"].asString();
-	string uri = _args["textDocument"]["uri"].asString();
-	m_openFiles.insert(uri);
-	m_fileRepository.setSourceByUri(uri, std::move(text));
-	compileAndUpdateDiagnostics();
-}
-
-void LanguageServer::handleTextDocumentDidChange(Json::Value const& _args)
-{
-	requireServerInitialized();
-
-	string const uri = _args["textDocument"]["uri"].asString();
-
-	for (Json::Value jsonContentChange: _args["contentChanges"])
+	if (_args["textDocument"].contains("text") && _args["textDocument"].contains("uri"))
 	{
-		lspRequire(
-			jsonContentChange.isObject(),
-			ErrorCode::RequestFailed,
-			"Invalid content reference."
-		);
-
-		string const sourceUnitName = m_fileRepository.uriToSourceUnitName(uri);
-		lspRequire(
-			m_fileRepository.sourceUnits().count(sourceUnitName),
-			ErrorCode::RequestFailed,
-			"Unknown file: " + uri
-		);
-
-		string text = jsonContentChange["text"].asString();
-		if (jsonContentChange["range"].isObject()) // otherwise full content update
-		{
-			optional<SourceLocation> change = parseRange(m_fileRepository, sourceUnitName, jsonContentChange["range"]);
-			lspRequire(
-				change && change->hasText(),
-				ErrorCode::RequestFailed,
-				"Invalid source range: " + util::jsonCompactPrint(jsonContentChange["range"])
-			);
-
-			string buffer = m_fileRepository.sourceUnits().at(sourceUnitName);
-			buffer.replace(static_cast<size_t>(change->start), static_cast<size_t>(change->end - change->start), std::move(text));
-			text = std::move(buffer);
-		}
+		std::string text = _args["textDocument"]["text"].get<std::string>();
+		std::string uri = _args["textDocument"]["uri"].get<std::string>();
+		m_openFiles.insert(uri);
 		m_fileRepository.setSourceByUri(uri, std::move(text));
+		compileAndUpdateDiagnostics();
 	}
-
-	compileAndUpdateDiagnostics();
 }
 
-void LanguageServer::handleTextDocumentDidClose(Json::Value const& _args)
+void LanguageServer::handleTextDocumentDidChange(Json const& _args)
+{
+	requireServerInitialized();
+
+	if (_args.contains("textDocument") && _args["textDocument"].contains("uri"))
+	{
+		std::string const uri = _args["textDocument"]["uri"].get<std::string>();
+
+		if (_args.contains("contentChanges"))
+			for (auto const& [_, jsonContentChange]: _args["contentChanges"].items())
+			{
+				lspRequire(jsonContentChange.is_object(), ErrorCode::RequestFailed, "Invalid content reference.");
+
+				std::string const sourceUnitName = m_fileRepository.uriToSourceUnitName(uri);
+				lspRequire(
+					m_fileRepository.sourceUnits().count(sourceUnitName),
+					ErrorCode::RequestFailed,
+					"Unknown file: " + uri);
+
+				if (jsonContentChange.contains("text"))
+				{
+					std::string text = jsonContentChange["text"].get<std::string>();
+					if (jsonContentChange.contains("range")
+						&& jsonContentChange["range"].is_object()) // otherwise full content update
+					{
+						std::optional<SourceLocation> change
+							= parseRange(m_fileRepository, sourceUnitName, jsonContentChange["range"]);
+						lspRequire(
+							change && change->hasText(),
+							ErrorCode::RequestFailed,
+							"Invalid source range: " + util::jsonCompactPrint(jsonContentChange["range"]));
+
+						std::string buffer = m_fileRepository.sourceUnits().at(sourceUnitName);
+						buffer.replace(
+							static_cast<size_t>(change->start),
+							static_cast<size_t>(change->end - change->start),
+							std::move(text));
+						text = std::move(buffer);
+					}
+					m_fileRepository.setSourceByUri(uri, std::move(text));
+				}
+			}
+
+		compileAndUpdateDiagnostics();
+	}
+}
+
+void LanguageServer::handleTextDocumentDidClose(Json const& _args)
 {
 	requireServerInitialized();
 
 	lspRequire(
-		_args["textDocument"],
+		_args.contains("textDocument"),
 		ErrorCode::RequestFailed,
 		"Text document parameter missing."
 	);
 
-	string uri = _args["textDocument"]["uri"].asString();
-	m_openFiles.erase(uri);
+	if (_args["textDocument"].contains("uri"))
+	{
+		std::string uri = _args["textDocument"]["uri"].get<std::string>();
+		m_openFiles.erase(uri);
 
-	compileAndUpdateDiagnostics();
+		compileAndUpdateDiagnostics();
+	}
 }
 
 ASTNode const* LanguageServer::astNodeAtSourceLocation(std::string const& _sourceUnitName, LineColumn const& _filePos)
 {
-	return get<ASTNode const*>(astNodeAndOffsetAtSourceLocation(_sourceUnitName, _filePos));
+	return std::get<ASTNode const*>(astNodeAndOffsetAtSourceLocation(_sourceUnitName, _filePos));
 }
 
-tuple<ASTNode const*, int> LanguageServer::astNodeAndOffsetAtSourceLocation(std::string const& _sourceUnitName, LineColumn const& _filePos)
+std::tuple<ASTNode const*, int> LanguageServer::astNodeAndOffsetAtSourceLocation(std::string const& _sourceUnitName, LineColumn const& _filePos)
 {
-	if (m_compilerStack.state() < CompilerStack::AnalysisPerformed)
+	if (m_compilerStack.state() < CompilerStack::AnalysisSuccessful)
 		return {nullptr, -1};
 	if (!m_fileRepository.sourceUnits().count(_sourceUnitName))
 		return {nullptr, -1};
 
-	optional<int> sourcePos = m_compilerStack.charStream(_sourceUnitName).translateLineColumnToPosition(_filePos);
+	std::optional<int> sourcePos = m_compilerStack.charStream(_sourceUnitName).translateLineColumnToPosition(_filePos);
 	if (!sourcePos)
 		return {nullptr, -1};
 
