@@ -41,10 +41,9 @@
 
 #include <fmt/format.h>
 
-#include <memory>
 #include <functional>
 
-using namespace std;
+using namespace std::string_literals;
 using namespace solidity;
 using namespace solidity::yul;
 using namespace solidity::util;
@@ -52,7 +51,7 @@ using namespace solidity::langutil;
 
 namespace
 {
-inline string to_string(LiteralKind _kind)
+inline std::string to_string(LiteralKind _kind)
 {
 	switch (_kind)
 	{
@@ -74,15 +73,27 @@ bool AsmAnalyzer::analyze(Block const& _block)
 
 		(*this)(_block);
 	}
-	catch (FatalError const&)
+	catch (FatalError const& error)
 	{
-		// This FatalError con occur if the errorReporter has too many errors.
-		yulAssert(!watcher.ok(), "Fatal error detected, but no error is reported.");
+		// NOTE: There's a cap on the number of reported errors, but watcher.ok() will work fine even if
+		// we exceed it because the reporter keeps counting (it just stops adding errors to the list).
+		// Note also that fact of exceeding the cap triggers a FatalError so one can get thrown even
+		// if we don't make any of our errors fatal.
+		yulAssert(!watcher.ok(), "Unreported fatal error: "s + error.what());
 	}
 	return watcher.ok();
 }
 
 AsmAnalysisInfo AsmAnalyzer::analyzeStrictAssertCorrect(Dialect const& _dialect, Object const& _object)
+{
+	return analyzeStrictAssertCorrect(_dialect, _object.code()->root(), _object.qualifiedDataNames());
+}
+
+AsmAnalysisInfo AsmAnalyzer::analyzeStrictAssertCorrect(
+	Dialect const& _dialect,
+	Block const& _astRoot,
+	std::set<std::string> const& _qualifiedDataNames
+)
 {
 	ErrorList errorList;
 	langutil::ErrorReporter errors(errorList);
@@ -92,41 +103,38 @@ AsmAnalysisInfo AsmAnalyzer::analyzeStrictAssertCorrect(Dialect const& _dialect,
 		errors,
 		_dialect,
 		{},
-		_object.qualifiedDataNames()
-	).analyze(*_object.code);
+		_qualifiedDataNames
+	).analyze(_astRoot);
 	yulAssert(success && !errors.hasErrors(), "Invalid assembly/yul code.");
 	return analysisInfo;
 }
 
-vector<YulString> AsmAnalyzer::operator()(Literal const& _literal)
+size_t AsmAnalyzer::operator()(Literal const& _literal)
 {
-	expectValidType(_literal.type, nativeLocationOf(_literal));
-	if (_literal.kind == LiteralKind::String && _literal.value.str().size() > 32)
+	bool erroneousLiteralValue = false;
+	if (_literal.kind == LiteralKind::String && !_literal.value.unlimited() && _literal.value.hint() && _literal.value.hint()->size() > 32)
+	{
+		erroneousLiteralValue = true;
 		m_errorReporter.typeError(
 			3069_error,
 			nativeLocationOf(_literal),
-			"String literal too long (" + to_string(_literal.value.str().size()) + " > 32)"
+			"String literal too long (" + std::to_string(formatLiteral(_literal, false /* _validated */ ).size()) + " > 32)"
 		);
-	else if (_literal.kind == LiteralKind::Number && bigint(_literal.value.str()) > u256(-1))
+	}
+	else if (_literal.kind == LiteralKind::Number && _literal.value.hint() && bigint(*_literal.value.hint()) > u256(-1))
+	{
+		erroneousLiteralValue = true;
 		m_errorReporter.typeError(6708_error, nativeLocationOf(_literal), "Number literal too large (> 256 bits)");
-	else if (_literal.kind == LiteralKind::Boolean)
-		yulAssert(_literal.value == "true"_yulstring || _literal.value == "false"_yulstring, "");
+	}
 
-	if (!m_dialect.validTypeForLiteral(_literal.kind, _literal.value, _literal.type))
-		m_errorReporter.typeError(
-			5170_error,
-			nativeLocationOf(_literal),
-			"Invalid type \"" + _literal.type.str() + "\" for literal \"" + _literal.value.str() + "\"."
-		);
-
-	return {_literal.type};
+	yulAssert(erroneousLiteralValue ^ validLiteral(_literal), "Invalid literal after validating it through AsmAnalyzer.");
+	return 1;
 }
 
-vector<YulString> AsmAnalyzer::operator()(Identifier const& _identifier)
+size_t AsmAnalyzer::operator()(Identifier const& _identifier)
 {
 	yulAssert(!_identifier.name.empty(), "");
 	auto watcher = m_errorReporter.errorWatcher();
-	YulString type = m_dialect.defaultType;
 
 	if (m_currentScope->lookup(_identifier.name, GenericVisitor{
 		[&](Scope::Variable const& _var)
@@ -137,7 +145,6 @@ vector<YulString> AsmAnalyzer::operator()(Identifier const& _identifier)
 					nativeLocationOf(_identifier),
 					"Variable " + _identifier.name.str() + " used before it was declared."
 				);
-			type = _var.type;
 		},
 		[&](Scope::Function const&)
 		{
@@ -174,21 +181,21 @@ vector<YulString> AsmAnalyzer::operator()(Identifier const& _identifier)
 
 	}
 
-	return {type};
+	return 1;
 }
 
 void AsmAnalyzer::operator()(ExpressionStatement const& _statement)
 {
 	auto watcher = m_errorReporter.errorWatcher();
-	vector<YulString> types = std::visit(*this, _statement.expression);
-	if (watcher.ok() && !types.empty())
+	size_t numReturns = std::visit(*this, _statement.expression);
+	if (watcher.ok() && numReturns > 0)
 		m_errorReporter.typeError(
 			3083_error,
 			nativeLocationOf(_statement),
 			"Top-level expressions are not supposed to return values (this expression returns " +
-			to_string(types.size()) +
+			std::to_string(numReturns) +
 			" value" +
-			(types.size() == 1 ? "" : "s") +
+			(numReturns == 1 ? "" : "s") +
 			"). Use ``pop()`` or assign them."
 		);
 }
@@ -199,7 +206,7 @@ void AsmAnalyzer::operator()(Assignment const& _assignment)
 	size_t const numVariables = _assignment.variableNames.size();
 	yulAssert(numVariables >= 1, "");
 
-	set<YulString> variables;
+	std::set<YulName> variables;
 	for (auto const& _variableName: _assignment.variableNames)
 		if (!variables.insert(_variableName.name).second)
 			m_errorReporter.declarationError(
@@ -210,24 +217,23 @@ void AsmAnalyzer::operator()(Assignment const& _assignment)
 				" occurs multiple times on the left-hand side of the assignment."
 			);
 
-	vector<YulString> types = std::visit(*this, *_assignment.value);
+	size_t numRhsValues = std::visit(*this, *_assignment.value);
 
-	if (types.size() != numVariables)
+	if (numRhsValues != numVariables)
 		m_errorReporter.declarationError(
 			8678_error,
 			nativeLocationOf(_assignment),
 			"Variable count for assignment to \"" +
 			joinHumanReadable(applyMap(_assignment.variableNames, [](auto const& _identifier){ return _identifier.name.str(); })) +
 			"\" does not match number of values (" +
-			to_string(numVariables) +
+			std::to_string(numVariables) +
 			" vs. " +
-			to_string(types.size()) +
+			std::to_string(numRhsValues) +
 			")"
 		);
 
 	for (size_t i = 0; i < numVariables; ++i)
-		if (i < types.size())
-			checkAssignment(_assignment.variableNames[i], types[i]);
+		checkAssignment(_assignment.variableNames[i]);
 }
 
 void AsmAnalyzer::operator()(VariableDeclaration const& _varDecl)
@@ -244,41 +250,26 @@ void AsmAnalyzer::operator()(VariableDeclaration const& _varDecl)
 	for (auto const& variable: _varDecl.variables)
 	{
 		expectValidIdentifier(variable.name, nativeLocationOf(variable));
-		expectValidType(variable.type, nativeLocationOf(variable));
 	}
 
 	if (_varDecl.value)
 	{
-		vector<YulString> types = std::visit(*this, *_varDecl.value);
-		if (types.size() != numVariables)
+		size_t numValues = std::visit(*this, *_varDecl.value);
+		if (numValues != numVariables)
 			m_errorReporter.declarationError(
 				3812_error,
 				nativeLocationOf(_varDecl),
 				"Variable count mismatch for declaration of \"" +
 				joinHumanReadable(applyMap(_varDecl.variables, [](auto const& _identifier){ return _identifier.name.str(); })) +
 				+ "\": " +
-				to_string(numVariables) +
+				std::to_string(numVariables) +
 				" variables and " +
-				to_string(types.size()) +
+				std::to_string(numValues) +
 				" values."
 			);
-
-		for (size_t i = 0; i < _varDecl.variables.size(); ++i)
-		{
-			YulString givenType = m_dialect.defaultType;
-			if (i < types.size())
-				givenType = types[i];
-			TypedName const& variable = _varDecl.variables[i];
-			if (variable.type != givenType)
-				m_errorReporter.typeError(
-					3947_error,
-					nativeLocationOf(variable),
-					"Assigning value of type \"" + givenType.str() + "\" to variable of type \"" + variable.type.str() + "\"."
-				);
-		}
 	}
 
-	for (TypedName const& variable: _varDecl.variables)
+	for (NameWithDebugData const& variable: _varDecl.variables)
 		m_activeVariables.insert(&std::get<Scope::Variable>(
 			m_currentScope->identifiers.at(variable.name))
 		);
@@ -294,33 +285,50 @@ void AsmAnalyzer::operator()(FunctionDefinition const& _funDef)
 	for (auto const& var: _funDef.parameters + _funDef.returnVariables)
 	{
 		expectValidIdentifier(var.name, nativeLocationOf(var));
-		expectValidType(var.type, nativeLocationOf(var));
 		m_activeVariables.insert(&std::get<Scope::Variable>(varScope.identifiers.at(var.name)));
 	}
 
 	(*this)(_funDef.body);
 }
 
-vector<YulString> AsmAnalyzer::operator()(FunctionCall const& _funCall)
+size_t AsmAnalyzer::operator()(FunctionCall const& _funCall)
 {
 	yulAssert(!_funCall.functionName.name.empty(), "");
 	auto watcher = m_errorReporter.errorWatcher();
-	vector<YulString> const* parameterTypes = nullptr;
-	vector<YulString> const* returnTypes = nullptr;
-	vector<optional<LiteralKind>> const* literalArguments = nullptr;
+	std::optional<size_t> numParameters;
+	std::optional<size_t> numReturns;
+	std::vector<std::optional<LiteralKind>> const* literalArguments = nullptr;
 
 	if (BuiltinFunction const* f = m_dialect.builtin(_funCall.functionName.name))
 	{
-		if (_funCall.functionName.name == "selfdestruct"_yulstring)
+		if (_funCall.functionName.name == "selfdestruct"_yulname)
 			m_errorReporter.warning(
 				1699_error,
 				nativeLocationOf(_funCall.functionName),
 				"\"selfdestruct\" has been deprecated. "
-				"The underlying opcode will eventually undergo breaking changes, "
-				"and its use is not recommended."
+				"Note that, starting from the Cancun hard fork, the underlying opcode no longer deletes the code and "
+				"data associated with an account and only transfers its Ether to the beneficiary, "
+				"unless executed in the same transaction in which the contract was created (see EIP-6780). "
+				"Any use in newly deployed contracts is strongly discouraged even if the new behavior is taken into account. "
+				"Future changes to the EVM might further reduce the functionality of the opcode."
 			);
-		parameterTypes = &f->parameters;
-		returnTypes = &f->returns;
+		else if (
+			m_evmVersion.supportsTransientStorage() &&
+			_funCall.functionName.name == "tstore"_yulname &&
+			!m_errorReporter.hasError({2394})
+		)
+			m_errorReporter.warning(
+				2394_error,
+				nativeLocationOf(_funCall.functionName),
+				"Transient storage as defined by EIP-1153 can break the composability of smart contracts: "
+				"Since transient storage is cleared only at the end of the transaction and not at the end of the outermost call frame to the contract within a transaction, "
+				"your contract may unintentionally misbehave when invoked multiple times in a complex transaction. "
+				"To avoid this, be sure to clear all transient storage at the end of any call to your contract. "
+				"The use of transient storage for reentrancy guards that are cleared at the end of the call is safe."
+			);
+
+		numParameters = f->numParameters;
+		numReturns = f->numReturns;
 		if (!f->literalArguments.empty())
 			literalArguments = &f->literalArguments;
 
@@ -338,8 +346,8 @@ vector<YulString> AsmAnalyzer::operator()(FunctionCall const& _funCall)
 		},
 		[&](Scope::Function const& _fun)
 		{
-			parameterTypes = &_fun.arguments;
-			returnTypes = &_fun.returns;
+			numParameters = _fun.numArguments;
+			numReturns = _fun.numReturns;
 		}
 	}))
 	{
@@ -362,17 +370,16 @@ vector<YulString> AsmAnalyzer::operator()(FunctionCall const& _funCall)
 		yulAssert(!watcher.ok(), "Expected a reported error.");
 	}
 
-	if (parameterTypes && _funCall.arguments.size() != parameterTypes->size())
+	if (numParameters && _funCall.arguments.size() != *numParameters)
 		m_errorReporter.typeError(
 			7000_error,
 			nativeLocationOf(_funCall.functionName),
 			"Function \"" + _funCall.functionName.name.str() + "\" expects " +
-			to_string(parameterTypes->size()) +
+			std::to_string(*numParameters) +
 			" arguments but got " +
-			to_string(_funCall.arguments.size()) + "."
+			std::to_string(_funCall.arguments.size()) + "."
 		);
 
-	vector<YulString> argTypes;
 	for (size_t i = _funCall.arguments.size(); i > 0; i--)
 	{
 		Expression const& arg = _funCall.arguments[i - 1];
@@ -382,13 +389,13 @@ vector<YulString> AsmAnalyzer::operator()(FunctionCall const& _funCall)
 				std::nullopt
 		)
 		{
-			if (!holds_alternative<Literal>(arg))
+			if (!std::holds_alternative<Literal>(arg))
 				m_errorReporter.typeError(
 					9114_error,
 					nativeLocationOf(_funCall.functionName),
 					"Function expects direct literals as arguments."
 				);
-			else if (*literalArgumentKind != get<Literal>(arg).kind)
+			else if (*literalArgumentKind != std::get<Literal>(arg).kind)
 				m_errorReporter.typeError(
 					5859_error,
 					nativeLocationOf(arg),
@@ -396,53 +403,50 @@ vector<YulString> AsmAnalyzer::operator()(FunctionCall const& _funCall)
 				);
 			else if (*literalArgumentKind == LiteralKind::String)
 			{
-				string functionName = _funCall.functionName.name.str();
+				std::string functionName = _funCall.functionName.name.str();
 				if (functionName == "datasize" || functionName == "dataoffset")
 				{
-					if (!m_dataNames.count(get<Literal>(arg).value))
+					auto const& argumentAsLiteral = std::get<Literal>(arg);
+					if (!m_dataNames.count(formatLiteral(argumentAsLiteral)))
 						m_errorReporter.typeError(
 							3517_error,
 							nativeLocationOf(arg),
-							"Unknown data object \"" + std::get<Literal>(arg).value.str() + "\"."
+							"Unknown data object \"" + formatLiteral(argumentAsLiteral) + "\"."
 						);
 				}
 				else if (functionName.substr(0, "verbatim_"s.size()) == "verbatim_")
 				{
-					if (get<Literal>(arg).value.empty())
+					auto const& literalValue = std::get<Literal>(arg).value;
+					yulAssert(literalValue.unlimited());  // verbatim literals are always unlimited
+					if (literalValue.builtinStringLiteralValue().empty())
 						m_errorReporter.typeError(
 							1844_error,
 							nativeLocationOf(arg),
 							"The \"verbatim_*\" builtins cannot be used with empty bytecode."
 						);
 				}
-
-				argTypes.emplace_back(expectUnlimitedStringLiteral(get<Literal>(arg)));
+				expectUnlimitedStringLiteral(std::get<Literal>(arg));
 				continue;
 			}
 		}
-		argTypes.emplace_back(expectExpression(arg));
+		expectExpression(arg);
 	}
-	std::reverse(argTypes.begin(), argTypes.end());
-
-	if (parameterTypes && parameterTypes->size() == argTypes.size())
-		for (size_t i = 0; i < parameterTypes->size(); ++i)
-			expectType((*parameterTypes)[i], argTypes[i], nativeLocationOf(_funCall.arguments[i]));
 
 	if (watcher.ok())
 	{
-		yulAssert(parameterTypes && parameterTypes->size() == argTypes.size(), "");
-		yulAssert(returnTypes, "");
-		return *returnTypes;
+		yulAssert(numParameters && numParameters == _funCall.arguments.size());
+		yulAssert(numReturns);
+		return *numReturns;
 	}
-	else if (returnTypes)
-		return vector<YulString>(returnTypes->size(), m_dialect.defaultType);
+	else if (numReturns)
+		return *numReturns;
 	else
 		return {};
 }
 
 void AsmAnalyzer::operator()(If const& _if)
 {
-	expectBoolExpression(*_if.condition);
+	expectExpression(*_if.condition);
 
 	(*this)(_if.body);
 }
@@ -458,28 +462,26 @@ void AsmAnalyzer::operator()(Switch const& _switch)
 			"\"switch\" statement with only a default case."
 		);
 
-	YulString valueType = expectExpression(*_switch.expression);
+	expectExpression(*_switch.expression);
 
-	set<u256> cases;
+	std::set<u256> cases;
 	for (auto const& _case: _switch.cases)
 	{
 		if (_case.value)
 		{
 			auto watcher = m_errorReporter.errorWatcher();
 
-			expectType(valueType, _case.value->type, nativeLocationOf(*_case.value));
-
 			// We cannot use "expectExpression" here because *_case.value is not an
 			// Expression and would be converted to an Expression otherwise.
 			(*this)(*_case.value);
 
 			/// Note: the parser ensures there is only one default case
-			if (watcher.ok() && !cases.insert(valueOfLiteral(*_case.value)).second)
+			if (watcher.ok() && !cases.insert(_case.value->value.value()).second)
 				m_errorReporter.declarationError(
 					6792_error,
 					nativeLocationOf(_case),
 					"Duplicate case \"" +
-					valueOfLiteral(*_case.value).str() +
+					formatLiteral(*_case.value) +
 					"\" defined."
 				);
 		}
@@ -500,7 +502,7 @@ void AsmAnalyzer::operator()(ForLoop const& _for)
 	// condition, the body and the post part inside.
 	m_currentScope = &scope(&_for.pre);
 
-	expectBoolExpression(*_for.condition);
+	expectExpression(*_for.condition);
 	// backup outer for-loop & create new state
 	auto outerForLoop = m_currentForLoop;
 	m_currentForLoop = &_for;
@@ -523,48 +525,30 @@ void AsmAnalyzer::operator()(Block const& _block)
 	m_currentScope = previousScope;
 }
 
-YulString AsmAnalyzer::expectExpression(Expression const& _expr)
+void AsmAnalyzer::expectExpression(Expression const& _expr)
 {
-	vector<YulString> types = std::visit(*this, _expr);
-	if (types.size() != 1)
+	size_t numValues = std::visit(*this, _expr);
+	if (numValues != 1)
 		m_errorReporter.typeError(
 			3950_error,
 			nativeLocationOf(_expr),
 			"Expected expression to evaluate to one value, but got " +
-			to_string(types.size()) +
+			std::to_string(numValues) +
 			" values instead."
 		);
-	return types.empty() ? m_dialect.defaultType : types.front();
 }
 
-YulString AsmAnalyzer::expectUnlimitedStringLiteral(Literal const& _literal)
+void AsmAnalyzer::expectUnlimitedStringLiteral(Literal const& _literal)
 {
-	yulAssert(_literal.kind == LiteralKind::String, "");
-	yulAssert(m_dialect.validTypeForLiteral(LiteralKind::String, _literal.value, _literal.type), "");
-
-	return {_literal.type};
+	yulAssert(_literal.kind == LiteralKind::String);
+	yulAssert(_literal.value.unlimited());
 }
 
-void AsmAnalyzer::expectBoolExpression(Expression const& _expr)
-{
-	YulString type = expectExpression(_expr);
-	if (type != m_dialect.boolType)
-		m_errorReporter.typeError(
-			1733_error,
-			nativeLocationOf(_expr),
-			"Expected a value of boolean type \"" +
-			m_dialect.boolType.str() +
-			"\" but got \"" +
-			type.str() +
-			"\""
-		);
-}
-
-void AsmAnalyzer::checkAssignment(Identifier const& _variable, YulString _valueType)
+void AsmAnalyzer::checkAssignment(Identifier const& _variable)
 {
 	yulAssert(!_variable.name.empty(), "");
 	auto watcher = m_errorReporter.errorWatcher();
-	YulString const* variableType = nullptr;
+	bool hasVariable = false;
 	bool found = false;
 	if (Scope::Identifier const* var = m_currentScope->lookup(_variable.name))
 	{
@@ -576,7 +560,7 @@ void AsmAnalyzer::checkAssignment(Identifier const& _variable, YulString _valueT
 				m_currentScope->insideFunction()
 			);
 
-		if (!holds_alternative<Scope::Variable>(*var))
+		if (!std::holds_alternative<Scope::Variable>(*var))
 			m_errorReporter.typeError(2657_error, nativeLocationOf(_variable), "Assignment requires variable.");
 		else if (!m_activeVariables.count(&std::get<Scope::Variable>(*var)))
 			m_errorReporter.declarationError(
@@ -585,7 +569,7 @@ void AsmAnalyzer::checkAssignment(Identifier const& _variable, YulString _valueT
 				"Variable " + _variable.name.str() + " used before it was declared."
 			);
 		else
-			variableType = &std::get<Scope::Variable>(*var).type;
+			hasVariable = true;
 		found = true;
 	}
 	else if (m_resolver)
@@ -594,25 +578,15 @@ void AsmAnalyzer::checkAssignment(Identifier const& _variable, YulString _valueT
 		if (m_resolver(_variable, yul::IdentifierContext::LValue, insideFunction))
 		{
 			found = true;
-			variableType = &m_dialect.defaultType;
+			hasVariable = true;
 		}
 	}
 
 	if (!found && watcher.ok())
 		// Only add message if the callback did not.
 		m_errorReporter.declarationError(4634_error, nativeLocationOf(_variable), "Variable not found or variable not lvalue.");
-	if (variableType && *variableType != _valueType)
-		m_errorReporter.typeError(
-			9547_error,
-			nativeLocationOf(_variable),
-			"Assigning a value of type \"" +
-			_valueType.str() +
-			"\" to a variable of type \"" +
-			variableType->str() +
-			"\"."
-		);
 
-	yulAssert(!watcher.ok() || variableType, "");
+	yulAssert(!watcher.ok() || hasVariable, "");
 }
 
 Scope& AsmAnalyzer::scope(Block const* _block)
@@ -623,7 +597,7 @@ Scope& AsmAnalyzer::scope(Block const* _block)
 	return *scopePtr;
 }
 
-void AsmAnalyzer::expectValidIdentifier(YulString _identifier, SourceLocation const& _location)
+void AsmAnalyzer::expectValidIdentifier(YulName _identifier, SourceLocation const& _location)
 {
 	// NOTE: the leading dot case is handled by the parser not allowing it.
 	if (boost::ends_with(_identifier.str(), "."))
@@ -648,29 +622,11 @@ void AsmAnalyzer::expectValidIdentifier(YulString _identifier, SourceLocation co
 		);
 }
 
-void AsmAnalyzer::expectValidType(YulString _type, SourceLocation const& _location)
-{
-	if (!m_dialect.types.count(_type))
-		m_errorReporter.typeError(
-			5473_error,
-			_location,
-			fmt::format("\"{}\" is not a valid type (user defined types are not yet supported).", _type)
-		);
-}
-
-void AsmAnalyzer::expectType(YulString _expectedType, YulString _givenType, SourceLocation const& _location)
-{
-	if (_expectedType != _givenType)
-		m_errorReporter.typeError(
-			3781_error,
-			_location,
-			fmt::format("Expected a value of type \"{}\" but got \"{}\".", _expectedType, _givenType)
-		);
-}
-
 bool AsmAnalyzer::validateInstructions(std::string const& _instructionIdentifier, langutil::SourceLocation const& _location)
 {
-	auto const builtin = EVMDialect::strictAssemblyForEVM(EVMVersion{}).builtin(YulString(_instructionIdentifier));
+	// NOTE: This function uses the default EVM version instead of the currently selected one.
+	// TODO: Add EOF support
+	auto const builtin = EVMDialect::strictAssemblyForEVM(EVMVersion{}, std::nullopt).builtin(YulName(_instructionIdentifier));
 	if (builtin && builtin->instruction.has_value())
 		return validateInstructions(builtin->instruction.value(), _location);
 	else
@@ -692,7 +648,7 @@ bool AsmAnalyzer::validateInstructions(evmasm::Instruction _instr, SourceLocatio
 		_instr != evmasm::Instruction::JUMPDEST,
 	"");
 
-	auto errorForVM = [&](ErrorId _errorId, string const& vmKindMessage) {
+	auto errorForVM = [&](ErrorId _errorId, std::string const& vmKindMessage) {
 		m_errorReporter.typeError(
 			_errorId,
 			_location,
@@ -705,6 +661,9 @@ bool AsmAnalyzer::validateInstructions(evmasm::Instruction _instr, SourceLocatio
 		);
 	};
 
+	// The errors below are meant to be issued when processing an undeclared identifier matching a builtin name
+	// present on the default EVM version but not on the currently selected one,
+	// since the other `validateInstructions()` overload uses the default EVM version.
 	if (_instr == evmasm::Instruction::RETURNDATACOPY && !m_evmVersion.supportsReturndata())
 		errorForVM(7756_error, "only available for Byzantium-compatible");
 	else if (_instr == evmasm::Instruction::RETURNDATASIZE && !m_evmVersion.supportsReturndata())
@@ -727,6 +686,14 @@ bool AsmAnalyzer::validateInstructions(evmasm::Instruction _instr, SourceLocatio
 		errorForVM(7721_error, "only available for Istanbul-compatible");
 	else if (_instr == evmasm::Instruction::BASEFEE && !m_evmVersion.hasBaseFee())
 		errorForVM(5430_error, "only available for London-compatible");
+	else if (_instr == evmasm::Instruction::BLOBBASEFEE && !m_evmVersion.hasBlobBaseFee())
+		errorForVM(6679_error, "only available for Cancun-compatible");
+	else if (_instr == evmasm::Instruction::BLOBHASH && !m_evmVersion.hasBlobHash())
+		errorForVM(8314_error, "only available for Cancun-compatible");
+	else if (_instr == evmasm::Instruction::MCOPY && !m_evmVersion.hasMcopy())
+		errorForVM(7755_error, "only available for Cancun-compatible");
+	else if ((_instr == evmasm::Instruction::TSTORE || _instr == evmasm::Instruction::TLOAD) && !m_evmVersion.supportsTransientStorage())
+		errorForVM(6243_error, "only available for Cancun-compatible");
 	else if (_instr == evmasm::Instruction::PC)
 		m_errorReporter.error(
 			2450_error,
