@@ -46,30 +46,29 @@
 
 using namespace solidity;
 using namespace solidity::yul;
-using namespace std;
 
 StackLayout StackLayoutGenerator::run(CFG const& _cfg)
 {
 	StackLayout stackLayout;
-	StackLayoutGenerator{stackLayout}.processEntryPoint(*_cfg.entry);
+	StackLayoutGenerator{stackLayout, nullptr}.processEntryPoint(*_cfg.entry);
 
 	for (auto& functionInfo: _cfg.functionInfo | ranges::views::values)
-		StackLayoutGenerator{stackLayout}.processEntryPoint(*functionInfo.entry, &functionInfo);
+		StackLayoutGenerator{stackLayout, &functionInfo}.processEntryPoint(*functionInfo.entry, &functionInfo);
 
 	return stackLayout;
 }
 
-map<YulString, vector<StackLayoutGenerator::StackTooDeep>> StackLayoutGenerator::reportStackTooDeep(CFG const& _cfg)
+std::map<YulName, std::vector<StackLayoutGenerator::StackTooDeep>> StackLayoutGenerator::reportStackTooDeep(CFG const& _cfg)
 {
-	map<YulString, vector<StackLayoutGenerator::StackTooDeep>> stackTooDeepErrors;
-	stackTooDeepErrors[YulString{}] = reportStackTooDeep(_cfg, YulString{});
+	std::map<YulName, std::vector<StackLayoutGenerator::StackTooDeep>> stackTooDeepErrors;
+	stackTooDeepErrors[YulName{}] = reportStackTooDeep(_cfg, YulName{});
 	for (auto const& function: _cfg.functions)
 		if (auto errors = reportStackTooDeep(_cfg, function->name); !errors.empty())
 			stackTooDeepErrors[function->name] = std::move(errors);
 	return stackTooDeepErrors;
 }
 
-vector<StackLayoutGenerator::StackTooDeep> StackLayoutGenerator::reportStackTooDeep(CFG const& _cfg, YulString _functionName)
+std::vector<StackLayoutGenerator::StackTooDeep> StackLayoutGenerator::reportStackTooDeep(CFG const& _cfg, YulName _functionName)
 {
 	StackLayout stackLayout;
 	CFG::FunctionInfo const* functionInfo = nullptr;
@@ -83,27 +82,29 @@ vector<StackLayoutGenerator::StackTooDeep> StackLayoutGenerator::reportStackTooD
 		yulAssert(functionInfo, "Function not found.");
 	}
 
-	StackLayoutGenerator generator{stackLayout};
+	StackLayoutGenerator generator{stackLayout, functionInfo};
 	CFG::BasicBlock const* entry = functionInfo ? functionInfo->entry : _cfg.entry;
 	generator.processEntryPoint(*entry);
 	return generator.reportStackTooDeep(*entry);
 }
 
-StackLayoutGenerator::StackLayoutGenerator(StackLayout& _layout): m_layout(_layout)
+StackLayoutGenerator::StackLayoutGenerator(StackLayout& _layout, CFG::FunctionInfo const* _functionInfo):
+	m_layout(_layout),
+	m_currentFunctionInfo(_functionInfo)
 {
 }
 
 namespace
 {
 /// @returns all stack too deep errors that would occur when shuffling @a _source to @a _target.
-vector<StackLayoutGenerator::StackTooDeep> findStackTooDeep(Stack const& _source, Stack const& _target)
+std::vector<StackLayoutGenerator::StackTooDeep> findStackTooDeep(Stack const& _source, Stack const& _target)
 {
 	Stack currentStack = _source;
-	vector<StackLayoutGenerator::StackTooDeep> stackTooDeepErrors;
+	std::vector<StackLayoutGenerator::StackTooDeep> stackTooDeepErrors;
 	auto getVariableChoices = [](auto&& _range) {
-		vector<YulString> result;
+		std::vector<YulName> result;
 		for (auto const& slot: _range)
-			if (auto const* variableSlot = get_if<VariableSlot>(&slot))
+			if (auto const* variableSlot = std::get_if<VariableSlot>(&slot))
 				if (!util::contains(result, variableSlot->variable.get().name))
 					result.push_back(variableSlot->variable.get().name);
 		return result;
@@ -158,7 +159,7 @@ Stack createIdealLayout(Stack const& _operationOutput, Stack const& _post, Calla
 	// PreviousSlot{0}, ..., PreviousSlot{n}, [output<0>], ..., [output<m>]
 	auto layout = ranges::views::iota(0u, preOperationLayoutSize) |
 		ranges::views::transform([](size_t _index) { return PreviousSlot{_index}; }) |
-		ranges::to<vector<variant<PreviousSlot, StackSlot>>>;
+		ranges::to<std::vector<std::variant<PreviousSlot, StackSlot>>>;
 	layout += _operationOutput;
 
 	// Shortcut for trivial case.
@@ -169,23 +170,23 @@ Stack createIdealLayout(Stack const& _operationOutput, Stack const& _post, Calla
 	// that are aware of PreviousSlot's.
 	struct ShuffleOperations
 	{
-		vector<variant<PreviousSlot, StackSlot>>& layout;
+		std::vector<std::variant<PreviousSlot, StackSlot>>& layout;
 		Stack const& post;
 		std::set<StackSlot> outputs;
-		std::map<StackSlot, int> multiplicity;
+		Multiplicity multiplicity;
 		Callable generateSlotOnTheFly;
 		ShuffleOperations(
-			vector<variant<PreviousSlot, StackSlot>>& _layout,
+			std::vector<std::variant<PreviousSlot, StackSlot>>& _layout,
 			Stack const& _post,
 			Callable _generateSlotOnTheFly
 		): layout(_layout), post(_post), generateSlotOnTheFly(_generateSlotOnTheFly)
 		{
 			for (auto const& layoutSlot: layout)
-				if (StackSlot const* slot = get_if<StackSlot>(&layoutSlot))
+				if (StackSlot const* slot = std::get_if<StackSlot>(&layoutSlot))
 					outputs.insert(*slot);
 
 			for (auto const& layoutSlot: layout)
-				if (StackSlot const* slot = get_if<StackSlot>(&layoutSlot))
+				if (StackSlot const* slot = std::get_if<StackSlot>(&layoutSlot))
 					--multiplicity[*slot];
 			for (auto&& slot: post)
 				if (outputs.count(slot) || generateSlotOnTheFly(slot))
@@ -233,7 +234,7 @@ Stack createIdealLayout(Stack const& _operationOutput, Stack const& _post, Calla
 		}
 		void swap(size_t _i)
 		{
-			yulAssert(!holds_alternative<PreviousSlot>(layout.at(layout.size() - _i - 1)) || !holds_alternative<PreviousSlot>(layout.back()), "");
+			yulAssert(!std::holds_alternative<PreviousSlot>(layout.at(layout.size() - _i - 1)) || !std::holds_alternative<PreviousSlot>(layout.back()), "");
 			std::swap(layout.at(layout.size() -  _i - 1), layout.back());
 		}
 		size_t sourceSize() { return layout.size(); }
@@ -248,7 +249,7 @@ Stack createIdealLayout(Stack const& _operationOutput, Stack const& _post, Calla
 	// output in place. The resulting permutation of the PreviousSlot yields the ideal positions of slots
 	// before the operation, i.e. if PreviousSlot{2} is at a position at which _post contains VariableSlot{"tmp"},
 	// then we want the variable tmp in the slot at offset 2 in the layout before the operation.
-	vector<optional<StackSlot>> idealLayout(_post.size(), nullopt);
+	std::vector<std::optional<StackSlot>> idealLayout(_post.size(), std::nullopt);
 	for (auto&& [slot, idealPosition]: ranges::zip_view(_post, layout))
 		if (PreviousSlot* previousSlot = std::get_if<PreviousSlot>(&idealPosition))
 			idealLayout.at(previousSlot->slot) = slot;
@@ -259,7 +260,7 @@ Stack createIdealLayout(Stack const& _operationOutput, Stack const& _post, Calla
 
 	yulAssert(idealLayout.size() == preOperationLayoutSize, "");
 
-	return idealLayout | ranges::views::transform([](optional<StackSlot> s) {
+	return idealLayout | ranges::views::transform([](std::optional<StackSlot> s) {
 		yulAssert(s, "");
 		return *s;
 	}) | ranges::to<Stack>;
@@ -269,7 +270,7 @@ Stack createIdealLayout(Stack const& _operationOutput, Stack const& _post, Calla
 Stack StackLayoutGenerator::propagateStackThroughOperation(Stack _exitStack, CFG::Operation const& _operation, bool _aggressiveStackCompression)
 {
 	// Enable aggressive stack compression for recursive calls.
-	if (auto const* functionCall = get_if<CFG::FunctionCall>(&_operation.operation))
+	if (auto const* functionCall = std::get_if<CFG::FunctionCall>(&_operation.operation))
 		if (functionCall->recursive)
 			_aggressiveStackCompression = true;
 
@@ -283,9 +284,9 @@ Stack StackLayoutGenerator::propagateStackThroughOperation(Stack _exitStack, CFG
 	Stack stack = createIdealLayout(_operation.output, _exitStack, generateSlotOnTheFly);
 
 	// Make sure the resulting previous slots do not overlap with any assignmed variables.
-	if (auto const* assignment = get_if<CFG::Assignment>(&_operation.operation))
+	if (auto const* assignment = std::get_if<CFG::Assignment>(&_operation.operation))
 		for (auto& stackSlot: stack)
-			if (auto const* varSlot = get_if<VariableSlot>(&stackSlot))
+			if (auto const* varSlot = std::get_if<VariableSlot>(&stackSlot))
 				yulAssert(!util::contains(assignment->variables, *varSlot), "");
 
 	// Since stack+_operation.output can be easily shuffled to _exitLayout, the desired layout before the operation
@@ -333,11 +334,11 @@ Stack StackLayoutGenerator::propagateStackThroughBlock(Stack _exitStack, CFG::Ba
 
 void StackLayoutGenerator::processEntryPoint(CFG::BasicBlock const& _entry, CFG::FunctionInfo const* _functionInfo)
 {
-	list<CFG::BasicBlock const*> toVisit{&_entry};
-	set<CFG::BasicBlock const*> visited;
+	std::list<CFG::BasicBlock const*> toVisit{&_entry};
+	std::set<CFG::BasicBlock const*> visited;
 
 	// TODO: check whether visiting only a subset of these in the outer iteration below is enough.
-	list<pair<CFG::BasicBlock const*, CFG::BasicBlock const*>> backwardsJumps = collectBackwardsJumps(_entry);
+	std::list<std::pair<CFG::BasicBlock const*, CFG::BasicBlock const*>> backwardsJumps = collectBackwardsJumps(_entry);
 
 	while (!toVisit.empty())
 	{
@@ -405,10 +406,10 @@ void StackLayoutGenerator::processEntryPoint(CFG::BasicBlock const& _entry, CFG:
 	fillInJunk(_entry, _functionInfo);
 }
 
-optional<Stack> StackLayoutGenerator::getExitLayoutOrStageDependencies(
+std::optional<Stack> StackLayoutGenerator::getExitLayoutOrStageDependencies(
 	CFG::BasicBlock const& _block,
-	set<CFG::BasicBlock const*> const& _visited,
-	list<CFG::BasicBlock const*>& _toVisit
+	std::set<CFG::BasicBlock const*> const& _visited,
+	std::list<CFG::BasicBlock const*>& _toVisit
 ) const
 {
 	return std::visit(util::GenericVisitor{
@@ -432,7 +433,7 @@ optional<Stack> StackLayoutGenerator::getExitLayoutOrStageDependencies(
 				return m_layout.blockInfos.at(_jump.target).entryLayout;
 			// Otherwise stage the jump target for visit and defer the current block.
 			_toVisit.emplace_front(_jump.target);
-			return nullopt;
+			return std::nullopt;
 		},
 		[&](CFG::BasicBlock::ConditionalJump const& _conditionalJump) -> std::optional<Stack>
 		{
@@ -454,7 +455,7 @@ optional<Stack> StackLayoutGenerator::getExitLayoutOrStageDependencies(
 				_toVisit.emplace_front(_conditionalJump.zero);
 			if (!nonZeroVisited)
 				_toVisit.emplace_front(_conditionalJump.nonZero);
-			return nullopt;
+			return std::nullopt;
 		},
 		[&](CFG::BasicBlock::FunctionReturn const& _functionReturn) -> std::optional<Stack>
 		{
@@ -474,9 +475,9 @@ optional<Stack> StackLayoutGenerator::getExitLayoutOrStageDependencies(
 	}, _block.exit);
 }
 
-list<pair<CFG::BasicBlock const*, CFG::BasicBlock const*>> StackLayoutGenerator::collectBackwardsJumps(CFG::BasicBlock const& _entry) const
+std::list<std::pair<CFG::BasicBlock const*, CFG::BasicBlock const*>> StackLayoutGenerator::collectBackwardsJumps(CFG::BasicBlock const& _entry) const
 {
-	list<pair<CFG::BasicBlock const*, CFG::BasicBlock const*>> backwardsJumps;
+	std::list<std::pair<CFG::BasicBlock const*, CFG::BasicBlock const*>> backwardsJumps;
 	util::BreadthFirstSearch<CFG::BasicBlock const*>{{&_entry}}.run([&](CFG::BasicBlock const* _block, auto _addChild) {
 		std::visit(util::GenericVisitor{
 			[&](CFG::BasicBlock::MainExit const&) {},
@@ -538,7 +539,7 @@ void StackLayoutGenerator::stitchConditionalJumps(CFG::BasicBlock const& _block)
 				_addChild(_conditionalJump.zero);
 				_addChild(_conditionalJump.nonZero);
 			},
-			[&](CFG::BasicBlock::FunctionReturn const&)	{},
+			[&](CFG::BasicBlock::FunctionReturn const&) {},
 			[&](CFG::BasicBlock::Terminated const&) { },
 		}, _block->exit);
 	});
@@ -574,7 +575,7 @@ Stack StackLayoutGenerator::combineStack(Stack const& _stack1, Stack const& _sta
 		if (!util::contains(candidate, slot))
 			candidate.emplace_back(slot);
 	cxx20::erase_if(candidate, [](StackSlot const& slot) {
-		return holds_alternative<LiteralSlot>(slot) || holds_alternative<FunctionCallReturnLabelSlot>(slot);
+		return std::holds_alternative<LiteralSlot>(slot) || std::holds_alternative<FunctionCallReturnLabelSlot>(slot);
 	});
 
 	auto evaluate = [&](Stack const& _candidate) -> size_t {
@@ -631,9 +632,9 @@ Stack StackLayoutGenerator::combineStack(Stack const& _stack1, Stack const& _sta
 	return commonPrefix + bestCandidate;
 }
 
-vector<StackLayoutGenerator::StackTooDeep> StackLayoutGenerator::reportStackTooDeep(CFG::BasicBlock const& _entry) const
+std::vector<StackLayoutGenerator::StackTooDeep> StackLayoutGenerator::reportStackTooDeep(CFG::BasicBlock const& _entry) const
 {
-	vector<StackTooDeep> stackTooDeepErrors;
+	std::vector<StackTooDeep> stackTooDeepErrors;
 	util::BreadthFirstSearch<CFG::BasicBlock const*> breadthFirstSearch{{&_entry}};
 	breadthFirstSearch.run([&](CFG::BasicBlock const* _block, auto _addChild) {
 		Stack currentStack = m_layout.blockInfos.at(_block).entryLayout;
@@ -681,7 +682,7 @@ vector<StackLayoutGenerator::StackTooDeep> StackLayoutGenerator::reportStackTooD
 
 Stack StackLayoutGenerator::compressStack(Stack _stack)
 {
-	optional<size_t> firstDupOffset;
+	std::optional<size_t> firstDupOffset;
 	do
 	{
 		if (firstDupOffset)
@@ -740,7 +741,7 @@ void StackLayoutGenerator::fillInJunk(CFG::BasicBlock const& _block, CFG::Functi
 		});
 	};
 	/// @returns the number of operations required to transform @a _source to @a _target.
-	auto evaluateTransform = [](Stack _source, Stack const& _target) -> size_t {
+	auto evaluateTransform = [&](Stack _source, Stack const& _target) -> size_t {
 		size_t opGas = 0;
 		auto swap = [&](unsigned _swapDepth)
 		{
@@ -755,12 +756,23 @@ void StackLayoutGenerator::fillInJunk(CFG::BasicBlock const& _block, CFG::Functi
 				opGas += evmasm::GasMeter::runGas(evmasm::pushInstruction(32), langutil::EVMVersion());
 			else
 			{
-				auto depth = util::findOffset(_source | ranges::views::reverse, _slot);
-				yulAssert(depth);
-				if (*depth < 16)
-					opGas += evmasm::GasMeter::runGas(evmasm::dupInstruction(static_cast<unsigned>(*depth + 1)), langutil::EVMVersion());
+				if (auto depth = util::findOffset(_source | ranges::views::reverse, _slot))
+				{
+					if (*depth < 16)
+						opGas += evmasm::GasMeter::runGas(evmasm::dupInstruction(static_cast<unsigned>(*depth + 1)), langutil::EVMVersion());
+					else
+						opGas += 1000;
+				}
 				else
-					opGas += 1000;
+				{
+					// This has to be a previously unassigned return variable.
+					// We at least sanity-check that it is among the return variables at all.
+					yulAssert(m_currentFunctionInfo && std::holds_alternative<VariableSlot>(_slot));
+					yulAssert(util::contains(m_currentFunctionInfo->returnVariables, std::get<VariableSlot>(_slot)));
+					// Strictly speaking the cost of the PUSH0 depends on the targeted EVM version, but the difference
+					// will not matter here.
+					opGas += evmasm::GasMeter::pushGas(u256(0), langutil::EVMVersion());
+				}
 			}
 		};
 		auto pop = [&]() { opGas += evmasm::GasMeter::runGas(evmasm::Instruction::POP,langutil::EVMVersion()); };
