@@ -22,7 +22,6 @@
 
 #include <functional>
 
-using namespace std;
 using namespace solidity;
 using namespace solidity::yul;
 
@@ -30,15 +29,15 @@ void ForLoopInitRewriter::operator()(Block& _block)
 {
 	util::iterateReplacing(
 		_block.statements,
-		[&](Statement& _stmt) -> std::optional<vector<Statement>>
+		[&](Statement& _stmt) -> std::optional<std::vector<Statement>>
 		{
-			if (holds_alternative<ForLoop>(_stmt))
+			if (std::holds_alternative<ForLoop>(_stmt))
 			{
 				auto& forLoop = std::get<ForLoop>(_stmt);
 				(*this)(forLoop.pre);
 				(*this)(forLoop.body);
 				(*this)(forLoop.post);
-				vector<Statement> rewrite;
+				std::vector<Statement> rewrite;
 				swap(rewrite, forLoop.pre.statements);
 				rewrite.emplace_back(std::move(forLoop));
 				return { std::move(rewrite) };
