@@ -23,15 +23,14 @@
 
 #include <libyul/AST.h>
 
-using namespace std;
 using namespace solidity;
 using namespace solidity::yul;
 
 Expression Substitution::translate(Expression const& _expression)
 {
-	if (holds_alternative<Identifier>(_expression))
+	if (std::holds_alternative<Identifier>(_expression))
 	{
-		YulString name = std::get<Identifier>(_expression).name;
+		YulName name = std::get<Identifier>(_expression).name;
 		if (m_substitutions.count(name))
 			// No recursive substitution
 			return ASTCopier().translate(*m_substitutions.at(name));
