@@ -43,7 +43,6 @@
 #include <libyul/optimiser/ForLoopInitRewriter.h>
 #include <libyul/optimiser/ForLoopConditionIntoBody.h>
 #include <libyul/optimiser/FunctionSpecializer.h>
-#include <libyul/optimiser/ReasoningBasedSimplifier.h>
 #include <libyul/optimiser/Rematerialiser.h>
 #include <libyul/optimiser/UnusedFunctionParameterPruner.h>
 #include <libyul/optimiser/UnusedPruner.h>
@@ -70,7 +69,6 @@
 #include <libyul/AST.h>
 #include <libyul/Object.h>
 
-#include <libyul/backends/wasm/WasmDialect.h>
 #include <libyul/backends/evm/NoOutputAssembly.h>
 
 #include <libsolutil/CommonData.h>
@@ -79,6 +77,8 @@
 
 #include <range/v3/view/map.hpp>
 #include <range/v3/action/remove.hpp>
+#include <range/v3/algorithm/count.hpp>
+#include <range/v3/algorithm/none_of.hpp>
 
 #include <limits>
 #include <tuple>
@@ -88,24 +88,24 @@
 #include <fmt/format.h>
 #endif
 
-using namespace std;
 using namespace solidity;
 using namespace solidity::yul;
 #ifdef PROFILE_OPTIMIZER_STEPS
 using namespace std::chrono;
 #endif
+using namespace std::string_literals;
 
 namespace
 {
 
 #ifdef PROFILE_OPTIMIZER_STEPS
-void outputPerformanceMetrics(map<string, int64_t> const& _metrics)
+void outputPerformanceMetrics(std::map<std::string, int64_t> const& _metrics)
 {
-	vector<pair<string, int64_t>> durations(_metrics.begin(), _metrics.end());
+	std::vector<std::pair<std::string, int64_t>> durations(_metrics.begin(), _metrics.end());
 	sort(
 		durations.begin(),
 		durations.end(),
-		[](pair<string, int64_t> const& _lhs, pair<string, int64_t> const& _rhs) -> bool
+		[](std::pair<std::string, int64_t> const& _lhs, std::pair<std::string, int64_t> const& _rhs) -> bool
 		{
 			return _lhs.second < _rhs.second;
 		}
@@ -115,18 +115,18 @@ void outputPerformanceMetrics(map<string, int64_t> const& _metrics)
 	for (auto&& [step, durationInMicroseconds]: durations)
 		totalDurationInMicroseconds += durationInMicroseconds;
 
-	cerr << "Performance metrics of optimizer steps" << endl;
-	cerr << "======================================" << endl;
+	std::cerr << "Performance metrics of optimizer steps" << std::endl;
+	std::cerr << "======================================" << std::endl;
 	constexpr double microsecondsInSecond = 1000000;
 	for (auto&& [step, durationInMicroseconds]: durations)
 	{
 		double percentage = 100.0 * static_cast<double>(durationInMicroseconds) / static_cast<double>(totalDurationInMicroseconds);
 		double sec = static_cast<double>(durationInMicroseconds) / microsecondsInSecond;
-		cerr << fmt::format("{:>7.3f}% ({} s): {}", percentage, sec, step) << endl;
+		std::cerr << fmt::format("{:>7.3f}% ({} s): {}", percentage, sec, step) << std::endl;
 	}
 	double totalDurationInSeconds = static_cast<double>(totalDurationInMicroseconds) / microsecondsInSecond;
-	cerr << "--------------------------------------" << endl;
-	cerr << fmt::format("{:>7}% ({:.3f} s)", 100, totalDurationInSeconds) << endl;
+	std::cerr << "--------------------------------------" << std::endl;
+	std::cerr << fmt::format("{:>7}% ({:.3f} s)", 100, totalDurationInSeconds) << std::endl;
 }
 #endif
 
@@ -138,10 +138,10 @@ void OptimiserSuite::run(
 	GasMeter const* _meter,
 	Object& _object,
 	bool _optimizeStackAllocation,
-	string_view _optimisationSequence,
-	string_view _optimisationCleanupSequence,
-	optional<size_t> _expectedExecutionsPerDeployment,
-	set<YulString> const& _externallyUsedIdentifiers
+	std::string_view _optimisationSequence,
+	std::string_view _optimisationCleanupSequence,
+	std::optional<size_t> _expectedExecutionsPerDeployment,
+	std::set<YulName> const& _externallyUsedIdentifiers
 )
 {
 	EVMDialect const* evmDialect = dynamic_cast<EVMDialect const*>(&_dialect);
@@ -150,96 +150,99 @@ void OptimiserSuite::run(
 		evmDialect &&
 		evmDialect->evmVersion().canOverchargeGasForCall() &&
 		evmDialect->providesObjectAccess();
-	set<YulString> reservedIdentifiers = _externallyUsedIdentifiers;
+	std::set<YulName> reservedIdentifiers = _externallyUsedIdentifiers;
 	reservedIdentifiers += _dialect.fixedFunctionNames();
 
-	*_object.code = std::get<Block>(Disambiguator(
+	auto astRoot = std::get<Block>(Disambiguator(
 		_dialect,
 		*_object.analysisInfo,
 		reservedIdentifiers
-	)(*_object.code));
-	Block& ast = *_object.code;
+	)(_object.code()->root()));
 
-	NameDispenser dispenser{_dialect, ast, reservedIdentifiers};
+	NameDispenser dispenser{_dialect, astRoot, reservedIdentifiers};
 	OptimiserStepContext context{_dialect, dispenser, reservedIdentifiers, _expectedExecutionsPerDeployment};
 
 	OptimiserSuite suite(context, Debug::None);
 
 	// Some steps depend on properties ensured by FunctionHoister, BlockFlattener, FunctionGrouper and
 	// ForLoopInitRewriter. Run them first to be able to run arbitrary sequences safely.
-	suite.runSequence("hgfo", ast);
+	suite.runSequence("hgfo", astRoot);
 
-	NameSimplifier::run(suite.m_context, ast);
+	NameSimplifier::run(suite.m_context, astRoot);
 	// Now the user-supplied part
-	suite.runSequence(_optimisationSequence, ast);
+	suite.runSequence(_optimisationSequence, astRoot);
 
 	// This is a tuning parameter, but actually just prevents infinite loops.
 	size_t stackCompressorMaxIterations = 16;
-	suite.runSequence("g", ast);
+	suite.runSequence("g", astRoot);
 
 	// We ignore the return value because we will get a much better error
 	// message once we perform code generation.
 	if (!usesOptimizedCodeGenerator)
-		StackCompressor::run(
+	{
+		_object.setCode(std::make_shared<AST>(std::move(astRoot)));
+		astRoot = std::get<1>(StackCompressor::run(
 			_dialect,
 			_object,
 			_optimizeStackAllocation,
 			stackCompressorMaxIterations
-		);
+		));
+	}
 
 	// Run the user-supplied clean up sequence
-	suite.runSequence(_optimisationCleanupSequence, ast);
+	suite.runSequence(_optimisationCleanupSequence, astRoot);
 	// Hard-coded FunctionGrouper step is used to bring the AST into a canonical form required by the StackCompressor
 	// and StackLimitEvader. This is hard-coded as the last step, as some previously executed steps may break the
 	// aforementioned form, thus causing the StackCompressor/StackLimitEvader to throw.
-	suite.runSequence("g", ast);
+	suite.runSequence("g", astRoot);
 
 	if (evmDialect)
 	{
 		yulAssert(_meter, "");
-		ConstantOptimiser{*evmDialect, *_meter}(ast);
+		ConstantOptimiser{*evmDialect, *_meter}(astRoot);
 		if (usesOptimizedCodeGenerator)
 		{
-			StackCompressor::run(
+			_object.setCode(std::make_shared<AST>(std::move(astRoot)));
+			astRoot = std::get<1>(StackCompressor::run(
 				_dialect,
 				_object,
 				_optimizeStackAllocation,
 				stackCompressorMaxIterations
-			);
+			));
 			if (evmDialect->providesObjectAccess())
-				StackLimitEvader::run(suite.m_context, _object);
+			{
+				_object.setCode(std::make_shared<AST>(std::move(astRoot)));
+				astRoot = StackLimitEvader::run(suite.m_context, _object);
+			}
 		}
 		else if (evmDialect->providesObjectAccess() && _optimizeStackAllocation)
-			StackLimitEvader::run(suite.m_context, _object);
-	}
-	else if (dynamic_cast<WasmDialect const*>(&_dialect))
-	{
-		// If the first statement is an empty block, remove it.
-		// We should only have function definitions after that.
-		if (ast.statements.size() > 1 && std::get<Block>(ast.statements.front()).statements.empty())
-			ast.statements.erase(ast.statements.begin());
+		{
+			_object.setCode(std::make_shared<AST>(std::move(astRoot)));
+			astRoot = StackLimitEvader::run(suite.m_context, _object);
+		}
 	}
 
-	dispenser.reset(ast);
-	NameSimplifier::run(suite.m_context, ast);
-	VarNameCleaner::run(suite.m_context, ast);
+	dispenser.reset(astRoot);
+	NameSimplifier::run(suite.m_context, astRoot);
+	VarNameCleaner::run(suite.m_context, astRoot);
 
 #ifdef PROFILE_OPTIMIZER_STEPS
 	outputPerformanceMetrics(suite.m_durationPerStepInMicroseconds);
 #endif
 
-	*_object.analysisInfo = AsmAnalyzer::analyzeStrictAssertCorrect(_dialect, _object);
+	_object.setCode(std::make_shared<AST>(std::move(astRoot)));
+	_object.analysisInfo = std::make_shared<AsmAnalysisInfo>(AsmAnalyzer::analyzeStrictAssertCorrect(_dialect, _object));
 }
 
 namespace
 {
 
 template <class... Step>
-map<string, unique_ptr<OptimiserStep>> optimiserStepCollection()
+std::map<std::string, std::unique_ptr<OptimiserStep>> optimiserStepCollection()
 {
-	map<string, unique_ptr<OptimiserStep>> ret;
-	for (unique_ptr<OptimiserStep>& s: util::make_vector<unique_ptr<OptimiserStep>>(
-		(make_unique<OptimiserStepInstance<Step>>())...
+	std::map<std::string, std::unique_ptr<OptimiserStep>> ret;
+	for (std::unique_ptr<OptimiserStep>& s: util::make_vector<std::unique_ptr<OptimiserStep>>(
+		(std::make_unique<OptimiserStepInstance<Step>>())...
 	))
 	{
 		yulAssert(!ret.count(s->name), "");
@@ -250,9 +253,9 @@ map<string, unique_ptr<OptimiserStep>> optimiserStepCollection()
 
 }
 
-map<string, unique_ptr<OptimiserStep>> const& OptimiserSuite::allSteps()
+std::map<std::string, std::unique_ptr<OptimiserStep>> const& OptimiserSuite::allSteps()
 {
-	static map<string, unique_ptr<OptimiserStep>> instance;
+	static std::map<std::string, std::unique_ptr<OptimiserStep>> instance;
 	if (instance.empty())
 		instance = optimiserStepCollection<
 			BlockFlattener,
@@ -280,7 +283,6 @@ map<string, unique_ptr<OptimiserStep>> const& OptimiserSuite::allSteps()
 			LoopInvariantCodeMotion,
 			UnusedAssignEliminator,
 			UnusedStoreEliminator,
-			ReasoningBasedSimplifier,
 			Rematerialiser,
 			SSAReverser,
 			SSATransform,
@@ -294,9 +296,9 @@ map<string, unique_ptr<OptimiserStep>> const& OptimiserSuite::allSteps()
 	return instance;
 }
 
-map<string, char> const& OptimiserSuite::stepNameToAbbreviationMap()
+std::map<std::string, char> const& OptimiserSuite::stepNameToAbbreviationMap()
 {
-	static map<string, char> lookupTable{
+	static std::map<std::string, char> lookupTable{
 		{BlockFlattener::name,                'f'},
 		{CircularReferencesPruner::name,      'l'},
 		{CommonSubexpressionEliminator::name, 'c'},
@@ -320,7 +322,6 @@ map<string, char> const& OptimiserSuite::stepNameToAbbreviationMap()
 		{LiteralRematerialiser::name,         'T'},
 		{LoadResolver::name,                  'L'},
 		{LoopInvariantCodeMotion::name,       'M'},
-		{ReasoningBasedSimplifier::name,      'R'},
 		{UnusedAssignEliminator::name,        'r'},
 		{UnusedStoreEliminator::name,         'S'},
 		{Rematerialiser::name,                'm'},
@@ -333,23 +334,23 @@ map<string, char> const& OptimiserSuite::stepNameToAbbreviationMap()
 	};
 	yulAssert(lookupTable.size() == allSteps().size(), "");
 	yulAssert((
-			util::convertContainer<set<char>>(string(NonStepAbbreviations)) -
-			util::convertContainer<set<char>>(lookupTable | ranges::views::values)
-		).size() == string(NonStepAbbreviations).size(),
+			util::convertContainer<std::set<char>>(std::string(NonStepAbbreviations)) -
+			util::convertContainer<std::set<char>>(lookupTable | ranges::views::values)
+		).size() == std::string(NonStepAbbreviations).size(),
 		"Step abbreviation conflicts with a character reserved for another syntactic element"
 	);
 
 	return lookupTable;
 }
 
-map<char, string> const& OptimiserSuite::stepAbbreviationToNameMap()
+std::map<char, std::string> const& OptimiserSuite::stepAbbreviationToNameMap()
 {
-	static map<char, string> lookupTable = util::invertMap(stepNameToAbbreviationMap());
+	static std::map<char, std::string> lookupTable = util::invertMap(stepNameToAbbreviationMap());
 
 	return lookupTable;
 }
 
-void OptimiserSuite::validateSequence(string_view _stepAbbreviations)
+void OptimiserSuite::validateSequence(std::string_view _stepAbbreviations)
 {
 	int8_t nestingLevel = 0;
 	int8_t colonDelimiters = 0;
@@ -360,7 +361,7 @@ void OptimiserSuite::validateSequence(string_view _stepAbbreviations)
 		case '\n':
 			break;
 		case '[':
-			assertThrow(nestingLevel < numeric_limits<int8_t>::max(), OptimizerException, "Brackets nested too deep");
+			assertThrow(nestingLevel < std::numeric_limits<int8_t>::max(), OptimizerException, "Brackets nested too deep");
 			nestingLevel++;
 			break;
 		case ']':
@@ -375,7 +376,7 @@ void OptimiserSuite::validateSequence(string_view _stepAbbreviations)
 		default:
 		{
 			yulAssert(
-				string(NonStepAbbreviations).find(abbreviation) == string::npos,
+				std::string(NonStepAbbreviations).find(abbreviation) == std::string::npos,
 				"Unhandled syntactic element in the abbreviation sequence"
 			);
 			assertThrow(
@@ -383,7 +384,7 @@ void OptimiserSuite::validateSequence(string_view _stepAbbreviations)
 				OptimizerException,
 				"'"s + abbreviation + "' is not a valid step abbreviation"
 			);
-			optional<string> invalid = allSteps().at(stepAbbreviationToNameMap().at(abbreviation))->invalidInCurrentEnvironment();
+			std::optional<std::string> invalid = allSteps().at(stepAbbreviationToNameMap().at(abbreviation))->invalidInCurrentEnvironment();
 			assertThrow(
 				!invalid.has_value(),
 				OptimizerException,
@@ -394,12 +395,19 @@ void OptimiserSuite::validateSequence(string_view _stepAbbreviations)
 	assertThrow(nestingLevel == 0, OptimizerException, "Unbalanced brackets");
 }
 
-void OptimiserSuite::runSequence(string_view _stepAbbreviations, Block& _ast, bool _repeatUntilStable)
+bool OptimiserSuite::isEmptyOptimizerSequence(std::string const& _sequence)
+{
+	return
+		ranges::count(_sequence, ':') == 1 &&
+		ranges::none_of(_sequence, [](auto _step) { return _step != ':' && _step != ' ' && _step != '\n'; });
+}
+
+void OptimiserSuite::runSequence(std::string_view _stepAbbreviations, Block& _ast, bool _repeatUntilStable)
 {
 	validateSequence(_stepAbbreviations);
 
 	// This splits 'aaa[bbb]ccc...' into 'aaa' and '[bbb]ccc...'.
-	auto extractNonNestedPrefix = [](string_view _tail) -> tuple<string_view, string_view>
+	auto extractNonNestedPrefix = [](std::string_view _tail) -> std::tuple<std::string_view, std::string_view>
 	{
 		for (size_t i = 0; i < _tail.size(); ++i)
 		{
@@ -411,7 +419,7 @@ void OptimiserSuite::runSequence(string_view _stepAbbreviations, Block& _ast, bo
 	};
 
 	// This splits '[bbb]ccc...' into 'bbb' and 'ccc...'.
-	auto extractBracketContent = [](string_view _tail) -> tuple<string_view, string_view>
+	auto extractBracketContent = [](std::string_view _tail) -> std::tuple<std::string_view, std::string_view>
 	{
 		yulAssert(!_tail.empty() && _tail[0] == '[');
 
@@ -421,7 +429,7 @@ void OptimiserSuite::runSequence(string_view _stepAbbreviations, Block& _ast, bo
 		{
 			if (abbreviation == '[')
 			{
-				yulAssert(nestingLevel < numeric_limits<int8_t>::max());
+				yulAssert(nestingLevel < std::numeric_limits<int8_t>::max());
 				++nestingLevel;
 			}
 			else if (abbreviation == ']')
@@ -438,20 +446,20 @@ void OptimiserSuite::runSequence(string_view _stepAbbreviations, Block& _ast, bo
 		return {_tail.substr(1, contentLength), _tail.substr(contentLength + 2)};
 	};
 
-	auto abbreviationsToSteps = [](string_view _sequence) -> vector<string>
+	auto abbreviationsToSteps = [](std::string_view _sequence) -> std::vector<std::string>
 	{
-		vector<string> steps;
+		std::vector<std::string> steps;
 		for (char abbreviation: _sequence)
 			if (abbreviation != ' ' && abbreviation != '\n')
 				steps.emplace_back(stepAbbreviationToNameMap().at(abbreviation));
 		return steps;
 	};
 
-	vector<tuple<string_view, bool>> subsequences;
-	string_view tail = _stepAbbreviations;
+	std::vector<std::tuple<std::string_view, bool>> subsequences;
+	std::string_view tail = _stepAbbreviations;
 	while (!tail.empty())
 	{
-		string_view subsequence;
+		std::string_view subsequence;
 		tie(subsequence, tail) = extractNonNestedPrefix(tail);
 		if (subsequence.size() > 0)
 			subsequences.push_back({subsequence, false});
@@ -464,7 +472,9 @@ void OptimiserSuite::runSequence(string_view _stepAbbreviations, Block& _ast, bo
 			subsequences.push_back({subsequence, true});
 	}
 
-	size_t codeSize = 0;
+	// NOTE: If _repeatUntilStable is false, the value will not be used so do not calculate it.
+	size_t codeSize = (_repeatUntilStable ? CodeSize::codeSizeIncludingFunctions(_ast) : 0);
+
 	for (size_t round = 0; round < MaxRounds; ++round)
 	{
 		for (auto const& [subsequence, repeat]: subsequences)
@@ -485,15 +495,15 @@ void OptimiserSuite::runSequence(string_view _stepAbbreviations, Block& _ast, bo
 	}
 }
 
-void OptimiserSuite::runSequence(std::vector<string> const& _steps, Block& _ast)
+void OptimiserSuite::runSequence(std::vector<std::string> const& _steps, Block& _ast)
 {
-	unique_ptr<Block> copy;
+	std::unique_ptr<Block> copy;
 	if (m_debug == Debug::PrintChanges)
-		copy = make_unique<Block>(std::get<Block>(ASTCopier{}(_ast)));
-	for (string const& step: _steps)
+		copy = std::make_unique<Block>(std::get<Block>(ASTCopier{}(_ast)));
+	for (std::string const& step: _steps)
 	{
 		if (m_debug == Debug::PrintStep)
-			cout << "Running " << step << endl;
+			std::cout << "Running " << step << std::endl;
 #ifdef PROFILE_OPTIMIZER_STEPS
 		steady_clock::time_point startTime = steady_clock::now();
 #endif
@@ -506,12 +516,12 @@ void OptimiserSuite::runSequence(std::vector<string> const& _steps, Block& _ast)
 		{
 			// TODO should add switch to also compare variable names!
 			if (SyntacticallyEqual{}.statementEqual(_ast, *copy))
-				cout << "== Running " << step << " did not cause changes." << endl;
+				std::cout << "== Running " << step << " did not cause changes." << std::endl;
 			else
 			{
-				cout << "== Running " << step << " changed the AST." << endl;
-				cout << AsmPrinter{}(_ast) << endl;
-				copy = make_unique<Block>(std::get<Block>(ASTCopier{}(_ast)));
+				std::cout << "== Running " << step << " changed the AST." << std::endl;
+				std::cout << AsmPrinter{}(_ast) << std::endl;
+				copy = std::make_unique<Block>(std::get<Block>(ASTCopier{}(_ast)));
 			}
 		}
 	}
