@@ -29,7 +29,6 @@
 
 #include <boost/test/unit_test.hpp>
 
-using namespace std;
 
 namespace solidity::frontend::test
 {
@@ -37,13 +36,13 @@ namespace solidity::frontend::test
 namespace
 {
 
-map<string, string> requireParsedCBORMetadata(bytes const& _bytecode, CompilerStack::MetadataFormat _metadataFormat)
+std::map<std::string, std::string> requireParsedCBORMetadata(bytes const& _bytecode, CompilerStack::MetadataFormat _metadataFormat)
 {
 	bytes cborMetadata = solidity::test::onlyMetadata(_bytecode);
 	if (_metadataFormat != CompilerStack::MetadataFormat::NoMetadata)
 	{
 		BOOST_REQUIRE(!cborMetadata.empty());
-		std::optional<map<string, string>> tmp = solidity::test::parseCBORMetadata(cborMetadata);
+		std::optional<std::map<std::string, std::string>> tmp = solidity::test::parseCBORMetadata(cborMetadata);
 		BOOST_REQUIRE(tmp);
 		return *tmp;
 	}
@@ -51,27 +50,27 @@ map<string, string> requireParsedCBORMetadata(bytes const& _bytecode, CompilerSt
 	return {};
 }
 
-optional<string> compileAndCheckLicenseMetadata(string const& _contractName, char const* _sourceCode)
+std::optional<std::string> compileAndCheckLicenseMetadata(std::string const& _contractName, char const* _sourceCode)
 {
 	CompilerStack compilerStack;
 	compilerStack.setSources({{"A.sol", _sourceCode}});
 	BOOST_REQUIRE_MESSAGE(compilerStack.compile(), "Compiling contract failed");
 
-	string const& serialisedMetadata = compilerStack.metadata(_contractName);
-	Json::Value metadata;
+	std::string const& serialisedMetadata = compilerStack.metadata(_contractName);
+	Json metadata;
 	BOOST_REQUIRE(util::jsonParseStrict(serialisedMetadata, metadata));
 	BOOST_CHECK(solidity::test::isValidMetadata(metadata));
 
 	BOOST_CHECK_EQUAL(metadata["sources"].size(), 1);
-	BOOST_REQUIRE(metadata["sources"].isMember("A.sol"));
+	BOOST_REQUIRE(metadata["sources"].contains("A.sol"));
 
-	if (metadata["sources"]["A.sol"].isMember("license"))
+	if (metadata["sources"]["A.sol"].contains("license"))
 	{
-		BOOST_REQUIRE(metadata["sources"]["A.sol"]["license"].isString());
-		return metadata["sources"]["A.sol"]["license"].asString();
+		BOOST_REQUIRE(metadata["sources"]["A.sol"]["license"].is_string());
+		return metadata["sources"]["A.sol"]["license"].get<std::string>();
 	}
 	else
-		return nullopt;
+		return std::nullopt;
 }
 
 }
@@ -93,7 +92,7 @@ BOOST_AUTO_TEST_CASE(metadata_stamp)
 		CompilerStack::MetadataFormat::WithReleaseVersionTag,
 		CompilerStack::MetadataFormat::WithPrereleaseVersionTag
 	})
-		for (auto metadataHash: set<CompilerStack::MetadataHash>{
+		for (auto metadataHash: std::set<CompilerStack::MetadataHash>{
 			CompilerStack::MetadataHash::IPFS,
 			CompilerStack::MetadataHash::Bzzr1,
 			CompilerStack::MetadataHash::None
@@ -107,7 +106,7 @@ BOOST_AUTO_TEST_CASE(metadata_stamp)
 			compilerStack.setMetadataHash(metadataHash);
 			BOOST_REQUIRE_MESSAGE(compilerStack.compile(), "Compiling contract failed");
 			bytes const& bytecode = compilerStack.runtimeObject("test").bytecode;
-			string const& metadata = compilerStack.metadata("test");
+			std::string const& metadata = compilerStack.metadata("test");
 			BOOST_CHECK(solidity::test::isValidMetadata(metadata));
 
 			auto const cborMetadata = requireParsedCBORMetadata(bytecode, metadataFormat);
@@ -116,7 +115,7 @@ BOOST_AUTO_TEST_CASE(metadata_stamp)
 			else
 			{
 				bytes hash;
-				string hashMethod;
+				std::string hashMethod;
 				if (metadataHash == CompilerStack::MetadataHash::IPFS)
 				{
 					hash = util::ipfsHash(metadata);
@@ -166,7 +165,7 @@ BOOST_AUTO_TEST_CASE(metadata_stamp_experimental)
 			CompilerStack::MetadataFormat::WithReleaseVersionTag,
 			CompilerStack::MetadataFormat::WithPrereleaseVersionTag
 	})
-		for (auto metadataHash: set<CompilerStack::MetadataHash>{
+		for (auto metadataHash: std::set<CompilerStack::MetadataHash>{
 			CompilerStack::MetadataHash::IPFS,
 			CompilerStack::MetadataHash::Bzzr1,
 			CompilerStack::MetadataHash::None
@@ -180,7 +179,7 @@ BOOST_AUTO_TEST_CASE(metadata_stamp_experimental)
 			compilerStack.setMetadataHash(metadataHash);
 			BOOST_REQUIRE_MESSAGE(compilerStack.compile(), "Compiling contract failed");
 			bytes const& bytecode = compilerStack.runtimeObject("test").bytecode;
-			string const& metadata = compilerStack.metadata("test");
+			std::string const& metadata = compilerStack.metadata("test");
 			BOOST_CHECK(solidity::test::isValidMetadata(metadata));
 
 			auto const cborMetadata = requireParsedCBORMetadata(bytecode, metadataFormat);
@@ -189,7 +188,7 @@ BOOST_AUTO_TEST_CASE(metadata_stamp_experimental)
 			else
 			{
 				bytes hash;
-				string hashMethod;
+				std::string hashMethod;
 				if (metadataHash == CompilerStack::MetadataHash::IPFS)
 				{
 					hash = util::ipfsHash(metadata);
@@ -226,45 +225,6 @@ BOOST_AUTO_TEST_CASE(metadata_stamp_experimental)
 		}
 }
 
-BOOST_AUTO_TEST_CASE(metadata_eof_experimental)
-{
-	// Check that setting an EOF version results in the experimental flag being set.
-	char const* sourceCode = R"(
-		pragma solidity >=0.0;
-		contract test {
-			function g(function(uint) external returns (uint) x) public {}
-		}
-	)";
-	for (auto metadataFormat: std::set<CompilerStack::MetadataFormat>{
-		CompilerStack::MetadataFormat::NoMetadata,
-		CompilerStack::MetadataFormat::WithReleaseVersionTag,
-		CompilerStack::MetadataFormat::WithPrereleaseVersionTag
-	})
-	{
-		CompilerStack compilerStack;
-		compilerStack.setMetadataFormat(metadataFormat);
-		compilerStack.setSources({{"", sourceCode}});
-		compilerStack.setEVMVersion({});
-		compilerStack.setViaIR(true);
-		compilerStack.setEOFVersion(1);
-		compilerStack.setOptimiserSettings(true);
-		BOOST_REQUIRE_MESSAGE(compilerStack.compile(), "Compiling contract failed");
-		bytes const& bytecode = compilerStack.runtimeObject("test").bytecode;
-		string const& metadata = compilerStack.metadata("test");
-		BOOST_CHECK(solidity::test::isValidMetadata(metadata));
-
-		auto const cborMetadata = requireParsedCBORMetadata(bytecode, metadataFormat);
-
-		if (metadataFormat == CompilerStack::MetadataFormat::NoMetadata)
-			BOOST_CHECK(cborMetadata.count("experimental") == 0);
-		else
-		{
-			BOOST_CHECK(cborMetadata.count("experimental") == 1);
-			BOOST_CHECK(cborMetadata.at("experimental") == "true");
-		}
-	}
-}
-
 BOOST_AUTO_TEST_CASE(metadata_relevant_sources)
 {
 	CompilerStack compilerStack;
@@ -288,13 +248,13 @@ BOOST_AUTO_TEST_CASE(metadata_relevant_sources)
 	compilerStack.setOptimiserSettings(solidity::test::CommonOptions::get().optimize);
 	BOOST_REQUIRE_MESSAGE(compilerStack.compile(), "Compiling contract failed");
 
-	string const& serialisedMetadata = compilerStack.metadata("A");
-	Json::Value metadata;
+	std::string const& serialisedMetadata = compilerStack.metadata("A");
+	Json metadata;
 	BOOST_REQUIRE(util::jsonParseStrict(serialisedMetadata, metadata));
 	BOOST_CHECK(solidity::test::isValidMetadata(metadata));
 
 	BOOST_CHECK_EQUAL(metadata["sources"].size(), 1);
-	BOOST_CHECK(metadata["sources"].isMember("A"));
+	BOOST_CHECK(metadata["sources"].contains("A"));
 }
 
 BOOST_AUTO_TEST_CASE(metadata_relevant_sources_imports)
@@ -329,15 +289,15 @@ BOOST_AUTO_TEST_CASE(metadata_relevant_sources_imports)
 	compilerStack.setOptimiserSettings(solidity::test::CommonOptions::get().optimize);
 	BOOST_REQUIRE_MESSAGE(compilerStack.compile(), "Compiling contract failed");
 
-	string const& serialisedMetadata = compilerStack.metadata("C");
-	Json::Value metadata;
+	std::string const& serialisedMetadata = compilerStack.metadata("C");
+	Json metadata;
 	BOOST_REQUIRE(util::jsonParseStrict(serialisedMetadata, metadata));
 	BOOST_CHECK(solidity::test::isValidMetadata(metadata));
 
 	BOOST_CHECK_EQUAL(metadata["sources"].size(), 3);
-	BOOST_CHECK(metadata["sources"].isMember("A"));
-	BOOST_CHECK(metadata["sources"].isMember("B"));
-	BOOST_CHECK(metadata["sources"].isMember("C"));
+	BOOST_CHECK(metadata["sources"].contains("A"));
+	BOOST_CHECK(metadata["sources"].contains("B"));
+	BOOST_CHECK(metadata["sources"].contains("C"));
 }
 
 BOOST_AUTO_TEST_CASE(metadata_useLiteralContent)
@@ -357,17 +317,17 @@ BOOST_AUTO_TEST_CASE(metadata_useLiteralContent)
 		compilerStack.setOptimiserSettings(solidity::test::CommonOptions::get().optimize);
 		compilerStack.useMetadataLiteralSources(_literal);
 		BOOST_REQUIRE_MESSAGE(compilerStack.compile(), "Compiling contract failed");
-		string metadata_str = compilerStack.metadata("test");
-		Json::Value metadata;
+		std::string metadata_str = compilerStack.metadata("test");
+		Json metadata;
 		BOOST_REQUIRE(util::jsonParseStrict(metadata_str, metadata));
 		BOOST_CHECK(solidity::test::isValidMetadata(metadata));
-		BOOST_CHECK(metadata.isMember("settings"));
-		BOOST_CHECK(metadata["settings"].isMember("metadata"));
-		BOOST_CHECK(metadata["settings"]["metadata"].isMember("bytecodeHash"));
+		BOOST_CHECK(metadata.contains("settings"));
+		BOOST_CHECK(metadata["settings"].contains("metadata"));
+		BOOST_CHECK(metadata["settings"]["metadata"].contains("bytecodeHash"));
 		if (_literal)
 		{
-			BOOST_CHECK(metadata["settings"]["metadata"].isMember("useLiteralContent"));
-			BOOST_CHECK(metadata["settings"]["metadata"]["useLiteralContent"].asBool());
+			BOOST_CHECK(metadata["settings"]["metadata"].contains("useLiteralContent"));
+			BOOST_CHECK(metadata["settings"]["metadata"]["useLiteralContent"].get<bool>());
 		}
 	};
 
@@ -392,22 +352,22 @@ BOOST_AUTO_TEST_CASE(metadata_viair)
 		compilerStack.setViaIR(_viaIR);
 		BOOST_REQUIRE_MESSAGE(compilerStack.compile(), "Compiling contract failed");
 
-		Json::Value metadata;
+		Json metadata;
 		BOOST_REQUIRE(util::jsonParseStrict(compilerStack.metadata("test"), metadata));
 		BOOST_CHECK(solidity::test::isValidMetadata(metadata));
-		BOOST_CHECK(metadata.isMember("settings"));
+		BOOST_CHECK(metadata.contains("settings"));
 		if (_viaIR)
 		{
-			BOOST_CHECK(metadata["settings"].isMember("viaIR"));
-			BOOST_CHECK(metadata["settings"]["viaIR"].asBool());
+			BOOST_CHECK(metadata["settings"].contains("viaIR"));
+			BOOST_CHECK(metadata["settings"]["viaIR"].get<bool>());
 		}
 		else
-			BOOST_CHECK(!metadata["settings"].isMember("viaIR"));
+			BOOST_CHECK(!metadata["settings"].contains("viaIR"));
 
 		BOOST_CHECK(compilerStack.cborMetadata("test") == compilerStack.cborMetadata("test", _viaIR));
 		BOOST_CHECK(compilerStack.cborMetadata("test") != compilerStack.cborMetadata("test", !_viaIR));
 
-		map<string, string> const parsedCBORMetadata = requireParsedCBORMetadata(
+		std::map<std::string, std::string> const parsedCBORMetadata = requireParsedCBORMetadata(
 			compilerStack.runtimeObject("test").bytecode,
 			CompilerStack::MetadataFormat::WithReleaseVersionTag
 		);
@@ -431,8 +391,8 @@ BOOST_AUTO_TEST_CASE(metadata_revert_strings)
 	compilerStack.setRevertStringBehaviour(RevertStrings::Strip);
 	BOOST_REQUIRE_MESSAGE(compilerStack.compile(), "Compiling contract failed");
 
-	string const& serialisedMetadata = compilerStack.metadata("A");
-	Json::Value metadata;
+	std::string const& serialisedMetadata = compilerStack.metadata("A");
+	Json metadata;
 	BOOST_REQUIRE(util::jsonParseStrict(serialisedMetadata, metadata));
 	BOOST_CHECK(solidity::test::isValidMetadata(metadata));
 
@@ -447,7 +407,7 @@ BOOST_AUTO_TEST_CASE(metadata_optimiser_sequence)
 		}
 	)";
 
-	vector<tuple<string, string>> sequences =
+	std::vector<std::tuple<std::string, std::string>> sequences =
 	{
 		// {"<optimizer sequence>", "<optimizer cleanup sequence>"}
 		{"", ""},
@@ -456,7 +416,7 @@ BOOST_AUTO_TEST_CASE(metadata_optimiser_sequence)
 		{"dhfoDgvulfnTUtnIf", "fDn"}
 	};
 
-	auto check = [sourceCode](string const& _optimizerSequence, string const& _optimizerCleanupSequence)
+	auto check = [sourceCode](std::string const& _optimizerSequence, std::string const& _optimizerCleanupSequence)
 	{
 		OptimiserSettings optimizerSettings = OptimiserSettings::minimal();
 		optimizerSettings.runYulOptimiser = true;
@@ -469,16 +429,16 @@ BOOST_AUTO_TEST_CASE(metadata_optimiser_sequence)
 
 		BOOST_REQUIRE_MESSAGE(compilerStack.compile(), "Compiling contract failed");
 
-		string const& serialisedMetadata = compilerStack.metadata("C");
-		Json::Value metadata;
+		std::string const& serialisedMetadata = compilerStack.metadata("C");
+		Json metadata;
 		BOOST_REQUIRE(util::jsonParseStrict(serialisedMetadata, metadata));
 		BOOST_CHECK(solidity::test::isValidMetadata(metadata));
-		BOOST_CHECK(metadata["settings"]["optimizer"].isMember("details"));
-		BOOST_CHECK(metadata["settings"]["optimizer"]["details"].isMember("yulDetails"));
-		BOOST_CHECK(metadata["settings"]["optimizer"]["details"]["yulDetails"].isMember("optimizerSteps"));
+		BOOST_CHECK(metadata["settings"]["optimizer"].contains("details"));
+		BOOST_CHECK(metadata["settings"]["optimizer"]["details"].contains("yulDetails"));
+		BOOST_CHECK(metadata["settings"]["optimizer"]["details"]["yulDetails"].contains("optimizerSteps"));
 
-		string const metadataOptimizerSteps = metadata["settings"]["optimizer"]["details"]["yulDetails"]["optimizerSteps"].asString();
-		string const expectedMetadataOptimiserSteps = _optimizerSequence + ":" + _optimizerCleanupSequence;
+		std::string const metadataOptimizerSteps = metadata["settings"]["optimizer"]["details"]["yulDetails"]["optimizerSteps"].get<std::string>();
+		std::string const expectedMetadataOptimiserSteps = _optimizerSequence + ":" + _optimizerCleanupSequence;
 		BOOST_CHECK_EQUAL(metadataOptimizerSteps, expectedMetadataOptimiserSteps);
 	};
 
@@ -493,7 +453,7 @@ BOOST_AUTO_TEST_CASE(metadata_license_missing)
 		contract C {
 		}
 	)";
-	BOOST_CHECK(compileAndCheckLicenseMetadata("C", sourceCode) == nullopt);
+	BOOST_CHECK(compileAndCheckLicenseMetadata("C", sourceCode) == std::nullopt);
 }
 
 BOOST_AUTO_TEST_CASE(metadata_license_gpl3)
@@ -544,7 +504,7 @@ BOOST_AUTO_TEST_CASE(metadata_license_bidi_marks)
 		"// NOTE: The text above is reversed using Unicode directional marks. In raw form it would look like this:\n"
 		"// <LRO>0.3-LPG :reifitnedI-esneciL-XDPS<PDF>\n"
 		"contract C {}\n";
-	BOOST_CHECK(compileAndCheckLicenseMetadata("C", sourceCode) == nullopt);
+	BOOST_CHECK(compileAndCheckLicenseMetadata("C", sourceCode) == std::nullopt);
 }
 
 BOOST_AUTO_TEST_CASE(metadata_license_bottom)
@@ -579,7 +539,7 @@ BOOST_AUTO_TEST_CASE(metadata_license_in_string)
 			bytes license = "// SPDX-License-Identifier: GPL-3.0";
 		}
 	)";
-	BOOST_CHECK(compileAndCheckLicenseMetadata("C", sourceCode) == nullopt);
+	BOOST_CHECK(compileAndCheckLicenseMetadata("C", sourceCode) == std::nullopt);
 }
 
 BOOST_AUTO_TEST_CASE(metadata_license_in_contract)
@@ -589,7 +549,7 @@ BOOST_AUTO_TEST_CASE(metadata_license_in_contract)
 		// SPDX-License-Identifier: GPL-3.0
 		}
 	)";
-	BOOST_CHECK(compileAndCheckLicenseMetadata("C", sourceCode) == nullopt);
+	BOOST_CHECK(compileAndCheckLicenseMetadata("C", sourceCode) == std::nullopt);
 }
 
 BOOST_AUTO_TEST_CASE(metadata_license_missing_colon)
@@ -598,7 +558,7 @@ BOOST_AUTO_TEST_CASE(metadata_license_missing_colon)
 		// SPDX-License-Identifier GPL-3.0
 		contract C {}
 	)";
-	BOOST_CHECK(compileAndCheckLicenseMetadata("C", sourceCode) == nullopt);
+	BOOST_CHECK(compileAndCheckLicenseMetadata("C", sourceCode) == std::nullopt);
 }
 
 BOOST_AUTO_TEST_CASE(metadata_license_multiline)
