@@ -23,7 +23,7 @@
 namespace solidity::yul::test
 {
 
-class EVMCodeTransformTest: public solidity::frontend::test::TestCase
+class EVMCodeTransformTest: public solidity::frontend::test::EVMVersionRestrictedTestCase
 {
 public:
 	static std::unique_ptr<TestCase> create(Config const& _config)
