@@ -50,14 +50,13 @@ BOOST_PP_REPEAT(32, USINGDECL, 0)
 /// Switch implementation that instantiates case statements for (un)signed
 /// Solidity integer types.
 #define SWITCHIMPL(sign)                                                       \
-	ostringstream stream;                                                      \
+	std::ostringstream stream;                                                      \
 	switch (_intWidth)                                                         \
 	{                                                                          \
-	BOOST_PP_REPEAT(32, CASEIMPL, sign)	                                       \
-	}	                                                                       \
+	BOOST_PP_REPEAT(32, CASEIMPL, sign)                                        \
+	}                                                                          \
 	return stream.str();
 
-using namespace std;
 using namespace solidity::util;
 using namespace solidity::test::abiv2fuzzer;
 
@@ -75,17 +74,17 @@ static V integerValue(unsigned _counter)
 		return value;
 }
 
-static string signedIntegerValue(unsigned _counter, unsigned _intWidth)
+static std::string signedIntegerValue(unsigned _counter, unsigned _intWidth)
 {
 	SWITCHIMPL(1)
 }
 
-static string unsignedIntegerValue(unsigned _counter, unsigned _intWidth)
+static std::string unsignedIntegerValue(unsigned _counter, unsigned _intWidth)
 {
 	SWITCHIMPL(0)
 }
 
-static string integerValue(unsigned _counter, unsigned _intWidth, bool _signed)
+static std::string integerValue(unsigned _counter, unsigned _intWidth, bool _signed)
 {
 	if (_signed)
 		return signedIntegerValue(_counter, _intWidth);
@@ -94,16 +93,16 @@ static string integerValue(unsigned _counter, unsigned _intWidth, bool _signed)
 }
 }
 
-string ProtoConverter::getVarDecl(
-	string const& _type,
-	string const& _varName,
-	string const& _qualifier
+std::string ProtoConverter::getVarDecl(
+	std::string const& _type,
+	std::string const& _varName,
+	std::string const& _qualifier
 )
 {
 	// One level of indentation for state variable declarations
 	// Two levels of indentation for local variable declarations
 	return Whiskers(R"(
-	<?isLocalVar>	</isLocalVar><type><?qual> <qualifier></qual> <varName>;)"
+	<?isLocalVar> </isLocalVar><type><?qual> <qualifier></qual> <varName>;)"
 		)
 		("isLocalVar", !m_isStateVar)
 		("type", _type)
@@ -114,7 +113,7 @@ string ProtoConverter::getVarDecl(
 		"\n";
 }
 
-pair<string, string> ProtoConverter::visit(Type const& _type)
+std::pair<std::string, std::string> ProtoConverter::visit(Type const& _type)
 {
 	switch (_type.type_oneof_case())
 	{
@@ -123,11 +122,11 @@ pair<string, string> ProtoConverter::visit(Type const& _type)
 	case Type::kNvtype:
 		return visit(_type.nvtype());
 	case Type::TYPE_ONEOF_NOT_SET:
-		return make_pair("", "");
+		return std::make_pair("", "");
 	}
 }
 
-pair<string, string> ProtoConverter::visit(ValueType const& _type)
+std::pair<std::string, std::string> ProtoConverter::visit(ValueType const& _type)
 {
 	switch (_type.value_type_oneof_case())
 	{
@@ -140,11 +139,11 @@ pair<string, string> ProtoConverter::visit(ValueType const& _type)
 	case ValueType::kAdty:
 		return visit(_type.adty());
 	case ValueType::VALUE_TYPE_ONEOF_NOT_SET:
-		return make_pair("", "");
+		return std::make_pair("", "");
 	}
 }
 
-pair<string, string> ProtoConverter::visit(NonValueType const& _type)
+std::pair<std::string, std::string> ProtoConverter::visit(NonValueType const& _type)
 {
 	switch (_type.nonvalue_type_oneof_case())
 	{
@@ -154,56 +153,56 @@ pair<string, string> ProtoConverter::visit(NonValueType const& _type)
 		if (ValidityVisitor().visit(_type.arrtype()))
 			return visit(_type.arrtype());
 		else
-			return make_pair("", "");
+			return std::make_pair("", "");
 	case NonValueType::kStype:
 		if (ValidityVisitor().visit(_type.stype()))
 			return visit(_type.stype());
 		else
-			return make_pair("", "");
+			return std::make_pair("", "");
 	case NonValueType::NONVALUE_TYPE_ONEOF_NOT_SET:
-		return make_pair("", "");
+		return std::make_pair("", "");
 	}
 }
 
-pair<string, string> ProtoConverter::visit(BoolType const& _type)
+std::pair<std::string, std::string> ProtoConverter::visit(BoolType const& _type)
 {
 	return processType(_type, true);
 }
 
-pair<string, string> ProtoConverter::visit(IntegerType const& _type)
+std::pair<std::string, std::string> ProtoConverter::visit(IntegerType const& _type)
 {
 	return processType(_type, true);
 }
 
-pair<string, string> ProtoConverter::visit(FixedByteType const& _type)
+std::pair<std::string, std::string> ProtoConverter::visit(FixedByteType const& _type)
 {
 	return processType(_type, true);
 }
 
-pair<string, string> ProtoConverter::visit(AddressType const& _type)
+std::pair<std::string, std::string> ProtoConverter::visit(AddressType const& _type)
 {
 	return processType(_type, true);
 }
 
-pair<string, string> ProtoConverter::visit(DynamicByteArrayType const& _type)
+std::pair<std::string, std::string> ProtoConverter::visit(DynamicByteArrayType const& _type)
 {
 	return processType(_type, false);
 }
 
-pair<string, string> ProtoConverter::visit(ArrayType const& _type)
+std::pair<std::string, std::string> ProtoConverter::visit(ArrayType const& _type)
 {
 	return processType(_type, false);
 }
 
-pair<string, string> ProtoConverter::visit(StructType const& _type)
+std::pair<std::string, std::string> ProtoConverter::visit(StructType const& _type)
 {
 	return processType(_type, false);
 }
 
 template <typename T>
-pair<string, string> ProtoConverter::processType(T const& _type, bool _isValueType)
+std::pair<std::string, std::string> ProtoConverter::processType(T const& _type, bool _isValueType)
 {
-	ostringstream local, global;
+	std::ostringstream local, global;
 	auto [varName, paramName] = newVarNames(getNextVarCounter(), m_isStateVar);
 
 	// Add variable name to the argument list of coder function call
@@ -212,7 +211,7 @@ pair<string, string> ProtoConverter::processType(T const& _type, bool _isValueTy
 	else
 		m_argsCoder << ", " << varName;
 
-	string location{};
+	std::string location{};
 	if (!m_isStateVar && !_isValueType)
 		location = "memory";
 
@@ -230,24 +229,24 @@ pair<string, string> ProtoConverter::processType(T const& _type, bool _isValueTy
 	local << assignCheckBuffers.second;
 
 	m_structCounter += m_numStructsAdded;
-	return make_pair(global.str(), local.str());
+	return std::make_pair(global.str(), local.str());
 }
 
 template <typename T>
-pair<string, string> ProtoConverter::varDecl(
-	string const& _varName,
-	string const& _paramName,
+std::pair<std::string, std::string> ProtoConverter::varDecl(
+	std::string const& _varName,
+	std::string const& _paramName,
 	T _type,
 	bool _isValueType,
-	string const& _location
+	std::string const& _location
 )
 {
-	ostringstream local, global;
+	std::ostringstream local, global;
 
 	TypeVisitor tVisitor(m_structCounter);
-	string typeStr = tVisitor.visit(_type);
+	std::string typeStr = tVisitor.visit(_type);
 	if (typeStr.empty())
-		return make_pair("", "");
+		return std::make_pair("", "");
 
 	// Append struct defs
 	global << tVisitor.structDef();
@@ -292,17 +291,17 @@ pair<string, string> ProtoConverter::varDecl(
 	if (tVisitor.isLastDynParamRightPadded())
 		m_isLastDynParamRightPadded = true;
 
-	return make_pair(global.str(), local.str());
+	return std::make_pair(global.str(), local.str());
 }
 
 template <typename T>
-pair<string, string> ProtoConverter::assignChecker(
-	string const& _varName,
-	string const& _paramName,
+std::pair<std::string, std::string> ProtoConverter::assignChecker(
+	std::string const& _varName,
+	std::string const& _paramName,
 	T _type
 )
 {
-	ostringstream local;
+	std::ostringstream local;
 	AssignCheckVisitor acVisitor(
 		_varName,
 		_paramName,
@@ -311,7 +310,7 @@ pair<string, string> ProtoConverter::assignChecker(
 		m_counter,
 		m_structCounter
 	);
-	pair<string, string> assignCheckStrPair = acVisitor.visit(_type);
+	std::pair<std::string, std::string> assignCheckStrPair = acVisitor.visit(_type);
 	m_returnValue += acVisitor.errorStmts();
 	m_counter += acVisitor.counted();
 
@@ -325,10 +324,10 @@ pair<string, string> ProtoConverter::assignChecker(
 	// Therefore, we buffer their assignments and
 	// render them in function scope later.
 	local << assignCheckStrPair.first;
-	return make_pair("", local.str());
+	return std::make_pair("", local.str());
 }
 
-pair<string, string> ProtoConverter::visit(VarDecl const& _x)
+std::pair<std::string, std::string> ProtoConverter::visit(VarDecl const& _x)
 {
 	return visit(_x.type());
 }
@@ -375,11 +374,11 @@ void ProtoConverter::appendTypedParams(
 
 void ProtoConverter::appendTypes(
 	bool _isValueType,
-	string const& _typeString,
+	std::string const& _typeString,
 	Delimiter _delimiter
 )
 {
-	string qualifiedTypeString = (
+	std::string qualifiedTypeString = (
 		_isValueType ?
 		_typeString :
 		_typeString + " memory"
@@ -392,11 +391,11 @@ void ProtoConverter::appendTypes(
 
 void ProtoConverter::appendTypedReturn(
 	bool _isValueType,
-	string const& _typeString,
+	std::string const& _typeString,
 	Delimiter _delimiter
 )
 {
-	string qualifiedTypeString = (
+	std::string qualifiedTypeString = (
 		_isValueType ?
 		_typeString :
 		_typeString + " memory"
@@ -404,7 +403,7 @@ void ProtoConverter::appendTypedReturn(
 	m_typedReturn << Whiskers(R"(<delimiter><type> <varName>)")
 		("delimiter", delimiterToString(_delimiter))
 		("type", qualifiedTypeString)
-		("varName", "lv_" + to_string(m_varCounter - 1))
+		("varName", "lv_" + std::to_string(m_varCounter - 1))
 		.render();
 }
 
@@ -467,16 +466,16 @@ std::string ProtoConverter::typedParametersAsString(CalleeType _calleeType)
 		return m_typedParamsPublic.str();
 	case CalleeType::EXTERNAL:
 	{
-		ostringstream typedParamsExternal;
+		std::ostringstream typedParamsExternal;
 		for (auto const& i: m_externalParamsRep)
 		{
-			Delimiter del = get<0>(i);
-			bool valueType = get<1>(i);
-			string typeString = get<2>(i);
-			string varName = get<3>(i);
+			Delimiter del = std::get<0>(i);
+			bool valueType = std::get<1>(i);
+			std::string typeString = std::get<2>(i);
+			std::string varName = std::get<3>(i);
 			bool isCalldata = randomBool(/*probability=*/0.5);
-			string location = (isCalldata ? "calldata" : "memory");
-			string qualifiedTypeString = (valueType ? typeString : typeString + " " + location);
+			std::string location = (isCalldata ? "calldata" : "memory");
+			std::string qualifiedTypeString = (valueType ? typeString : typeString + " " + location);
 			typedParamsExternal << Whiskers(R"(<delimiter><type> <varName>)")
 				("delimiter", delimiterToString(del))
 				("type", qualifiedTypeString)
@@ -488,17 +487,17 @@ std::string ProtoConverter::typedParametersAsString(CalleeType _calleeType)
 	}
 }
 
-string ProtoConverter::visit(TestFunction const& _x, string const& _storageVarDefs)
+std::string ProtoConverter::visit(TestFunction const& _x, std::string const& _storageVarDefs)
 {
 	// TODO: Support more than one but less than N local variables
 	auto localVarBuffers = visit(_x.local_vars());
 
-	string structTypeDecl = localVarBuffers.first;
-	string localVarDefs = localVarBuffers.second;
+	std::string structTypeDecl = localVarBuffers.first;
+	std::string localVarDefs = localVarBuffers.second;
 
-	ostringstream testBuffer;
+	std::ostringstream testBuffer;
 
-	string testFunction = Whiskers(R"(
+	std::string testFunction = Whiskers(R"(
 	function test() public returns (uint) {
 		<?calldata>return test_calldata_coding();</calldata>
 		<?returndata>return test_returndata_coding();</returndata>
@@ -507,8 +506,8 @@ string ProtoConverter::visit(TestFunction const& _x, string const& _storageVarDe
 	("returndata", m_test == Contract_Test::Contract_Test_RETURNDATA_CODER)
 	.render();
 
-	string functionDeclCalldata = "function test_calldata_coding() internal returns (uint)";
-	string functionDeclReturndata = "function test_returndata_coding() internal returns (uint)";
+	std::string functionDeclCalldata = "function test_calldata_coding() internal returns (uint)";
+	std::string functionDeclReturndata = "function test_returndata_coding() internal returns (uint)";
 
 	testBuffer << Whiskers(R"(<structTypeDecl>
 <testFunction>
@@ -551,7 +550,7 @@ string ProtoConverter::visit(TestFunction const& _x, string const& _storageVarDe
 	return testBuffer.str();
 }
 
-string ProtoConverter::testCallDataFunction(unsigned _invalidLength)
+std::string ProtoConverter::testCallDataFunction(unsigned _invalidLength)
 {
 	return Whiskers(R"(
 		uint returnVal = this.coder_calldata_public(<argumentNames>);
@@ -592,7 +591,7 @@ string ProtoConverter::testCallDataFunction(unsigned _invalidLength)
 		.render();
 }
 
-string ProtoConverter::testReturnDataFunction()
+std::string ProtoConverter::testReturnDataFunction()
 {
 	return Whiskers(R"(
 <?varsPresent>
@@ -607,9 +606,9 @@ string ProtoConverter::testReturnDataFunction()
 		.render();
 }
 
-string ProtoConverter::calldataHelperFunctions()
+std::string ProtoConverter::calldataHelperFunctions()
 {
-	stringstream calldataHelperFuncs;
+	std::stringstream calldataHelperFuncs;
 	calldataHelperFuncs << R"(
 	/// Accepts function selector, correct argument encoding, and length of
 	/// invalid encoding and returns the correct and incorrect abi encoding
@@ -676,12 +675,12 @@ string ProtoConverter::calldataHelperFunctions()
 	})";
 
 	/// These are indirections to test memory-calldata codings more robustly.
-	stringstream indirections;
+	std::stringstream indirections;
 	unsigned numIndirections = randomNumberOneToN(s_maxIndirections);
 	for (unsigned i = 1; i <= numIndirections; i++)
 	{
 		bool finalIndirection = i == numIndirections;
-		string mutability = (finalIndirection ? "pure" : "view");
+		std::string mutability = (finalIndirection ? "pure" : "view");
 		indirections << Whiskers(R"(
 	function coder_calldata_external_i<N>(<parameters>) external <mutability> returns (uint) {
 <?finalIndirection>
@@ -692,12 +691,12 @@ string ProtoConverter::calldataHelperFunctions()
 </finalIndirection>
 	}
 		)")
-		("N", to_string(i))
+		("N", std::to_string(i))
 		("parameters", typedParametersAsString(CalleeType::EXTERNAL))
 		("mutability", mutability)
 		("finalIndirection", finalIndirection)
 		("equality_checks", equalityChecksAsString())
-		("NPlusOne", to_string(i + 1))
+		("NPlusOne", std::to_string(i + 1))
 		("untyped_parameters", m_untypedParamsExternal.str())
 		.render();
 	}
@@ -726,9 +725,9 @@ string ProtoConverter::calldataHelperFunctions()
 	return calldataHelperFuncs.str();
 }
 
-string ProtoConverter::commonHelperFunctions()
+std::string ProtoConverter::commonHelperFunctions()
 {
-	stringstream helperFuncs;
+	std::stringstream helperFuncs;
 	helperFuncs << R"(
 	/// Compares bytes, returning true if they are equal and false otherwise.
 	function bytesCompare(bytes memory a, bytes memory b) internal pure returns (bool) {
@@ -746,7 +745,7 @@ string ProtoConverter::commonHelperFunctions()
 
 void ProtoConverter::visit(Contract const& _x)
 {
-	string pragmas = R"(pragma solidity >=0.0;
+	std::string pragmas = R"(pragma solidity >=0.0;
 pragma experimental ABIEncoderV2;)";
 
 	// Record test spec
@@ -754,10 +753,10 @@ pragma experimental ABIEncoderV2;)";
 
 	// TODO: Support more than one but less than N state variables
 	auto storageBuffers = visit(_x.state_vars());
-	string storageVarDecls = storageBuffers.first;
-	string storageVarDefs = storageBuffers.second;
+	std::string storageVarDecls = storageBuffers.first;
+	std::string storageVarDefs = storageBuffers.second;
 	m_isStateVar = false;
-	string testFunction = visit(_x.testfunction(), storageVarDefs);
+	std::string testFunction = visit(_x.testfunction(), storageVarDefs);
 	/* Structure of contract body
 	 * - Storage variable declarations
 	 * - Struct definitions
@@ -767,7 +766,7 @@ pragma experimental ABIEncoderV2;)";
 	 *     - Test code proper (calls public and external functions)
 	 * - Helper functions
 	 */
-	ostringstream contractBody;
+	std::ostringstream contractBody;
 	contractBody << storageVarDecls
 	             << testFunction
 	             << commonHelperFunctions();
@@ -782,32 +781,32 @@ pragma experimental ABIEncoderV2;)";
 		.render();
 }
 
-string ProtoConverter::isabelleTypeString() const
+std::string ProtoConverter::isabelleTypeString() const
 {
-	string typeString = m_isabelleTypeString.str();
+	std::string typeString = m_isabelleTypeString.str();
 	if (!typeString.empty())
 		return "(" + typeString + ")";
 	else
 		return typeString;
 }
 
-string ProtoConverter::isabelleValueString() const
+std::string ProtoConverter::isabelleValueString() const
 {
-	string valueString = m_isabelleValueString.str();
+	std::string valueString = m_isabelleValueString.str();
 	if (!valueString.empty())
 		return "(" + valueString + ")";
 	else
 		return valueString;
 }
 
-string ProtoConverter::contractToString(Contract const& _input)
+std::string ProtoConverter::contractToString(Contract const& _input)
 {
 	visit(_input);
 	return m_output.str();
 }
 
 /// Type visitor
-void TypeVisitor::StructTupleString::addTypeStringToTuple(string& _typeString)
+void TypeVisitor::StructTupleString::addTypeStringToTuple(std::string& _typeString)
 {
 	index++;
 	if (index > 1)
@@ -815,51 +814,51 @@ void TypeVisitor::StructTupleString::addTypeStringToTuple(string& _typeString)
 	stream << _typeString;
 }
 
-void TypeVisitor::StructTupleString::addArrayBracketToType(string& _arrayBracket)
+void TypeVisitor::StructTupleString::addArrayBracketToType(std::string& _arrayBracket)
 {
 	stream << _arrayBracket;
 }
 
-string TypeVisitor::visit(BoolType const&)
+std::string TypeVisitor::visit(BoolType const&)
 {
 	m_baseType = "bool";
 	m_structTupleString.addTypeStringToTuple(m_baseType);
 	return m_baseType;
 }
 
-string TypeVisitor::visit(IntegerType const& _type)
+std::string TypeVisitor::visit(IntegerType const& _type)
 {
 	m_baseType = getIntTypeAsString(_type);
 	m_structTupleString.addTypeStringToTuple(m_baseType);
 	return m_baseType;
 }
 
-string TypeVisitor::visit(FixedByteType const& _type)
+std::string TypeVisitor::visit(FixedByteType const& _type)
 {
 	m_baseType = getFixedByteTypeAsString(_type);
 	m_structTupleString.addTypeStringToTuple(m_baseType);
 	return m_baseType;
 }
 
-string TypeVisitor::visit(AddressType const&)
+std::string TypeVisitor::visit(AddressType const&)
 {
 	m_baseType = "address";
 	m_structTupleString.addTypeStringToTuple(m_baseType);
 	return m_baseType;
 }
 
-string TypeVisitor::visit(ArrayType const& _type)
+std::string TypeVisitor::visit(ArrayType const& _type)
 {
 	if (!ValidityVisitor().visit(_type))
 		return "";
 
-	string baseType = visit(_type.t());
+	std::string baseType = visit(_type.t());
 	solAssert(!baseType.empty(), "");
-	string arrayBracket = _type.is_static() ?
-	                     string("[") +
-	                     to_string(getStaticArrayLengthFromFuzz(_type.length())) +
-	                     string("]") :
-	                     string("[]");
+	std::string arrayBracket = _type.is_static() ?
+	                     std::string("[") +
+	                     std::to_string(getStaticArrayLengthFromFuzz(_type.length())) +
+	                     std::string("]") :
+	                     std::string("[]");
 	m_baseType += arrayBracket;
 	m_structTupleString.addArrayBracketToType(arrayBracket);
 
@@ -872,7 +871,7 @@ string TypeVisitor::visit(ArrayType const& _type)
 	return baseType + arrayBracket;
 }
 
-string TypeVisitor::visit(DynamicByteArrayType const&)
+std::string TypeVisitor::visit(DynamicByteArrayType const&)
 {
 	m_isLastDynParamRightPadded = true;
 	m_baseType = "bytes";
@@ -893,10 +892,10 @@ void TypeVisitor::structDefinition(StructType const& _type)
 	m_structFieldCounter = 0;
 
 	// Commence struct declaration
-	string structDef = lineString(
+	std::string structDef = lineString(
 		"struct " +
-		string(s_structNamePrefix) +
-		to_string(m_structCounter) +
+		std::string(s_structNamePrefix) +
+		std::to_string(m_structCounter) +
 		" {"
 	);
 	// Start tuple of types with parenthesis
@@ -905,7 +904,7 @@ void TypeVisitor::structDefinition(StructType const& _type)
 	m_indentation++;
 	for (auto const& t: _type.t())
 	{
-		string type{};
+		std::string type{};
 
 		if (!ValidityVisitor().visit(t))
 			continue;
@@ -920,10 +919,10 @@ void TypeVisitor::structDefinition(StructType const& _type)
 		structDef += lineString(
 			Whiskers(R"(<type> <member>;)")
 				("type", type)
-				("member", "m" + to_string(m_structFieldCounter++))
+				("member", "m" + std::to_string(m_structFieldCounter++))
 				.render()
 		);
-		string isabelleTypeStr = tVisitor.isabelleTypeString();
+		std::string isabelleTypeStr = tVisitor.isabelleTypeString();
 		m_structTupleString.addTypeStringToTuple(isabelleTypeStr);
 	}
 	m_indentation--;
@@ -936,7 +935,7 @@ void TypeVisitor::structDefinition(StructType const& _type)
 	m_structFieldCounter = wasFieldCounter;
 }
 
-string TypeVisitor::visit(StructType const& _type)
+std::string TypeVisitor::visit(StructType const& _type)
 {
 	if (ValidityVisitor().visit(_type))
 	{
@@ -945,7 +944,7 @@ string TypeVisitor::visit(StructType const& _type)
 		// Set last dyn param if struct contains a dyn param e.g., bytes, array etc.
 		m_isLastDynParamRightPadded = DynParamVisitor().visit(_type);
 		// If top-level struct is a non-empty struct, assign the name S<suffix>
-		m_baseType = s_structTypeName + to_string(m_structStartCounter);
+		m_baseType = s_structTypeName + std::to_string(m_structStartCounter);
 	}
 	else
 		m_baseType = {};
@@ -954,7 +953,7 @@ string TypeVisitor::visit(StructType const& _type)
 }
 
 /// AssignCheckVisitor implementation
-void AssignCheckVisitor::ValueStream::appendValue(string& _value)
+void AssignCheckVisitor::ValueStream::appendValue(std::string& _value)
 {
 	solAssert(!_value.empty(), "Abiv2 fuzzer: Empty value");
 	index++;
@@ -963,77 +962,77 @@ void AssignCheckVisitor::ValueStream::appendValue(string& _value)
 	stream << _value;
 }
 
-pair<string, string> AssignCheckVisitor::visit(BoolType const& _type)
+std::pair<std::string, std::string> AssignCheckVisitor::visit(BoolType const& _type)
 {
-	string value = ValueGetterVisitor(counter()).visit(_type);
+	std::string value = ValueGetterVisitor(counter()).visit(_type);
 	if (!m_forcedVisit)
 		m_valueStream.appendValue(value);
 	return assignAndCheckStringPair(m_varName, m_paramName, value, value, DataType::VALUE);
 }
 
-pair<string, string> AssignCheckVisitor::visit(IntegerType const& _type)
+std::pair<std::string, std::string> AssignCheckVisitor::visit(IntegerType const& _type)
 {
-	string value = ValueGetterVisitor(counter()).visit(_type);
+	std::string value = ValueGetterVisitor(counter()).visit(_type);
 	if (!m_forcedVisit)
 		m_valueStream.appendValue(value);
 	return assignAndCheckStringPair(m_varName, m_paramName, value, value, DataType::VALUE);
 }
 
-pair<string, string> AssignCheckVisitor::visit(FixedByteType const& _type)
+std::pair<std::string, std::string> AssignCheckVisitor::visit(FixedByteType const& _type)
 {
-	string value = ValueGetterVisitor(counter()).visit(_type);
+	std::string value = ValueGetterVisitor(counter()).visit(_type);
 	if (!m_forcedVisit)
 	{
-		string isabelleValue = ValueGetterVisitor{}.isabelleBytesValueAsString(value);
+		std::string isabelleValue = ValueGetterVisitor{}.isabelleBytesValueAsString(value);
 		m_valueStream.appendValue(isabelleValue);
 	}
 	return assignAndCheckStringPair(m_varName, m_paramName, value, value, DataType::VALUE);
 }
 
-pair<string, string> AssignCheckVisitor::visit(AddressType const& _type)
+std::pair<std::string, std::string> AssignCheckVisitor::visit(AddressType const& _type)
 {
-	string value = ValueGetterVisitor(counter()).visit(_type);
+	std::string value = ValueGetterVisitor(counter()).visit(_type);
 	if (!m_forcedVisit)
 	{
-		string isabelleValue = ValueGetterVisitor{}.isabelleAddressValueAsString(value);
+		std::string isabelleValue = ValueGetterVisitor{}.isabelleAddressValueAsString(value);
 		m_valueStream.appendValue(isabelleValue);
 	}
 	return assignAndCheckStringPair(m_varName, m_paramName, value, value, DataType::VALUE);
 }
 
-pair<string, string> AssignCheckVisitor::visit(DynamicByteArrayType const& _type)
+std::pair<std::string, std::string> AssignCheckVisitor::visit(DynamicByteArrayType const& _type)
 {
-	string value = ValueGetterVisitor(counter()).visit(_type);
+	std::string value = ValueGetterVisitor(counter()).visit(_type);
 	if (!m_forcedVisit)
 	{
-		string isabelleValue = ValueGetterVisitor{}.isabelleBytesValueAsString(value);
+		std::string isabelleValue = ValueGetterVisitor{}.isabelleBytesValueAsString(value);
 		m_valueStream.appendValue(isabelleValue);
 	}
 	DataType dataType = DataType::BYTES;
 	return assignAndCheckStringPair(m_varName, m_paramName, value, value, dataType);
 }
 
-pair<string, string> AssignCheckVisitor::visit(ArrayType const& _type)
+std::pair<std::string, std::string> AssignCheckVisitor::visit(ArrayType const& _type)
 {
 	if (!ValidityVisitor().visit(_type))
-		return make_pair("", "");
+		return std::make_pair("", "");
 
 	// Obtain type of array to be resized and initialized
-	string typeStr{};
+	std::string typeStr{};
 
 	unsigned wasStructCounter = m_structCounter;
 	TypeVisitor tVisitor(m_structCounter);
 	typeStr = tVisitor.visit(_type);
 
-	pair<string, string> resizeBuffer;
-	string lengthStr;
+	std::pair<std::string, std::string> resizeBuffer;
+	std::string lengthStr;
 	unsigned length;
 
 	// Resize dynamic arrays
 	if (!_type.is_static())
 	{
 		length = getDynArrayLengthFromFuzz(_type.length(), counter());
-		lengthStr = to_string(length);
+		lengthStr = std::to_string(length);
 		if (m_stateVar)
 		{
 			// Dynamic storage arrays are resized via the empty push() operation
@@ -1048,7 +1047,7 @@ pair<string, string> AssignCheckVisitor::visit(ArrayType const& _type)
 		else
 		{
 			// Resizing memory arrays via the new operator
-			string resizeOp = Whiskers(R"(new <fullTypeStr>(<length>))")
+			std::string resizeOp = Whiskers(R"(new <fullTypeStr>(<length>))")
 				("fullTypeStr", typeStr)
 				("length", lengthStr)
 				.render();
@@ -1064,22 +1063,22 @@ pair<string, string> AssignCheckVisitor::visit(ArrayType const& _type)
 	else
 	{
 		length = getStaticArrayLengthFromFuzz(_type.length());
-		lengthStr = to_string(length);
+		lengthStr = std::to_string(length);
 		// Add check on length
 		resizeBuffer.second = checkString(m_paramName + ".length", lengthStr, DataType::VALUE);
 	}
 
 	// Add assignCheckBuffer and check statements
-	pair<string, string> assignCheckBuffer;
-	string wasVarName = m_varName;
-	string wasParamName = m_paramName;
+	std::pair<std::string, std::string> assignCheckBuffer;
+	std::string wasVarName = m_varName;
+	std::string wasParamName = m_paramName;
 	if (!m_forcedVisit)
 		m_valueStream.startArray();
 	for (unsigned i = 0; i < length; i++)
 	{
-		m_varName = wasVarName + "[" + to_string(i) + "]";
-		m_paramName = wasParamName + "[" + to_string(i) + "]";
-		pair<string, string> assign = visit(_type.t());
+		m_varName = wasVarName + "[" + std::to_string(i) + "]";
+		m_paramName = wasParamName + "[" + std::to_string(i) + "]";
+		std::pair<std::string, std::string> assign = visit(_type.t());
 		assignCheckBuffer.first += assign.first;
 		assignCheckBuffer.second += assign.second;
 		if (i < length - 1)
@@ -1102,33 +1101,33 @@ pair<string, string> AssignCheckVisitor::visit(ArrayType const& _type)
 	m_paramName = wasParamName;
 
 	// Compose resize and initialization assignment and check
-	return make_pair(
+	return std::make_pair(
 		resizeBuffer.first + assignCheckBuffer.first,
 		resizeBuffer.second + assignCheckBuffer.second
 	);
 }
 
-pair<string, string> AssignCheckVisitor::visit(StructType const& _type)
+std::pair<std::string, std::string> AssignCheckVisitor::visit(StructType const& _type)
 {
 	if (!ValidityVisitor().visit(_type))
-		return make_pair("", "");
+		return std::make_pair("", "");
 
-	pair<string, string> assignCheckBuffer;
+	std::pair<std::string, std::string> assignCheckBuffer;
 	unsigned i = 0;
 
 	// Increment struct counter
 	m_structCounter++;
 
-	string wasVarName = m_varName;
-	string wasParamName = m_paramName;
+	std::string wasVarName = m_varName;
+	std::string wasParamName = m_paramName;
 
 	if (!m_forcedVisit)
 		m_valueStream.startStruct();
 	for (auto const& t: _type.t())
 	{
-		m_varName = wasVarName + ".m" + to_string(i);
-		m_paramName = wasParamName + ".m" + to_string(i);
-		pair<string, string> assign = visit(t);
+		m_varName = wasVarName + ".m" + std::to_string(i);
+		m_paramName = wasParamName + ".m" + std::to_string(i);
+		std::pair<std::string, std::string> assign = visit(t);
 		// If type is not well formed continue without
 		// updating state.
 		if (assign.first.empty() && assign.second.empty())
@@ -1144,29 +1143,29 @@ pair<string, string> AssignCheckVisitor::visit(StructType const& _type)
 	return assignCheckBuffer;
 }
 
-pair<string, string> AssignCheckVisitor::assignAndCheckStringPair(
-	string const& _varRef,
-	string const& _checkRef,
-	string const& _assignValue,
-	string const& _checkValue,
+std::pair<std::string, std::string> AssignCheckVisitor::assignAndCheckStringPair(
+	std::string const& _varRef,
+	std::string const& _checkRef,
+	std::string const& _assignValue,
+	std::string const& _checkValue,
 	DataType _type
 )
 {
-	return make_pair(assignString(_varRef, _assignValue), checkString(_checkRef, _checkValue, _type));
+	return std::make_pair(assignString(_varRef, _assignValue), checkString(_checkRef, _checkValue, _type));
 }
 
-string AssignCheckVisitor::assignString(string const& _ref, string const& _value)
+std::string AssignCheckVisitor::assignString(std::string const& _ref, std::string const& _value)
 {
-	string assignStmt = Whiskers(R"(<ref> = <value>;)")
+	std::string assignStmt = Whiskers(R"(<ref> = <value>;)")
 		("ref", _ref)
 		("value", _value)
 		.render();
 	return indentation() + assignStmt + "\n";
 }
 
-string AssignCheckVisitor::checkString(string const& _ref, string const& _value, DataType _type)
+std::string AssignCheckVisitor::checkString(std::string const& _ref, std::string const& _value, DataType _type)
 {
-	string checkPred;
+	std::string checkPred;
 	switch (_type)
 	{
 	case DataType::BYTES:
@@ -1184,25 +1183,25 @@ string AssignCheckVisitor::checkString(string const& _ref, string const& _value,
 	case DataType::ARRAY:
 		solUnimplemented("Proto ABIv2 fuzzer: Invalid data type.");
 	}
-	string checkStmt = Whiskers(R"(if (<checkPred>) return <errCode>;)")
+	std::string checkStmt = Whiskers(R"(if (<checkPred>) return <errCode>;)")
 		("checkPred", checkPred)
-		("errCode", to_string(m_errorCode++))
+		("errCode", std::to_string(m_errorCode++))
 		.render();
 	return indentation() + checkStmt + "\n";
 }
 
 /// ValueGetterVisitor
-string ValueGetterVisitor::visit(BoolType const&)
+std::string ValueGetterVisitor::visit(BoolType const&)
 {
 	return counter() % 2 ? "true" : "false";
 }
 
-string ValueGetterVisitor::visit(IntegerType const& _type)
+std::string ValueGetterVisitor::visit(IntegerType const& _type)
 {
 	return integerValue(counter(), getIntWidth(_type), _type.is_signed());
 }
 
-string ValueGetterVisitor::visit(FixedByteType const& _type)
+std::string ValueGetterVisitor::visit(FixedByteType const& _type)
 {
 	return fixedByteValueAsString(
 		getFixedByteWidth(_type),
@@ -1210,12 +1209,12 @@ string ValueGetterVisitor::visit(FixedByteType const& _type)
 	);
 }
 
-string ValueGetterVisitor::visit(AddressType const&)
+std::string ValueGetterVisitor::visit(AddressType const&)
 {
 	return addressValueAsString(counter());
 }
 
-string ValueGetterVisitor::visit(DynamicByteArrayType const&)
+std::string ValueGetterVisitor::visit(DynamicByteArrayType const&)
 {
 	return bytesArrayValueAsString(
 		counter(),
@@ -1331,7 +1330,7 @@ std::string ValueGetterVisitor::variableLengthValueAsString(
 
 	unsigned numBytesRemaining = _numBytes;
 	// Stores the literal
-	string output{};
+	std::string output{};
 	// If requested value is shorter than or exactly 32 bytes,
 	// the literal is the return value of hexValueAsString.
 	if (numBytesRemaining <= 32)
@@ -1350,7 +1349,7 @@ std::string ValueGetterVisitor::variableLengthValueAsString(
 		// update number of bytes to be appended.
 		// Stores the cached literal that saves us
 		// (expensive) calls to keccak256.
-		string cachedString = hexValueAsString(
+		std::string cachedString = hexValueAsString(
 			/*numBytes=*/32,
 			             _counter,
 			             _isHexLiteral,
@@ -1398,7 +1397,7 @@ std::string ValueGetterVisitor::variableLengthValueAsString(
 		.render();
 }
 
-string ValueGetterVisitor::bytesArrayValueAsString(unsigned _counter, bool _isHexLiteral)
+std::string ValueGetterVisitor::bytesArrayValueAsString(unsigned _counter, bool _isHexLiteral)
 {
 	return variableLengthValueAsString(
 		getVarLength(_counter),
