@@ -37,7 +37,6 @@
 
 #include <iostream>
 
-using namespace std;
 using namespace solidity;
 using namespace solidity::langutil;
 using namespace solidity::util;
@@ -49,58 +48,58 @@ namespace po = boost::program_options;
 namespace
 {
 
-map<PhaserMode, string> const PhaserModeToStringMap =
+std::map<PhaserMode, std::string> const PhaserModeToStringMap =
 {
 	{PhaserMode::RunAlgorithm, "run-algorithm"},
 	{PhaserMode::PrintOptimisedPrograms, "print-optimised-programs"},
 	{PhaserMode::PrintOptimisedASTs, "print-optimised-asts"},
 };
-map<string, PhaserMode> const StringToPhaserModeMap = invertMap(PhaserModeToStringMap);
+std::map<std::string, PhaserMode> const StringToPhaserModeMap = invertMap(PhaserModeToStringMap);
 
-map<Algorithm, string> const AlgorithmToStringMap =
+std::map<Algorithm, std::string> const AlgorithmToStringMap =
 {
 	{Algorithm::Random, "random"},
 	{Algorithm::GEWEP, "GEWEP"},
 	{Algorithm::Classic, "classic"},
 };
-map<string, Algorithm> const StringToAlgorithmMap = invertMap(AlgorithmToStringMap);
+std::map<std::string, Algorithm> const StringToAlgorithmMap = invertMap(AlgorithmToStringMap);
 
-map<MetricChoice, string> MetricChoiceToStringMap =
+std::map<MetricChoice, std::string> MetricChoiceToStringMap =
 {
 	{MetricChoice::CodeSize, "code-size"},
 	{MetricChoice::RelativeCodeSize, "relative-code-size"},
 };
-map<string, MetricChoice> const StringToMetricChoiceMap = invertMap(MetricChoiceToStringMap);
+std::map<std::string, MetricChoice> const StringToMetricChoiceMap = invertMap(MetricChoiceToStringMap);
 
-map<MetricAggregatorChoice, string> const MetricAggregatorChoiceToStringMap =
+std::map<MetricAggregatorChoice, std::string> const MetricAggregatorChoiceToStringMap =
 {
 	{MetricAggregatorChoice::Average, "average"},
 	{MetricAggregatorChoice::Sum, "sum"},
 	{MetricAggregatorChoice::Maximum, "maximum"},
 	{MetricAggregatorChoice::Minimum, "minimum"},
 };
-map<string, MetricAggregatorChoice> const StringToMetricAggregatorChoiceMap = invertMap(MetricAggregatorChoiceToStringMap);
+std::map<std::string, MetricAggregatorChoice> const StringToMetricAggregatorChoiceMap = invertMap(MetricAggregatorChoiceToStringMap);
 
-map<CrossoverChoice, string> const CrossoverChoiceToStringMap =
+std::map<CrossoverChoice, std::string> const CrossoverChoiceToStringMap =
 {
 	{CrossoverChoice::SinglePoint, "single-point"},
 	{CrossoverChoice::TwoPoint, "two-point"},
 	{CrossoverChoice::Uniform, "uniform"},
 };
-map<string, CrossoverChoice> const StringToCrossoverChoiceMap = invertMap(CrossoverChoiceToStringMap);
+std::map<std::string, CrossoverChoice> const StringToCrossoverChoiceMap = invertMap(CrossoverChoiceToStringMap);
 
 }
 
-istream& phaser::operator>>(istream& _inputStream, PhaserMode& _phaserMode) { return deserializeChoice(_inputStream, _phaserMode, StringToPhaserModeMap); }
-ostream& phaser::operator<<(ostream& _outputStream, PhaserMode _phaserMode) { return serializeChoice(_outputStream, _phaserMode, PhaserModeToStringMap); }
-istream& phaser::operator>>(istream& _inputStream, Algorithm& _algorithm) { return deserializeChoice(_inputStream, _algorithm, StringToAlgorithmMap); }
-ostream& phaser::operator<<(ostream& _outputStream, Algorithm _algorithm) { return serializeChoice(_outputStream, _algorithm, AlgorithmToStringMap); }
-istream& phaser::operator>>(istream& _inputStream, MetricChoice& _metric) { return deserializeChoice(_inputStream, _metric, StringToMetricChoiceMap); }
-ostream& phaser::operator<<(ostream& _outputStream, MetricChoice _metric) { return serializeChoice(_outputStream, _metric, MetricChoiceToStringMap); }
-istream& phaser::operator>>(istream& _inputStream, MetricAggregatorChoice& _aggregator) { return deserializeChoice(_inputStream, _aggregator, StringToMetricAggregatorChoiceMap); }
-ostream& phaser::operator<<(ostream& _outputStream, MetricAggregatorChoice _aggregator) { return serializeChoice(_outputStream, _aggregator, MetricAggregatorChoiceToStringMap); }
-istream& phaser::operator>>(istream& _inputStream, CrossoverChoice& _crossover) { return deserializeChoice(_inputStream, _crossover, StringToCrossoverChoiceMap); }
-ostream& phaser::operator<<(ostream& _outputStream, CrossoverChoice _crossover) { return serializeChoice(_outputStream, _crossover, CrossoverChoiceToStringMap); }
+std::istream& phaser::operator>>(std::istream& _inputStream, PhaserMode& _phaserMode) { return deserializeChoice(_inputStream, _phaserMode, StringToPhaserModeMap); }
+std::ostream& phaser::operator<<(std::ostream& _outputStream, PhaserMode _phaserMode) { return serializeChoice(_outputStream, _phaserMode, PhaserModeToStringMap); }
+std::istream& phaser::operator>>(std::istream& _inputStream, Algorithm& _algorithm) { return deserializeChoice(_inputStream, _algorithm, StringToAlgorithmMap); }
+std::ostream& phaser::operator<<(std::ostream& _outputStream, Algorithm _algorithm) { return serializeChoice(_outputStream, _algorithm, AlgorithmToStringMap); }
+std::istream& phaser::operator>>(std::istream& _inputStream, MetricChoice& _metric) { return deserializeChoice(_inputStream, _metric, StringToMetricChoiceMap); }
+std::ostream& phaser::operator<<(std::ostream& _outputStream, MetricChoice _metric) { return serializeChoice(_outputStream, _metric, MetricChoiceToStringMap); }
+std::istream& phaser::operator>>(std::istream& _inputStream, MetricAggregatorChoice& _aggregator) { return deserializeChoice(_inputStream, _aggregator, StringToMetricAggregatorChoiceMap); }
+std::ostream& phaser::operator<<(std::ostream& _outputStream, MetricAggregatorChoice _aggregator) { return serializeChoice(_outputStream, _aggregator, MetricAggregatorChoiceToStringMap); }
+std::istream& phaser::operator>>(std::istream& _inputStream, CrossoverChoice& _crossover) { return deserializeChoice(_inputStream, _crossover, StringToCrossoverChoiceMap); }
+std::ostream& phaser::operator<<(std::ostream& _outputStream, CrossoverChoice _crossover) { return serializeChoice(_outputStream, _crossover, CrossoverChoiceToStringMap); }
 
 GeneticAlgorithmFactory::Options GeneticAlgorithmFactory::Options::fromCommandLine(po::variables_map const& _arguments)
 {
@@ -112,17 +111,17 @@ GeneticAlgorithmFactory::Options GeneticAlgorithmFactory::Options::fromCommandLi
 		_arguments["uniform-crossover-swap-chance"].as<double>(),
 		_arguments.count("random-elite-pool-size") > 0 ?
 			_arguments["random-elite-pool-size"].as<double>() :
-			optional<double>{},
+			std::optional<double>{},
 		_arguments["gewep-mutation-pool-size"].as<double>(),
 		_arguments["gewep-crossover-pool-size"].as<double>(),
 		_arguments["gewep-randomisation-chance"].as<double>(),
 		_arguments["gewep-deletion-vs-addition-chance"].as<double>(),
 		_arguments.count("gewep-genes-to-randomise") > 0 ?
 			_arguments["gewep-genes-to-randomise"].as<double>() :
-			optional<double>{},
+			std::optional<double>{},
 		_arguments.count("gewep-genes-to-add-or-delete") > 0 ?
 			_arguments["gewep-genes-to-add-or-delete"].as<double>() :
-			optional<double>{},
+			std::optional<double>{},
 		_arguments["classic-elite-pool-size"].as<double>(),
 		_arguments["classic-crossover-chance"].as<double>(),
 		_arguments["classic-mutation-chance"].as<double>(),
@@ -131,7 +130,7 @@ GeneticAlgorithmFactory::Options GeneticAlgorithmFactory::Options::fromCommandLi
 	};
 }
 
-unique_ptr<GeneticAlgorithm> GeneticAlgorithmFactory::build(
+std::unique_ptr<GeneticAlgorithm> GeneticAlgorithmFactory::build(
 	Options const& _options,
 	size_t _populationSize
 )
@@ -147,7 +146,7 @@ unique_ptr<GeneticAlgorithm> GeneticAlgorithmFactory::build(
 			if (_options.randomElitePoolSize.has_value())
 				elitePoolSize = _options.randomElitePoolSize.value();
 
-			return make_unique<RandomAlgorithm>(RandomAlgorithm::Options{
+			return std::make_unique<RandomAlgorithm>(RandomAlgorithm::Options{
 				/* elitePoolSize = */ elitePoolSize,
 				/* minChromosomeLength = */ _options.minChromosomeLength,
 				/* maxChromosomeLength = */ _options.maxChromosomeLength,
@@ -163,7 +162,7 @@ unique_ptr<GeneticAlgorithm> GeneticAlgorithmFactory::build(
 			if (_options.gewepGenesToAddOrDelete.has_value())
 				percentGenesToAddOrDelete = _options.gewepGenesToAddOrDelete.value();
 
-			return make_unique<GenerationalElitistWithExclusivePools>(GenerationalElitistWithExclusivePools::Options{
+			return std::make_unique<GenerationalElitistWithExclusivePools>(GenerationalElitistWithExclusivePools::Options{
 				/* mutationPoolSize = */ _options.gewepMutationPoolSize,
 				/* crossoverPoolSize = */ _options.gewepCrossoverPoolSize,
 				/* randomisationChance = */ _options.gewepRandomisationChance,
@@ -176,7 +175,7 @@ unique_ptr<GeneticAlgorithm> GeneticAlgorithmFactory::build(
 		}
 		case Algorithm::Classic:
 		{
-			return make_unique<ClassicGeneticAlgorithm>(ClassicGeneticAlgorithm::Options{
+			return std::make_unique<ClassicGeneticAlgorithm>(ClassicGeneticAlgorithm::Options{
 				/* elitePoolSize = */ _options.classicElitePoolSize,
 				/* crossoverChance = */ _options.classicCrossoverChance,
 				/* mutationChance = */ _options.classicMutationChance,
@@ -222,24 +221,24 @@ FitnessMetricFactory::Options FitnessMetricFactory::Options::fromCommandLine(po:
 	};
 }
 
-unique_ptr<FitnessMetric> FitnessMetricFactory::build(
+std::unique_ptr<FitnessMetric> FitnessMetricFactory::build(
 	Options const& _options,
-	vector<Program> _programs,
-	vector<shared_ptr<ProgramCache>> _programCaches,
+	std::vector<Program> _programs,
+	std::vector<std::shared_ptr<ProgramCache>> _programCaches,
 	CodeWeights const& _weights
 )
 {
 	assert(_programCaches.size() == _programs.size());
 	assert(_programs.size() > 0 && "Validations should prevent this from being executed with zero files.");
 
-	vector<shared_ptr<FitnessMetric>> metrics;
+	std::vector<std::shared_ptr<FitnessMetric>> metrics;
 	switch (_options.metric)
 	{
 		case MetricChoice::CodeSize:
 		{
 			for (size_t i = 0; i < _programs.size(); ++i)
-				metrics.push_back(make_unique<ProgramSize>(
-					_programCaches[i] != nullptr ? optional<Program>{} : std::move(_programs[i]),
+				metrics.push_back(std::make_unique<ProgramSize>(
+					_programCaches[i] != nullptr ? std::optional<Program>{} : std::move(_programs[i]),
 					std::move(_programCaches[i]),
 					_weights,
 					_options.chromosomeRepetitions
@@ -250,8 +249,8 @@ unique_ptr<FitnessMetric> FitnessMetricFactory::build(
 		case MetricChoice::RelativeCodeSize:
 		{
 			for (size_t i = 0; i < _programs.size(); ++i)
-				metrics.push_back(make_unique<RelativeProgramSize>(
-					_programCaches[i] != nullptr ? optional<Program>{} : std::move(_programs[i]),
+				metrics.push_back(std::make_unique<RelativeProgramSize>(
+					_programCaches[i] != nullptr ? std::optional<Program>{} : std::move(_programs[i]),
 					std::move(_programCaches[i]),
 					_options.relativeMetricScale,
 					_weights,
@@ -266,13 +265,13 @@ unique_ptr<FitnessMetric> FitnessMetricFactory::build(
 	switch (_options.metricAggregator)
 	{
 		case MetricAggregatorChoice::Average:
-			return make_unique<FitnessMetricAverage>(std::move(metrics));
+			return std::make_unique<FitnessMetricAverage>(std::move(metrics));
 		case MetricAggregatorChoice::Sum:
-			return make_unique<FitnessMetricSum>(std::move(metrics));
+			return std::make_unique<FitnessMetricSum>(std::move(metrics));
 		case MetricAggregatorChoice::Maximum:
-			return make_unique<FitnessMetricMaximum>(std::move(metrics));
+			return std::make_unique<FitnessMetricMaximum>(std::move(metrics));
 		case MetricAggregatorChoice::Minimum:
-			return make_unique<FitnessMetricMinimum>(std::move(metrics));
+			return std::make_unique<FitnessMetricMinimum>(std::move(metrics));
 		default:
 			assertThrow(false, solidity::util::Exception, "Invalid MetricAggregatorChoice value.");
 	}
@@ -287,20 +286,20 @@ PopulationFactory::Options PopulationFactory::Options::fromCommandLine(po::varia
 		_arguments["min-chromosome-length"].as<size_t>(),
 		_arguments["max-chromosome-length"].as<size_t>(),
 		_arguments.count("population") > 0 ?
-			_arguments["population"].as<vector<string>>() :
-			vector<string>{},
+			_arguments["population"].as<std::vector<std::string>>() :
+			std::vector<std::string>{},
 		_arguments.count("random-population") > 0 ?
-			_arguments["random-population"].as<vector<size_t>>() :
-			vector<size_t>{},
+			_arguments["random-population"].as<std::vector<size_t>>() :
+			std::vector<size_t>{},
 		_arguments.count("population-from-file") > 0 ?
-			_arguments["population-from-file"].as<vector<string>>() :
-			vector<string>{},
+			_arguments["population-from-file"].as<std::vector<std::string>>() :
+			std::vector<std::string>{},
 	};
 }
 
 Population PopulationFactory::build(
 	Options const& _options,
-	shared_ptr<FitnessMetric> _fitnessMetric
+	std::shared_ptr<FitnessMetric> _fitnessMetric
 )
 {
 	Population population = buildFromStrings(_options.population, _fitnessMetric);
@@ -316,19 +315,19 @@ Population PopulationFactory::build(
 		_fitnessMetric
 	);
 
-	for (string const& populationFilePath: _options.populationFromFile)
+	for (std::string const& populationFilePath: _options.populationFromFile)
 		population = std::move(population) + buildFromFile(populationFilePath, _fitnessMetric);
 
 	return population;
 }
 
 Population PopulationFactory::buildFromStrings(
-	vector<string> const& _geneSequences,
-	shared_ptr<FitnessMetric> _fitnessMetric
+	std::vector<std::string> const& _geneSequences,
+	std::shared_ptr<FitnessMetric> _fitnessMetric
 )
 {
-	vector<Chromosome> chromosomes;
-	for (string const& geneSequence: _geneSequences)
+	std::vector<Chromosome> chromosomes;
+	for (std::string const& geneSequence: _geneSequences)
 		chromosomes.emplace_back(geneSequence);
 
 	return Population(std::move(_fitnessMetric), std::move(chromosomes));
@@ -338,7 +337,7 @@ Population PopulationFactory::buildRandom(
 	size_t _populationSize,
 	size_t _minChromosomeLength,
 	size_t _maxChromosomeLength,
-	shared_ptr<FitnessMetric> _fitnessMetric
+	std::shared_ptr<FitnessMetric> _fitnessMetric
 )
 {
 	return Population::makeRandom(
@@ -350,8 +349,8 @@ Population PopulationFactory::buildRandom(
 }
 
 Population PopulationFactory::buildFromFile(
-	string const& _filePath,
-	shared_ptr<FitnessMetric> _fitnessMetric
+	std::string const& _filePath,
+	std::shared_ptr<FitnessMetric> _fitnessMetric
 )
 {
 	return buildFromStrings(readLinesFromFile(_filePath), std::move(_fitnessMetric));
@@ -364,14 +363,14 @@ ProgramCacheFactory::Options ProgramCacheFactory::Options::fromCommandLine(po::v
 	};
 }
 
-vector<shared_ptr<ProgramCache>> ProgramCacheFactory::build(
+std::vector<std::shared_ptr<ProgramCache>> ProgramCacheFactory::build(
 	Options const& _options,
-	vector<Program> _programs
+	std::vector<Program> _programs
 )
 {
-	vector<shared_ptr<ProgramCache>> programCaches;
+	std::vector<std::shared_ptr<ProgramCache>> programCaches;
 	for (Program& program: _programs)
-		programCaches.push_back(_options.programCacheEnabled ? make_shared<ProgramCache>(std::move(program)) : nullptr);
+		programCaches.push_back(_options.programCacheEnabled ? std::make_shared<ProgramCache>(std::move(program)) : nullptr);
 
 	return programCaches;
 }
@@ -379,28 +378,28 @@ vector<shared_ptr<ProgramCache>> ProgramCacheFactory::build(
 ProgramFactory::Options ProgramFactory::Options::fromCommandLine(po::variables_map const& _arguments)
 {
 	return {
-		_arguments["input-files"].as<vector<string>>(),
-		_arguments["prefix"].as<string>(),
+		_arguments["input-files"].as<std::vector<std::string>>(),
+		_arguments["prefix"].as<std::string>(),
 	};
 }
 
-vector<Program> ProgramFactory::build(Options const& _options)
+std::vector<Program> ProgramFactory::build(Options const& _options)
 {
-	vector<Program> inputPrograms;
+	std::vector<Program> inputPrograms;
 	for (auto& path: _options.inputFiles)
 	{
 		CharStream sourceCode = loadSource(path);
-		variant<Program, ErrorList> programOrErrors = Program::load(sourceCode);
-		if (holds_alternative<ErrorList>(programOrErrors))
+		std::variant<Program, ErrorList> programOrErrors = Program::load(sourceCode);
+		if (std::holds_alternative<ErrorList>(programOrErrors))
 		{
-			SourceReferenceFormatter{cerr, SingletonCharStreamProvider(sourceCode), true, false}
-				.printErrorInformation(get<ErrorList>(programOrErrors));
-			cerr << endl;
+			SourceReferenceFormatter{std::cerr, SingletonCharStreamProvider(sourceCode), true, false}
+				.printErrorInformation(std::get<ErrorList>(programOrErrors));
+			std::cerr << std::endl;
 			assertThrow(false, InvalidProgram, "Failed to load program " + path);
 		}
 
-		get<Program>(programOrErrors).optimise(Chromosome(_options.prefix).optimisationSteps());
-		inputPrograms.push_back(std::move(get<Program>(programOrErrors)));
+		std::get<Program>(programOrErrors).optimise(Chromosome(_options.prefix).optimisationSteps());
+		inputPrograms.push_back(std::move(std::get<Program>(programOrErrors)));
 	}
 
 	return inputPrograms;
@@ -410,13 +409,13 @@ CharStream ProgramFactory::loadSource(boost::filesystem::path const& _sourcePath
 {
 	assertThrow(boost::filesystem::exists(_sourcePath), MissingFile, "Source file does not exist: " + _sourcePath.string());
 
-	string sourceCode = readFileAsString(_sourcePath);
+	std::string sourceCode = readFileAsString(_sourcePath);
 	return CharStream(sourceCode, _sourcePath.string());
 }
 
 void Phaser::main(int _argc, char** _argv)
 {
-	optional<po::variables_map> arguments = parseCommandLine(_argc, _argv);
+	std::optional<po::variables_map> arguments = parseCommandLine(_argc, _argv);
 	if (!arguments.has_value())
 		return;
 
@@ -447,10 +446,10 @@ Phaser::CommandLineDescription Phaser::buildCommandLineDescription()
 	po::options_description generalDescription("GENERAL", lineLength, minDescriptionLength);
 	generalDescription.add_options()
 		("help", "Show help message and exit.")
-		("input-files", po::value<vector<string>>()->required()->value_name("<PATH>"), "Input files.")
+		("input-files", po::value<std::vector<std::string>>()->required()->value_name("<PATH>"), "Input files.")
 		(
 			"prefix",
-			po::value<string>()->value_name("<CHROMOSOME>")->default_value(""),
+			po::value<std::string>()->value_name("<CHROMOSOME>")->default_value(""),
 			"Initial optimisation steps automatically applied to every input program.\n"
 			"The result is treated as if it was the actual input, i.e. the steps are not considered "
 			"a part of the chromosomes and cannot be mutated. The values of relative metric values "
@@ -615,24 +614,24 @@ Phaser::CommandLineDescription Phaser::buildCommandLineDescription()
 	populationDescription.add_options()
 		(
 			"population",
-			po::value<vector<string>>()->multitoken()->value_name("<CHROMOSOMES>"),
+			po::value<std::vector<std::string>>()->multitoken()->value_name("<CHROMOSOMES>"),
 			"List of chromosomes to be included in the initial population. "
 			"You can specify multiple values separated with spaces or invoke the option multiple times "
 			"and all the values will be included."
 		)
 		(
 			"random-population",
-			po::value<vector<size_t>>()->value_name("<SIZE>"),
+			po::value<std::vector<size_t>>()->value_name("<SIZE>"),
 			"The number of randomly generated chromosomes to be included in the initial population."
 		)
 		(
 			"population-from-file",
-			po::value<vector<string>>()->value_name("<FILE>"),
+			po::value<std::vector<std::string>>()->value_name("<FILE>"),
 			"A text file with a list of chromosomes (one per line) to be included in the initial population."
 		)
 		(
 			"population-autosave",
-			po::value<string>()->value_name("<FILE>"),
+			po::value<std::string>()->value_name("<FILE>"),
 			"If specified, the population is saved in the specified file after each round. (default=autosave disabled)"
 		)
 	;
@@ -756,7 +755,7 @@ Phaser::CommandLineDescription Phaser::buildCommandLineDescription()
 	return {keywordDescription, positionalDescription};
 }
 
-optional<po::variables_map> Phaser::parseCommandLine(int _argc, char** _argv)
+std::optional<po::variables_map> Phaser::parseCommandLine(int _argc, char** _argv)
 {
 	auto [keywordDescription, positionalDescription] = buildCommandLineDescription();
 
@@ -769,8 +768,8 @@ optional<po::variables_map> Phaser::parseCommandLine(int _argc, char** _argv)
 
 	if (arguments.count("help") > 0)
 	{
-		cout << keywordDescription << endl;
-		return nullopt;
+		std::cout << keywordDescription << std::endl;
+		return std::nullopt;
 	}
 
 	if (arguments.count("input-files") == 0)
@@ -789,14 +788,14 @@ void Phaser::initialiseRNG(po::variables_map const& _arguments)
 
 	SimulationRNG::reset(seed);
 	if (_arguments["show-seed"].as<bool>())
-		cout << "Random seed: " << seed << endl;
+		std::cout << "Random seed: " << seed << std::endl;
 }
 
 AlgorithmRunner::Options Phaser::buildAlgorithmRunnerOptions(po::variables_map const& _arguments)
 {
 	return {
-		_arguments.count("rounds") > 0 ? static_cast<optional<size_t>>(_arguments["rounds"].as<size_t>()) : nullopt,
-		_arguments.count("population-autosave") > 0 ? static_cast<optional<string>>(_arguments["population-autosave"].as<string>()) : nullopt,
+		_arguments.count("rounds") > 0 ? static_cast<std::optional<size_t>>(_arguments["rounds"].as<size_t>()) : std::nullopt,
+		_arguments.count("population-autosave") > 0 ? static_cast<std::optional<std::string>>(_arguments["population-autosave"].as<std::string>()) : std::nullopt,
 		!_arguments["no-randomise-duplicates"].as<bool>(),
 		_arguments["min-chromosome-length"].as<size_t>(),
 		_arguments["max-chromosome-length"].as<size_t>(),
@@ -814,10 +813,10 @@ void Phaser::runPhaser(po::variables_map const& _arguments)
 	auto metricOptions = FitnessMetricFactory::Options::fromCommandLine(_arguments);
 	auto populationOptions = PopulationFactory::Options::fromCommandLine(_arguments);
 
-	vector<Program> programs = ProgramFactory::build(programOptions);
-	vector<shared_ptr<ProgramCache>> programCaches = ProgramCacheFactory::build(cacheOptions, programs);
+	std::vector<Program> programs = ProgramFactory::build(programOptions);
+	std::vector<std::shared_ptr<ProgramCache>> programCaches = ProgramCacheFactory::build(cacheOptions, programs);
 	CodeWeights codeWeights = CodeWeightFactory::buildFromCommandLine(_arguments);
-	unique_ptr<FitnessMetric> fitnessMetric = FitnessMetricFactory::build(
+	std::unique_ptr<FitnessMetric> fitnessMetric = FitnessMetricFactory::build(
 		metricOptions,
 		programs,
 		programCaches,
@@ -834,51 +833,51 @@ void Phaser::runPhaser(po::variables_map const& _arguments)
 void Phaser::runAlgorithm(
 	po::variables_map const& _arguments,
 	Population _population,
-	vector<shared_ptr<ProgramCache>> _programCaches
+	std::vector<std::shared_ptr<ProgramCache>> _programCaches
 )
 {
 	auto algorithmOptions = GeneticAlgorithmFactory::Options::fromCommandLine(_arguments);
 
-	unique_ptr<GeneticAlgorithm> geneticAlgorithm = GeneticAlgorithmFactory::build(
+	std::unique_ptr<GeneticAlgorithm> geneticAlgorithm = GeneticAlgorithmFactory::build(
 		algorithmOptions,
 		_population.individuals().size()
 	);
 
-	AlgorithmRunner algorithmRunner(std::move(_population), std::move(_programCaches), buildAlgorithmRunnerOptions(_arguments), cout);
+	AlgorithmRunner algorithmRunner(std::move(_population), std::move(_programCaches), buildAlgorithmRunnerOptions(_arguments), std::cout);
 	algorithmRunner.run(*geneticAlgorithm);
 }
 
 void Phaser::printOptimisedProgramsOrASTs(
 	po::variables_map const& _arguments,
 	Population const& _population,
-	vector<Program> _programs,
+	std::vector<Program> _programs,
 	PhaserMode phaserMode
 )
 {
 	assert(phaserMode == PhaserMode::PrintOptimisedPrograms || phaserMode == PhaserMode::PrintOptimisedASTs);
-	assert(_programs.size() == _arguments["input-files"].as<vector<string>>().size());
+	assert(_programs.size() == _arguments["input-files"].as<std::vector<std::string>>().size());
 
 	if (_population.individuals().size() == 0)
 	{
-		cout << "<EMPTY POPULATION>" << endl;
+		std::cout << "<EMPTY POPULATION>" << std::endl;
 		return;
 	}
 
-	vector<string> const& paths = _arguments["input-files"].as<vector<string>>();
+	std::vector<std::string> const& paths = _arguments["input-files"].as<std::vector<std::string>>();
 	for (auto& individual: _population.individuals())
 	{
-		cout << "Chromosome: " << individual.chromosome << endl;
+		std::cout << "Chromosome: " << individual.chromosome << std::endl;
 
 		for (size_t i = 0; i < _programs.size(); ++i)
 		{
 			for (size_t j = 0; j < _arguments["chromosome-repetitions"].as<size_t>(); ++j)
 				_programs[i].optimise(individual.chromosome.optimisationSteps());
 
-			cout << "Program: " << paths[i] << endl;
+			std::cout << "Program: " << paths[i] << std::endl;
 			if (phaserMode == PhaserMode::PrintOptimisedPrograms)
-				cout << _programs[i] << endl;
+				std::cout << _programs[i] << std::endl;
 			else
-				cout << _programs[i].toJson() << endl;
+				std::cout << _programs[i].toJson() << std::endl;
 		}
 	}
 }
